@@ -149,7 +149,9 @@ var (
 	SnapResourceRevisionType = &AssertionType{"snap-resource-revision", []string{"snap-id", "resource-name", "resource-sha3-384", "provenance"}, map[string]string{"provenance": naming.DefaultProvenance}, assembleSnapResourceRevision, 0}
 	SnapResourcePairType     = &AssertionType{"snap-resource-pair", []string{"snap-id", "resource-name", "resource-revision", "snap-revision", "provenance"}, map[string]string{"provenance": naming.DefaultProvenance}, assembleSnapResourcePair, 0}
 	ConfdbSchemaType         = &AssertionType{"confdb-schema", []string{"account-id", "name"}, nil, assembleConfdbSchema, jsonBody}
-
+	ClusterType              = &AssertionType{"cluster", []string{"cluster-id", "sequence"}, nil, assembleCluster, sequenceForming}
+	RequestMessageType       = &AssertionType{"request-message", []string{"account-id", "message-id"}, nil, assembleRequestMessage, 0}
+	HardwareIdentityType     = &AssertionType{"hardware-identity", []string{"issuer-id", "hardware-id-key-sha3-384"}, nil, assembleHardwareIdentity, 0}
 	// ...
 )
 
@@ -159,6 +161,7 @@ var (
 	SerialRequestType        = &AssertionType{"serial-request", nil, nil, assembleSerialRequest, noAuthority}
 	AccountKeyRequestType    = &AssertionType{"account-key-request", []string{"public-key-sha3-384"}, nil, assembleAccountKeyRequest, noAuthority}
 	ConfdbControlType        = &AssertionType{"confdb-control", []string{"brand-id", "model", "serial"}, nil, assembleConfdbControl, noAuthority}
+	ResponseMessageType      = &AssertionType{"response-message", []string{"account-id", "message-id", "device"}, nil, assembleResponseMessage, noAuthority}
 )
 
 var typeRegistry = map[string]*AssertionType{
@@ -180,11 +183,15 @@ var typeRegistry = map[string]*AssertionType{
 	SnapResourceRevisionType.Name: SnapResourceRevisionType,
 	SnapResourcePairType.Name:     SnapResourcePairType,
 	ConfdbSchemaType.Name:         ConfdbSchemaType,
+	ClusterType.Name:              ClusterType,
+	RequestMessageType.Name:       RequestMessageType,
+	HardwareIdentityType.Name:     HardwareIdentityType,
 	// no authority
 	DeviceSessionRequestType.Name: DeviceSessionRequestType,
 	SerialRequestType.Name:        SerialRequestType,
 	AccountKeyRequestType.Name:    AccountKeyRequestType,
 	ConfdbControlType.Name:        ConfdbControlType,
+	ResponseMessageType.Name:      ResponseMessageType,
 }
 
 // Type returns the AssertionType with name or nil
