@@ -52,12 +52,16 @@ func (as *assertsSuite) TestTypeNames(c *C) {
 		"account-key",
 		"account-key-request",
 		"base-declaration",
+		"cluster",
 		"confdb-control",
 		"confdb-schema",
 		"device-session-request",
+		"hardware-identity",
 		"model",
 		"preseed",
 		"repair",
+		"request-message",
+		"response-message",
 		"serial",
 		"serial-request",
 		"snap-build",
@@ -1192,6 +1196,7 @@ func (as *assertsSuite) TestWithAuthority(c *C) {
 		"account",
 		"account-key",
 		"base-declaration",
+		"cluster",
 		"store",
 		"snap-declaration",
 		"snap-build",
@@ -1202,14 +1207,16 @@ func (as *assertsSuite) TestWithAuthority(c *C) {
 		"model",
 		"preseed",
 		"confdb-schema",
+		"hardware-identity",
 		"serial",
 		"system-user",
 		"validation",
 		"validation-set",
 		"repair",
+		"request-message",
 	}
-	// excluding device-session-request, serial-request, account-key-request, confdb-control
-	c.Check(withAuthority, HasLen, asserts.NumAssertionType-4)
+	// excluding device-session-request, serial-request, account-key-request, confdb-control, response-message
+	c.Check(withAuthority, HasLen, asserts.NumAssertionType-5)
 	for _, name := range withAuthority {
 		typ := asserts.Type(name)
 		_, err := asserts.AssembleAndSignInTest(typ, nil, []byte("{}"), testPrivKey1)
