@@ -31,8 +31,12 @@ import (
 
 // AttrMatchContext has contextual helpers for evaluating attribute constraints.
 type AttrMatchContext interface {
-	PlugAttr(arg string) (interface{}, error)
-	SlotAttr(arg string) (interface{}, error)
+	PlugAttr(arg string) (any, error)
+	SlotAttr(arg string) (any, error)
+	PlugPublisherID() string
+	SlotPublisherID() string
+	// This should be removed when the ContentCompatLabel feature is enabled by default.
+	CompatLabelsEnabled() bool
 }
 
 const (
@@ -53,10 +57,11 @@ func (ac *AttributeConstraints) feature(flabel string) bool {
 
 // compileAttributeConstraints checks and compiles a mapping or list
 // from the assertion format into AttributeConstraints.
-func compileAttributeConstraints(constraints interface{}) (*AttributeConstraints, error) {
+func compileAttributeConstraints(constraints any) (*AttributeConstraints, error) {
 	cc := compileContext{
 		opts: &compileAttrMatcherOptions{
-			allowedOperations: []string{"SLOT", "PLUG"},
+			allowedOperations: []string{"SLOT", "PLUG", "SLOT_COMPAT", "PLUG_COMPAT"},
+			allowedRefs:       []string{"PLUG_PUBLISHER_ID", "SLOT_PUBLISHER_ID"},
 		},
 	}
 	matcher, err := compileAttrMatcher(cc, constraints)
@@ -74,7 +79,7 @@ func (matcher fixedAttrMatcher) feature(flabel string) bool {
 	return false
 }
 
-func (matcher fixedAttrMatcher) match(apath string, v interface{}, ctx *attrMatchingContext) error {
+func (matcher fixedAttrMatcher) match(apath string, v any, ctx *attrMatchingContext) error {
 	return matcher.result
 }
 
@@ -85,7 +90,7 @@ var (
 
 // Attrer reflects part of the Attrer interface (see interfaces.Attrer).
 type Attrer interface {
-	Lookup(path string) (interface{}, bool)
+	Lookup(path string) (any, bool)
 }
 
 // Check checks whether attrs don't match the constraints.
@@ -117,7 +122,7 @@ func (ac SideArityConstraint) Any() bool {
 	return ac.N == -1
 }
 
-func compileSideArityConstraint(context *subruleContext, which string, v interface{}) (SideArityConstraint, error) {
+func compileSideArityConstraint(context *subruleContext, which string, v any) (SideArityConstraint, error) {
 	var a SideArityConstraint
 	if context.installation() || !context.allow() {
 		return a, fmt.Errorf("%s cannot specify a %s constraint, they apply only to allow-*connection", context, which)
@@ -182,6 +187,11 @@ type OnClassicConstraint struct {
 	SystemIDs []string
 }
 
+// OnCoreDesktopConstraint specifies a constraint based whether the system is core desktop.
+type OnCoreDesktopConstraint struct {
+	CoreDesktop bool
+}
+
 type nameMatcher interface {
 	match(name string, special map[string]string) error
 }
@@ -191,7 +201,7 @@ var (
 	validSpecialNameConstraint = regexp.MustCompile(`^\$[A-Z][A-Z0-9_]*$`)
 )
 
-func compileNameMatcher(whichName string, v interface{}) (nameMatcher, error) {
+func compileNameMatcher(whichName string, v any) (nameMatcher, error) {
 	s, ok := v.(string)
 	if !ok {
 		return nil, fmt.Errorf("%s constraint entry must be a regexp or special $ value", whichName)
@@ -241,8 +251,8 @@ type NameConstraints struct {
 	matchers []nameMatcher
 }
 
-func compileNameConstraints(whichName string, constraints interface{}) (*NameConstraints, error) {
-	l, ok := constraints.([]interface{})
+func compileNameConstraints(whichName string, constraints any) (*NameConstraints, error) {
+	l, ok := constraints.([]any)
 	if !ok {
 		return nil, fmt.Errorf("%s constraints must be a list of regexps and special $ values", whichName)
 	}
@@ -284,9 +294,9 @@ var (
 	}
 )
 
-func checkMapOrShortcut(v interface{}) (m map[string]interface{}, invert bool, err error) {
+func checkMapOrShortcut(v any) (m map[string]any, invert bool, err error) {
 	switch x := v.(type) {
-	case map[string]interface{}:
+	case map[string]any:
 		return x, false, nil
 	case string:
 		switch x {
@@ -304,6 +314,7 @@ type constraintsHolder interface {
 	setAttributeConstraints(field string, cstrs *AttributeConstraints)
 	setIDConstraints(field string, cstrs []string)
 	setOnClassicConstraint(onClassic *OnClassicConstraint)
+	setOnCoreDesktopConstraint(onCoreDesktop *OnCoreDesktopConstraint)
 	setDeviceScopeConstraint(deviceScope *DeviceScopeConstraint)
 }
 
@@ -385,7 +396,7 @@ func baseCompileConstraints(context *subruleContext, cDef constraintsDef, target
 			case "false":
 				c = &OnClassicConstraint{Classic: false}
 			}
-		case []interface{}:
+		case []any:
 			lst, err := checkStringListInMap(cMap, "on-classic", fmt.Sprintf("on-classic in %s", context), validDistro)
 			if err != nil {
 				return err
@@ -397,6 +408,25 @@ func baseCompileConstraints(context *subruleContext, cDef constraintsDef, target
 		}
 		target.setOnClassicConstraint(c)
 	}
+	onCoreDesktop := cMap["on-core-desktop"]
+	if onCoreDesktop == nil {
+		defaultUsed++
+	} else {
+		var c *OnCoreDesktopConstraint
+		switch x := onCoreDesktop.(type) {
+		case string:
+			switch x {
+			case "true":
+				c = &OnCoreDesktopConstraint{CoreDesktop: true}
+			case "false":
+				c = &OnCoreDesktopConstraint{CoreDesktop: false}
+			}
+		}
+		if c == nil {
+			return fmt.Errorf("on-core-desktop in %s must be 'true' or 'false'", context)
+		}
+		target.setOnCoreDesktopConstraint(c)
+	}
 	dsc, err := compileDeviceScopeConstraint(cMap, context.String())
 	if err != nil {
 		return err
@@ -408,10 +438,10 @@ func baseCompileConstraints(context *subruleContext, cDef constraintsDef, target
 	}
 	// checks whether defaults have been used for everything, which is not
 	// well-formed
-	// +1+1 accounts for defaults for missing on-classic plus missing
+	// +1+1+1 accounts for defaults for missing on-classic, on-core-desktop plus missing
 	// on-store/on-brand/on-model
-	if defaultUsed == len(nameConstraints)+len(attributeConstraints)+len(idConstraints)+len(sideArityConstraints)+1+1 {
-		return fmt.Errorf("%s must specify at least one of %s, %s, %s, %s, on-classic, on-store, on-brand, on-model", context, strings.Join(nameConstraints, ", "), strings.Join(attrConstraints, ", "), strings.Join(idConstraints, ", "), strings.Join(sideArityConstraints, ", "))
+	if defaultUsed == len(nameConstraints)+len(attributeConstraints)+len(idConstraints)+len(sideArityConstraints)+1+1+1 {
+		return fmt.Errorf("%s must specify at least one of %s, %s, %s, %s, on-classic, on-core-desktop, on-store, on-brand, on-model", context, strings.Join(nameConstraints, ", "), strings.Join(attrConstraints, ", "), strings.Join(idConstraints, ", "), strings.Join(sideArityConstraints, ", "))
 	}
 	return nil
 }
@@ -421,7 +451,7 @@ type rule interface {
 }
 
 type constraintsDef struct {
-	cMap   map[string]interface{}
+	cMap   map[string]any
 	invert bool
 }
 
@@ -467,7 +497,7 @@ func (c *subruleContext) autoConnection() bool {
 
 type subruleCompiler func(context *subruleContext, def constraintsDef) (constraintsHolder, error)
 
-func baseCompileRule(context string, rule interface{}, target rule, subrules []string, compilers map[string]subruleCompiler, defaultOutcome, invertedOutcome map[string]interface{}) error {
+func baseCompileRule(context string, rule any, target rule, subrules []string, compilers map[string]subruleCompiler, defaultOutcome, invertedOutcome map[string]any) error {
 	rMap, invert, err := checkMapOrShortcut(rule)
 	if err != nil {
 		return fmt.Errorf("%s must be a map or one of the shortcuts 'true' or 'false'", context)
@@ -482,18 +512,18 @@ func baseCompileRule(context string, rule interface{}, target rule, subrules []s
 	// compile and set subrules
 	for _, subrule := range subrules {
 		v := rMap[subrule]
-		var lst []interface{}
+		var lst []any
 		alternatives := false
 		switch x := v.(type) {
 		case nil:
 			v = defaultOutcome[subrule]
 			defaultUsed++
-		case []interface{}:
+		case []any:
 			alternatives = true
 			lst = x
 		}
 		if lst == nil { // v is map or a string, checked below
-			lst = []interface{}{v}
+			lst = []any{v}
 		}
 		compiler := compilers[subrule]
 		if compiler == nil {
@@ -627,7 +657,8 @@ type PlugInstallationConstraints struct {
 
 	PlugAttributes *AttributeConstraints
 
-	OnClassic *OnClassicConstraint
+	OnClassic     *OnClassicConstraint
+	OnCoreDesktop *OnCoreDesktopConstraint
 
 	DeviceScope *DeviceScopeConstraint
 }
@@ -675,6 +706,10 @@ func (c *PlugInstallationConstraints) setOnClassicConstraint(onClassic *OnClassi
 	c.OnClassic = onClassic
 }
 
+func (c *PlugInstallationConstraints) setOnCoreDesktopConstraint(onCoreDesktop *OnCoreDesktopConstraint) {
+	c.OnCoreDesktop = onCoreDesktop
+}
+
 func (c *PlugInstallationConstraints) setDeviceScopeConstraint(deviceScope *DeviceScopeConstraint) {
 	c.DeviceScope = deviceScope
 }
@@ -709,7 +744,8 @@ type PlugConnectionConstraints struct {
 	// PlugsPerSlot is always * (any) (for now)
 	PlugsPerSlot SideArityConstraint
 
-	OnClassic *OnClassicConstraint
+	OnClassic     *OnClassicConstraint
+	OnCoreDesktop *OnCoreDesktopConstraint
 
 	DeviceScope *DeviceScopeConstraint
 }
@@ -779,6 +815,10 @@ func (c *PlugConnectionConstraints) setOnClassicConstraint(onClassic *OnClassicC
 	c.OnClassic = onClassic
 }
 
+func (c *PlugConnectionConstraints) setOnCoreDesktopConstraint(onCoreDesktop *OnCoreDesktopConstraint) {
+	c.OnCoreDesktop = onCoreDesktop
+}
+
 func (c *PlugConnectionConstraints) setDeviceScopeConstraint(deviceScope *DeviceScopeConstraint) {
 	c.DeviceScope = deviceScope
 }
@@ -800,7 +840,7 @@ func compilePlugConnectionConstraints(context *subruleContext, cDef constraintsD
 }
 
 var (
-	defaultOutcome = map[string]interface{}{
+	defaultOutcome = map[string]any{
 		"allow-installation":    "true",
 		"allow-connection":      "true",
 		"allow-auto-connection": "true",
@@ -809,7 +849,7 @@ var (
 		"deny-auto-connection":  "false",
 	}
 
-	invertedOutcome = map[string]interface{}{
+	invertedOutcome = map[string]any{
 		"allow-installation":    "false",
 		"allow-connection":      "false",
 		"allow-auto-connection": "false",
@@ -830,7 +870,7 @@ var plugRuleCompilers = map[string]subruleCompiler{
 	"deny-auto-connection":  compilePlugConnectionConstraints,
 }
 
-func compilePlugRule(interfaceName string, rule interface{}) (*PlugRule, error) {
+func compilePlugRule(interfaceName string, rule any) (*PlugRule, error) {
 	context := fmt.Sprintf("plug rule for interface %q", interfaceName)
 	plugRule := &PlugRule{
 		Interface: interfaceName,
@@ -936,7 +976,8 @@ type SlotInstallationConstraints struct {
 
 	SlotAttributes *AttributeConstraints
 
-	OnClassic *OnClassicConstraint
+	OnClassic     *OnClassicConstraint
+	OnCoreDesktop *OnCoreDesktopConstraint
 
 	DeviceScope *DeviceScopeConstraint
 }
@@ -984,6 +1025,10 @@ func (c *SlotInstallationConstraints) setOnClassicConstraint(onClassic *OnClassi
 	c.OnClassic = onClassic
 }
 
+func (c *SlotInstallationConstraints) setOnCoreDesktopConstraint(onCoreDesktop *OnCoreDesktopConstraint) {
+	c.OnCoreDesktop = onCoreDesktop
+}
+
 func (c *SlotInstallationConstraints) setDeviceScopeConstraint(deviceScope *DeviceScopeConstraint) {
 	c.DeviceScope = deviceScope
 }
@@ -1032,7 +1077,8 @@ type SlotConnectionConstraints struct {
 	// PlugsPerSlot is always * (any) (for now)
 	PlugsPerSlot SideArityConstraint
 
-	OnClassic *OnClassicConstraint
+	OnClassic     *OnClassicConstraint
+	OnCoreDesktop *OnCoreDesktopConstraint
 
 	DeviceScope *DeviceScopeConstraint
 }
@@ -1108,6 +1154,10 @@ func (c *SlotConnectionConstraints) setOnClassicConstraint(onClassic *OnClassicC
 	c.OnClassic = onClassic
 }
 
+func (c *SlotConnectionConstraints) setOnCoreDesktopConstraint(onCoreDesktop *OnCoreDesktopConstraint) {
+	c.OnCoreDesktop = onCoreDesktop
+}
+
 func (c *SlotConnectionConstraints) setDeviceScopeConstraint(deviceScope *DeviceScopeConstraint) {
 	c.DeviceScope = deviceScope
 }
@@ -1131,7 +1181,7 @@ var slotRuleCompilers = map[string]subruleCompiler{
 	"deny-auto-connection":  compileSlotConnectionConstraints,
 }
 
-func compileSlotRule(interfaceName string, rule interface{}) (*SlotRule, error) {
+func compileSlotRule(interfaceName string, rule any) (*SlotRule, error) {
 	context := fmt.Sprintf("slot rule for interface %q", interfaceName)
 	slotRule := &SlotRule{
 		Interface: interfaceName,
