@@ -281,7 +281,7 @@ func (s *attrConstraintsSuite) TestCompileErrors(c *C) {
 		_, err := asserts.CompileAttributeConstraints(map[string]any{
 			"foo": wrong,
 		})
-		c.Check(err, ErrorMatches, fmt.Sprintf(`cannot compile "foo" constraint "%s": not a valid \$SLOT\(\)/\$PLUG\(\)/\$PLUG_PUBLISHER_ID/\$SLOT_PUBLISHER_ID constraint`, regexp.QuoteMeta(wrong)))
+		c.Check(err, ErrorMatches, fmt.Sprintf(`cannot compile "foo" constraint "%s": not a valid \$SLOT\(\)/\$PLUG\(\)/\$SLOT_COMPAT\(\)/\$PLUG_COMPAT\(\)/\$PLUG_PUBLISHER_ID/\$SLOT_PUBLISHER_ID constraint`, regexp.QuoteMeta(wrong)))
 
 	}
 }
@@ -290,6 +290,7 @@ type testEvalAttr struct {
 	comp            func(side string, arg string) (any, error)
 	plugPublisherID string
 	slotPublisherID string
+	compatLabels    bool
 }
 
 func (ca testEvalAttr) SlotAttr(arg string) (any, error) {
@@ -308,6 +309,10 @@ func (ca testEvalAttr) SlotPublisherID() string {
 	return ca.slotPublisherID
 }
 
+func (ca testEvalAttr) CompatLabelsEnabled() bool {
+	return ca.compatLabels
+}
+
 func (s *attrConstraintsSuite) TestEvalCheck(c *C) {
 	m, err := asserts.ParseHeaders([]byte(`attrs:
   foo: $SLOT(foo)
@@ -368,6 +373,53 @@ bar: bar.baz
 	c.Check(err, ErrorMatches, `attribute "foo" does not match \$SLOT\(foo\): foo != other-value`)
 }
 
+func (s *attrConstraintsSuite) TestEvalCheckSlotCompat(c *C) {
+	s.testEvalCheckCompat(c, "SLOT_COMPAT")
+}
+
+func (s *attrConstraintsSuite) TestEvalCheckPlugCompat(c *C) {
+	s.testEvalCheckCompat(c, "PLUG_COMPAT")
+}
+
+func (s *attrConstraintsSuite) testEvalCheckCompat(c *C, compatOper string) {
+	m, err := asserts.ParseHeaders([]byte(fmt.Sprintf(`attrs:
+  foo: $%s(foo)`, compatOper)))
+	c.Assert(err, IsNil)
+
+	// No context
+	cstrs, err := asserts.CompileAttributeConstraints(m["attrs"].(map[string]any))
+	c.Assert(err, IsNil)
+	c.Check(asserts.RuleFeature(cstrs, "dollar-attr-constraints"), Equals, true)
+	err = cstrs.Check(attrs(`
+foo: libx-3
+`), nil)
+	c.Check(err, ErrorMatches, `attribute "foo" cannot be matched without context`)
+
+	// Context, but no match
+	comp1 := func(op string, arg string) (any, error) {
+		return "libx-1", nil
+	}
+	err = cstrs.Check(attrs(`
+foo: libx
+`), testEvalAttr{comp: comp1, compatLabels: true})
+	c.Check(err, ErrorMatches, fmt.Sprintf(`attribute "foo" does not match \$%s\(foo\): libx != libx-1`, compatOper))
+
+	// Success case
+	comp2 := func(op string, arg string) (any, error) {
+		return "libx-1", nil
+	}
+	err = cstrs.Check(attrs(`
+foo: libx-(0..2)
+`), testEvalAttr{comp: comp2, compatLabels: true})
+	c.Check(err, IsNil)
+
+	// Now disabling labels produces an error
+	err = cstrs.Check(attrs(`
+foo: libx-(0..2)
+`), testEvalAttr{comp: comp2, compatLabels: false})
+	c.Check(err, ErrorMatches, fmt.Sprintf(`attribute "foo" constraint \$%s\(foo\) not evaluated: compatibility labels are disabled`, compatOper))
+}
+
 func (s *attrConstraintsSuite) TestCheckWithAttrPlugPublisherID(c *C) {
 	m, err := asserts.ParseHeaders([]byte(`attrs:
   my-attr: $PLUG_PUBLISHER_ID`))
