@@ -21,6 +21,7 @@ package boot
 
 import (
 	"fmt"
+	"os"
 	"path/filepath"
 	"sync"
 	"sync/atomic"
@@ -50,7 +51,6 @@ func newBootState20(typ snap.Type, dev snap.Device) bootState {
 
 // modeenvMu is used to protect sections doing:
 //   - read moddeenv/modify it(/reseal from it)
-//   - write modeenv/seal from it
 //
 // while we might want to release the global state lock as seal/reseal are slow
 // (see Unlocker for that)
@@ -59,6 +59,10 @@ var (
 	modeenvLocked int32
 )
 
+// TODO: we need to rethink the modeenv mutexes as naively releasing the state
+// lock while holding them can create deadlocks when we try to reacquire the
+// former
+
 func modeenvLock() {
 	modeenvMu.Lock()
 	atomic.AddInt32(&modeenvLocked, 1)
@@ -69,12 +73,12 @@ func modeenvUnlock() {
 	modeenvMu.Unlock()
 }
 
-func isModeeenvLocked() bool {
+func isModeenvLocked() bool {
 	return atomic.LoadInt32(&modeenvLocked) == 1
 }
 
 func loadModeenv() (*Modeenv, error) {
-	if !isModeeenvLocked() {
+	if !isModeenvLocked() {
 		return nil, fmt.Errorf("internal error: cannot read modeenv without the lock")
 	}
 	modeenv, err := ReadModeenv("")
@@ -153,6 +157,8 @@ type bootStateUpdate20 struct {
 
 	// tasks to run after the modeenv has been written
 	postModeenvTasks []bootCommitTask
+
+	revokeOldKeys bool
 }
 
 func (u20 *bootStateUpdate20) preModeenv(task bootCommitTask) {
@@ -183,8 +189,8 @@ func newBootStateUpdate20(m *Modeenv) (*bootStateUpdate20, error) {
 }
 
 // commit will write out boot state persistently to disk.
-func (u20 *bootStateUpdate20) commit() error {
-	if !isModeeenvLocked() {
+func (u20 *bootStateUpdate20) commit(markedSuccessful bool) error {
+	if !isModeenvLocked() {
 		return fmt.Errorf("internal error: cannot commit modeenv without the lock")
 	}
 
@@ -207,13 +213,28 @@ func (u20 *bootStateUpdate20) commit() error {
 		}
 	}
 
-	expectReseal := false
+	// None of the implementation of successfulBootState is expected to modify the model.
+	// So we can safely ignore FDE hooks.
+	resealOpts := ResealKeyToModeenvOptions{IgnoreFDEHooks: true, RevokeOldKeys: u20.revokeOldKeys}
+
 	// next write the modeenv if it changed
 	if !u20.writeModeenv.deepEqual(u20.modeenv) {
 		if err := u20.writeModeenv.Write(); err != nil {
 			return err
 		}
-		expectReseal = resealExpectedByModeenvChange(u20.writeModeenv, u20.modeenv)
+		resealOpts.ExpectReseal = resealExpectedByModeenvChange(u20.writeModeenv, u20.modeenv)
+	}
+
+	if markedSuccessful {
+		autoRepair, err := IsUnlockedWithRecoveryKey()
+		if err != nil {
+			if !os.IsNotExist(err) {
+				return err
+			}
+		} else if autoRepair {
+			resealOpts.Force = true
+			resealOpts.EnsureProvisioned = true
+		}
 	}
 
 	// next reseal using the modeenv values, we do this before any
@@ -225,7 +246,7 @@ func (u20 *bootStateUpdate20) commit() error {
 	// changed because of unasserted kernels, then pass a
 	// flag as hint whether to reseal based on whether we
 	// wrote the modeenv
-	if err := resealKeyToModeenv(dirs.GlobalRootDir, u20.writeModeenv, expectReseal, nil); err != nil {
+	if err := resealKeyToModeenv(dirs.GlobalRootDir, u20.writeModeenv, resealOpts, nil); err != nil {
 		return err
 	}
 
@@ -817,7 +838,10 @@ func (ba20 *bootState20BootAssets) markSuccessful(update bootStateUpdate) (bootS
 		return update, nil
 	}
 
-	newM, dropAssets, err := observeSuccessfulBootAssets(u20.writeModeenv)
+	newM, dropAssets, revokeOldKeys, err := observeSuccessfulBootAssets(u20.writeModeenv)
+	if revokeOldKeys {
+		u20.revokeOldKeys = true
+	}
 	if err != nil {
 		return nil, fmt.Errorf("cannot mark successful boot assets: %v", err)
 	}
