@@ -20,6 +20,7 @@
 package boot
 
 import (
+	"errors"
 	"fmt"
 	"os"
 	"os/exec"
@@ -34,6 +35,7 @@ import (
 	"github.com/snapcore/snapd/logger"
 	"github.com/snapcore/snapd/osutil"
 	"github.com/snapcore/snapd/osutil/kcmdline"
+	"github.com/snapcore/snapd/secboot"
 	"github.com/snapcore/snapd/snap"
 	"github.com/snapcore/snapd/snap/snapfile"
 	"github.com/snapcore/snapd/strutil"
@@ -604,7 +606,6 @@ func makeRunnableSystem(model *asserts.Model, bootWith *BootableSet, observer Tr
 		if err != nil {
 			return fmt.Errorf("cannot compose the candidate command line: %v", err)
 		}
-		modeenv.CurrentKernelCommandLines = bootCommandLines{cmdline}
 
 		// Look at gadget default values for system.kernel.*cmdline-append options
 		cmdlineAppend, err := buildOptionalKernelCommandLine(model, bootWith.UnpackedGadgetDir)
@@ -612,6 +613,9 @@ func makeRunnableSystem(model *asserts.Model, bootWith *BootableSet, observer Tr
 			return fmt.Errorf("while retrieving system.kernel.*cmdline-append defaults: %v", err)
 		}
 
+		modeenv.CurrentKernelCommandLines = bootCommandLines{
+			strutil.JoinNonEmpty([]string{cmdline, cmdlineAppend}, " ")}
+
 		candidate := false
 		defaultCmdLine, err := tbl.DefaultCommandLine(candidate)
 		if err != nil {
@@ -634,9 +638,9 @@ func makeRunnableSystem(model *asserts.Model, bootWith *BootableSet, observer Tr
 	}
 
 	if observer != nil && observerImpl.useEncryption {
-		hasHook, err := HasFDESetupHook(bootWith.Kernel)
-		if err != nil {
-			return fmt.Errorf("cannot check for fde-setup hook: %v", err)
+		protector, err := HookKeyProtectorFactory(bootWith.Kernel)
+		if err != nil && !errors.Is(err, secboot.ErrNoKeyProtector) {
+			return fmt.Errorf("cannot check for fde-setup hook key protector: %v", err)
 		}
 
 		tokens := UseTokens(model)
@@ -647,11 +651,11 @@ func makeRunnableSystem(model *asserts.Model, bootWith *BootableSet, observer Tr
 		}
 
 		flags := sealKeyToModeenvFlags{
-			HasFDESetupHook: hasHook,
-			FactoryReset:    makeOpts.AfterDataReset,
-			SeedDir:         makeOpts.SeedDir,
-			StateUnlocker:   makeOpts.StateUnlocker,
-			UseTokens:       tokens,
+			HookKeyProtectorFactory: protector,
+			FactoryReset:            makeOpts.AfterDataReset,
+			SeedDir:                 makeOpts.SeedDir,
+			StateUnlocker:           makeOpts.StateUnlocker,
+			UseTokens:               tokens,
 		}
 		if makeOpts.Standalone {
 			flags.SnapsDir = snapBlobDir
