@@ -20,6 +20,7 @@
 package assemblestate
 
 import (
+	"bytes"
 	"context"
 	"crypto/hmac"
 	"crypto/sha512"
@@ -31,39 +32,11 @@ import (
 	"sync"
 	"time"
 
+	"github.com/snapcore/snapd/asserts"
 	"github.com/snapcore/snapd/logger"
 	"github.com/snapcore/snapd/randutil"
 )
 
-// Transport provides an abstraction for defining how incoming and outgoing
-// messages are handled in an assembly session.
-type Transport interface {
-	// Serve starts a server that handles incoming requests and routes them to
-	// the provided [AssembleState].
-	Serve(ctx context.Context, addr string, cert tls.Certificate, as *AssembleState) error
-
-	// NewClient creates a client for sending outbound messages compatible with
-	// this [Transport].
-	NewClient(cert tls.Certificate) Client
-
-	// Stats returns the sent and received byte counts for this assembly
-	// session.
-	Stats() (sent, received, tx, rx int64)
-}
-
-// Client is used to communicate with our peers.
-type Client interface {
-	// Trusted sends a message to a trusted peer. Implementations must verify
-	// that the peer is using the given certificate.
-	Trusted(ctx context.Context, addr string, cert []byte, kind string, message any) error
-	// Untrusted sends a message to a peer that we do not yet trust. The
-	// certificate that the peer used to communicate is returned.
-	Untrusted(ctx context.Context, addr string, kind string, message any) (cert []byte, err error)
-}
-
-// Discoverer returns a set of addresses that should be considered for assembly.
-type Discoverer = func(context.Context) ([]string, error)
-
 // AssembleState contains this device's knowledge of the state of an assembly
 // session.
 type AssembleState struct {
@@ -79,11 +52,11 @@ type AssembleState struct {
 	// with the lock held.
 	lock sync.Mutex
 
-	// trusted keeps track of all trusted peers.
-	trusted map[Fingerprint]Peer
+	// trusted keeps track of all trusted peers by fingerprint.
+	trusted map[Fingerprint]DeviceToken
 
 	// fingerprints keeps track of the TLS certificate fingerprints we know and
-	// the RDTs that they are is associated with.
+	// the RDTs that they are associated with.
 	fingerprints map[DeviceToken]Fingerprint
 
 	// addresses keeps track of which address we can reach each device at.
@@ -103,13 +76,18 @@ type AssembleState struct {
 	// selector keeps track of our routes and decides the strategy for
 	// publishing routes to our peers.
 	selector RouteSelector
+
+	// completed is used to signal to the [AssembleState.Run] method that we
+	// have assembled a fully connected graph of the expected number of devices.
+	// This is only relevant when an expected size is provided.
+	completed chan struct{}
 }
 
 // AssembleSession provides a method for serializing our current state of
 // assembly to JSON.
 type AssembleSession struct {
 	Initiated  time.Time              `json:"initiated"`
-	Trusted    map[string]Peer        `json:"trusted"`
+	Trusted    map[string]DeviceToken `json:"trusted"`
 	Addresses  map[string]string      `json:"addresses"`
 	Discovered []string               `json:"discovered"`
 	Routes     Routes                 `json:"routes"`
@@ -117,11 +95,11 @@ type AssembleSession struct {
 }
 
 func (as *AssembleState) export() AssembleSession {
-	var trusted map[string]Peer
+	var trusted map[string]DeviceToken
 	if len(as.trusted) > 0 {
-		trusted = make(map[string]Peer, len(as.trusted))
-		for fp, p := range as.trusted {
-			trusted[base64.StdEncoding.EncodeToString(fp[:])] = p
+		trusted = make(map[string]DeviceToken, len(as.trusted))
+		for fp, rdt := range as.trusted {
+			trusted[base64.StdEncoding.EncodeToString(fp[:])] = rdt
 		}
 	}
 
@@ -151,27 +129,14 @@ func (as *AssembleState) export() AssembleSession {
 	}
 }
 
-// Peer is a peer that has established trust via proof of the shared secret in
-// an assemble session.
-type Peer struct {
-	// RDT is the device token that this peer used to identity itself.
-	RDT DeviceToken `json:"rdt"`
-	// Cert is the TLS certificate that this peer used to send its messages.
-	Cert []byte `json:"cert"`
-}
-
 // AssembleConfig contains the configuration parameters required to initialize
 // an AssembleState and participate in an assembly session.
 type AssembleConfig struct {
 	// Secret is the shared secret used for HMAC-based peer authentication.
 	Secret string
-	// RDT is this device's random device token used to uniquely identity this
+	// RDT is this device's random device token used to uniquely identify this
 	// device.
 	RDT DeviceToken
-	// IP is the IP address to bind the assembly server to.
-	IP net.IP
-	// Port is the port number to bind the assembly server to.
-	Port int
 	// TLSCert is the PEM-encoded TLS certificate for this device.
 	TLSCert []byte
 	// TLSKey is the PEM-encoded private key corresponding to TLSCert.
@@ -179,6 +144,14 @@ type AssembleConfig struct {
 	// Clock is an optional function to retrieve the current time. If nil,
 	// defaults to time.Now.
 	Clock func() time.Time
+	// ExpectedSize is the expected size of the cluster. If unset, cluster
+	// assembly will not terminate automatically.
+	ExpectedSize int
+	// Serial is this device's serial assertion.
+	Serial *asserts.Serial
+	// Signer is a function that signs the given data with the private key that
+	// matches the public key embedded in the serial assertion.
+	Signer func([]byte) ([]byte, error)
 }
 
 const AssembleSessionLength = time.Hour
@@ -190,13 +163,21 @@ func NewAssembleState(
 	session AssembleSession,
 	selector func(self DeviceToken, identified func(DeviceToken) bool) (RouteSelector, error),
 	commit func(AssembleSession),
+	assertDB asserts.RODatabase,
 ) (*AssembleState, error) {
+	if config.Serial == nil {
+		return nil, errors.New("serial assertion is required")
+	}
+	if config.Signer == nil {
+		return nil, errors.New("signer function is required")
+	}
+
 	// default clock to time.Now if not provided
 	if config.Clock == nil {
 		config.Clock = time.Now
 	}
 
-	// validate the given session parse it into more useful data structures
+	// validate the given session and parse it into more useful data structures
 	validated, err := validateSession(session, config.Clock)
 	if err != nil {
 		return nil, fmt.Errorf("invalid session data: %w", err)
@@ -207,14 +188,34 @@ func NewAssembleState(
 		return nil, err
 	}
 
-	if err := ensureLocalDevicePresent(&validated.devices, Identity{
-		RDT: config.RDT,
-		FP:  CalculateFP(cert.Certificate[0]),
+	// calculate the HMAC that this device would use to authenticate itself
+	fp := CalculateFP(cert.Certificate[0])
+	hmac := CalculateHMAC(config.RDT, fp, config.Secret)
+
+	// sign the HMAC with our private key to create the SerialProof
+	proof, err := config.Signer(hmac)
+	if err != nil {
+		return nil, fmt.Errorf("cannot sign hmac for serial proof: %v", err)
+	}
+
+	bundle, err := buildSerialBundle(config.Serial, assertDB)
+	if err != nil {
+		return nil, fmt.Errorf("cannot build serial bundle: %w", err)
+	}
+
+	if err := ensureLocalDevicePresent(&validated, assertDB, config.Secret, Identity{
+		RDT:          config.RDT,
+		FP:           CalculateFP(cert.Certificate[0]),
+		SerialBundle: bundle,
+		SerialProof:  proof,
 	}); err != nil {
 		return nil, err
 	}
 
-	devices := NewDeviceQueryTracker(validated.devices, time.Minute*5, config.Clock)
+	devices, err := NewDeviceQueryTracker(validated.devices, time.Minute*5, config.Clock, assertDB, config.Secret)
+	if err != nil {
+		return nil, err
+	}
 
 	sel, err := selector(config.RDT, devices.Identified)
 	if err != nil {
@@ -232,40 +233,44 @@ func NewAssembleState(
 	// for any peers that we already trust and know their address, we can safely
 	// inform the selector that the route from our local node to that peer can
 	// be published
-	for fp, peer := range validated.trusted {
+	for fp, rdt := range validated.trusted {
 		addr, ok := validated.addresses[fp]
 		if !ok {
 			continue
 		}
 
-		sel.AddAuthoritativeRoute(peer.RDT, addr)
+		sel.AddAuthoritativeRoute(rdt, addr)
 	}
 
-	// calculate the HMAC once for this device's authentication
-	authHMAC := CalculateHMAC(config.RDT, CalculateFP(cert.Certificate[0]), config.Secret)
-
 	as := AssembleState{
 		initiated:    validated.initiated,
 		config:       config,
 		commit:       commit,
 		clock:        config.Clock,
 		cert:         cert,
-		authHMAC:     authHMAC,
+		authHMAC:     hmac,
 		trusted:      validated.trusted,
 		fingerprints: validated.fingerprints,
 		addresses:    validated.addresses,
 		discovered:   validated.discovered,
 		devices:      devices,
 		selector:     sel,
+		completed:    make(chan struct{}, 1),
+	}
+
+	// there is a chance that we resume an assemble session that was already
+	// finished. make sure to mark it as complete in that case.
+	if err := as.shutdownAtExpectedSize(); err != nil {
+		return nil, err
 	}
 
 	return &as, nil
 }
 
 // publishAuthAndCommit uses the given [Client] to publish to each device. If
-// publication succeeds, then the certificate returned by [Client.Untrusted]
-// (the certificate that the peer used to communicate with us) is associated
-// with the address that we reached them at.
+// publication succeeds, then the fingerprint returned by [Client.Untrusted]
+// (the fingerprint of the certificate that the peer used to communicate with
+// us) is associated with the address that we reached them at.
 //
 // If the certificate the peer used is already trusted (they've already
 // published their auth message to us), then we verify the route from this local
@@ -281,7 +286,7 @@ func (as *AssembleState) publishAuthAndCommit(ctx context.Context, addresses []s
 			continue
 		}
 
-		cert, err := untrustedSend(ctx, &as.lock, client, addr, "auth", Auth{
+		fp, err := untrustedSend(ctx, &as.lock, client, addr, "auth", Auth{
 			HMAC: as.authHMAC,
 			RDT:  as.config.RDT,
 		})
@@ -292,8 +297,6 @@ func (as *AssembleState) publishAuthAndCommit(ctx context.Context, addresses []s
 
 		logger.Debugf("sent auth message to %s", addr)
 
-		fp := CalculateFP(cert)
-
 		if expected, ok := as.addresses[fp]; ok && expected != addr {
 			return fmt.Errorf("found new address %s using same certificate as other address %s", addr, expected)
 		}
@@ -302,8 +305,8 @@ func (as *AssembleState) publishAuthAndCommit(ctx context.Context, addresses []s
 		as.addresses[fp] = addr
 		as.discovered[addr] = true
 
-		if p, ok := as.trusted[fp]; ok {
-			as.selector.AddAuthoritativeRoute(p.RDT, addr)
+		if rdt, ok := as.trusted[fp]; ok {
+			as.selector.AddAuthoritativeRoute(rdt, addr)
 		}
 	}
 
@@ -319,18 +322,18 @@ func (as *AssembleState) publishDeviceQueries(ctx context.Context, client Client
 	as.lock.Lock()
 	defer as.lock.Unlock()
 
-	for fp, p := range as.trusted {
+	for fp, rdt := range as.trusted {
 		addr, ok := as.addresses[fp]
 		if !ok {
 			continue
 		}
 
-		queries, ack := as.devices.OutgoingQueriesTo(p.RDT)
+		queries, ack := as.devices.OutgoingQueriesTo(rdt)
 		if len(queries) == 0 {
 			continue
 		}
 
-		err := trustedSend(ctx, &as.lock, client, addr, p.Cert, "unknown", UnknownDevices{
+		err := trustedSend(ctx, &as.lock, client, addr, fp, "unknown", UnknownDevices{
 			Devices: queries,
 		})
 		ack(err == nil)
@@ -339,7 +342,7 @@ func (as *AssembleState) publishDeviceQueries(ctx context.Context, client Client
 			continue
 		}
 
-		logger.Debugf("sent device queries to %s at %s, count: %d", p.RDT, addr, len(queries))
+		logger.Debugf("sent device queries to %s at %s, count: %d", rdt, addr, len(queries))
 	}
 
 	// we don't commit when publishing device queries since we don't keep track
@@ -355,18 +358,18 @@ func (as *AssembleState) publishDevicesAndCommit(ctx context.Context, client Cli
 	as.lock.Lock()
 	defer as.lock.Unlock()
 
-	for fp, p := range as.trusted {
+	for fp, rdt := range as.trusted {
 		addr, ok := as.addresses[fp]
 		if !ok {
 			continue
 		}
 
-		ids, ack := as.devices.ResponsesTo(p.RDT)
+		ids, ack := as.devices.ResponsesTo(rdt)
 		if len(ids) == 0 {
 			continue
 		}
 
-		err := trustedSend(ctx, &as.lock, client, addr, p.Cert, "devices", Devices{
+		err := trustedSend(ctx, &as.lock, client, addr, fp, "devices", Devices{
 			Devices: ids,
 		})
 		ack(err == nil)
@@ -375,7 +378,7 @@ func (as *AssembleState) publishDevicesAndCommit(ctx context.Context, client Cli
 			continue
 		}
 
-		logger.Debugf("sent device information to %s at %s, count: %d", p.RDT, addr, len(ids))
+		logger.Debugf("sent device information to %s at %s, count: %d", rdt, addr, len(ids))
 	}
 
 	as.commit(as.export())
@@ -388,11 +391,11 @@ func (as *AssembleState) publishRoutes(ctx context.Context, client Client, maxPe
 	as.lock.Lock()
 	defer as.lock.Unlock()
 
-	// collect all trusted peers that have also addresses
-	var available []Peer
-	for fp, p := range as.trusted {
+	// collect all trusted peers that have addresses
+	var available []DeviceToken
+	for fp, rdt := range as.trusted {
 		if _, ok := as.addresses[fp]; ok {
-			available = append(available, p)
+			available = append(available, rdt)
 		}
 	}
 
@@ -406,13 +409,13 @@ func (as *AssembleState) publishRoutes(ctx context.Context, client Client, maxPe
 	selected := available[:min(maxPeers, len(available))]
 
 	// for each randomly selected peer, get routes and send them
-	for _, p := range selected {
-		routes, ack, ok := as.selector.Select(p.RDT, maxRoutes)
+	for _, rdt := range selected {
+		routes, ack, ok := as.selector.Select(rdt, maxRoutes)
 		if !ok {
 			continue
 		}
 
-		fp, ok := as.fingerprints[p.RDT]
+		fp, ok := as.fingerprints[rdt]
 		if !ok {
 			continue
 		}
@@ -422,13 +425,13 @@ func (as *AssembleState) publishRoutes(ctx context.Context, client Client, maxPe
 			continue
 		}
 
-		if err := trustedSend(ctx, &as.lock, client, addr, p.Cert, "routes", routes); err != nil {
+		if err := trustedSend(ctx, &as.lock, client, addr, fp, "routes", routes); err != nil {
 			logger.Debugf("cannot publish routes: %v", err)
 			continue
 		}
 
 		ack()
-		logger.Debugf("sent routes to %s at %s, count: %d", p.RDT, addr, len(routes.Routes)/3)
+		logger.Debugf("sent routes to %s at %s, count: %d", rdt, addr, len(routes.Routes)/3)
 	}
 
 	// we don't commit on route publishes since we don't keep track of which
@@ -443,10 +446,10 @@ func shuffle[T any](available []T) {
 	}
 }
 
-// authenticateAndCommit checks that the given [Auth] message is valid and proves
+// AuthenticateAndCommit checks that the given [Auth] message is valid and proves
 // knowledge of the shared secret. If this check is passed, we allow mutation of
 // this [AssembleState] via future calls to [AssembleState.verifyPeer] with the same
-// certificate.
+// certificate fingerprint.
 //
 // An error is returned if the message's HMAC is found to not prove knowledge of
 // the shared secret.
@@ -454,26 +457,21 @@ func shuffle[T any](available []T) {
 // This method is to be called by an implementation of the [Transport] interface.
 //
 // This method calls AssembleState.commit with the current state.
-func (as *AssembleState) authenticateAndCommit(auth Auth, cert []byte) error {
+func (as *AssembleState) AuthenticateAndCommit(auth Auth, fp Fingerprint) error {
 	as.lock.Lock()
 	defer as.lock.Unlock()
 
-	fp := CalculateFP(cert)
-
 	expectedHMAC := CalculateHMAC(auth.RDT, fp, as.config.Secret)
 	if !hmac.Equal(expectedHMAC, auth.HMAC) {
 		return errors.New("received invalid HMAC from peer")
 	}
 
-	if _, ok := as.trusted[fp]; ok {
-		if as.trusted[fp].RDT != auth.RDT {
-			return fmt.Errorf("peer %q and %q are using the same TLS certificate", as.trusted[fp].RDT, auth.RDT)
+	if existing, ok := as.trusted[fp]; ok {
+		if existing != auth.RDT {
+			return fmt.Errorf("peer %q and %q are using the same TLS certificate", existing, auth.RDT)
 		}
 	} else {
-		as.trusted[fp] = Peer{
-			RDT:  auth.RDT,
-			Cert: cert,
-		}
+		as.trusted[fp] = auth.RDT
 	}
 
 	if existing, ok := as.fingerprints[auth.RDT]; ok {
@@ -503,49 +501,54 @@ func (as *AssembleState) authenticateAndCommit(auth Auth, cert []byte) error {
 	return nil
 }
 
-// verifyPeer checks if the given certificate is trusted and maps to a known RDT.
-// If it is, then a [peerHandle] is returned that can be used to modify the state
-// of the cluster on this peer's behalf.
+// VerifyPeer checks if the given certificate fingerprint is trusted and maps to
+// a known RDT. If it is, then a [VerifiedPeer] is returned that can be used to
+// modify the state of the cluster on this peer's behalf.
 //
 // An error is returned if the certificate isn't trusted.
 //
 // This method is to be called by an implementation of the [Transport] interface.
-func (as *AssembleState) verifyPeer(cert []byte) (*peerHandle, error) {
+func (as *AssembleState) VerifyPeer(fp Fingerprint) (VerifiedPeer, error) {
 	as.lock.Lock()
 	defer as.lock.Unlock()
 
-	fp := CalculateFP(cert)
-
-	p, ok := as.trusted[fp]
+	rdt, ok := as.trusted[fp]
 	if !ok {
 		return nil, errors.New("given TLS certificate is not associated with a trusted RDT")
 	}
 
 	return &peerHandle{
-		as:   as,
-		peer: p.RDT,
+		as:  as,
+		rdt: rdt,
 	}, nil
 }
 
-// Run starts the assembly process, managing both the server and periodic client operations.
-// It returns when the context is cancelled, returning the final routes discovered.
-//
-// TODO: a good chunk of this method is missing coverage, this should be
-// addressed once a real [Transport] is merged.
+// RunOptions carries some options relevant to [AssembleState.Run]. These should
+// be only runtime-relevant parameters.
+type RunOptions struct {
+	// Period is how often we publish route information.
+	Period time.Duration
+}
+
+// Run starts the assembly process, managing both the server and periodic client
+// operations. It returns when the context is cancelled, returning the
+// discovered device identities and routes.
 func (as *AssembleState) Run(
 	ctx context.Context,
+	ln net.Listener,
 	transport Transport,
-	discover Discoverer,
-) (Routes, error) {
+	discoveries <-chan []string,
+	opts RunOptions,
+) ([]Identity, Routes, error) {
 	if as.initiated.IsZero() {
 		as.initiated = as.clock()
 	}
 
 	if as.clock().Sub(as.initiated) > AssembleSessionLength {
-		return Routes{}, errors.New("cannot resume an assembly session that began more than an hour ago")
+		return nil, Routes{}, errors.New("cannot resume an assembly session that began more than an hour ago")
 	}
 
-	addr := fmt.Sprintf("%s:%d", as.config.IP, as.config.Port)
+	addr := ln.Addr().String()
 	client := transport.NewClient(as.cert)
 
 	ctx, cancel := context.WithCancel(ctx)
@@ -554,13 +557,14 @@ func (as *AssembleState) Run(
 	var wg sync.WaitGroup
 
 	// channel to receive server errors that should cause the process to fail
+	// TODO:GOVERSION: replace with [context.Cause] when we're on go >= 1.20
 	serverError := make(chan error, 1)
 
 	// start the server that handles incoming requests
 	wg.Add(1)
 	go func() {
 		defer wg.Done()
-		if err := transport.Serve(ctx, addr, as.cert, as); err != nil {
+		if err := transport.Serve(ctx, ln, as.cert, as); err != nil {
 			// only propagate non-context.Canceled errors
 			if !errors.Is(err, context.Canceled) {
 				logger.Debugf("server error: %v", err)
@@ -570,31 +574,29 @@ func (as *AssembleState) Run(
 		}
 	}()
 
-	// start periodic discovery of peers
+	// start discovery of peers from channel
 	wg.Add(1)
 	go func() {
 		defer wg.Done()
-		periodic(ctx, time.Second*5, func(ctx context.Context) {
-			discoveries, err := discover(ctx)
-			if err != nil {
-				logger.Debugf("error discovering peers: %v", err)
-				return
-			}
-
-			// filter out our address
-			addrs := make([]string, 0, len(discoveries))
-			for _, d := range discoveries {
-				if d == addr {
-					continue
+		for {
+			select {
+			case discoveries := <-discoveries:
+				// filter out our address
+				addrs := make([]string, 0, len(discoveries))
+				for _, d := range discoveries {
+					if d == addr {
+						continue
+					}
+					addrs = append(addrs, d)
 				}
-				addrs = append(addrs, d)
-			}
 
-			if err := as.publishAuthAndCommit(ctx, addrs, client); err != nil {
-				logger.Debugf("error publishing auth messages: %v", err)
+				if err := as.publishAuthAndCommit(ctx, addrs, client); err != nil {
+					logger.Debugf("error publishing auth messages: %v", err)
+				}
+			case <-ctx.Done():
 				return
 			}
-		})
+		}
 	}()
 
 	var rounds int
@@ -604,11 +606,10 @@ func (as *AssembleState) Run(
 	go func() {
 		defer wg.Done()
 		const (
-			period = time.Second * 5
 			peers  = 5
 			routes = 5000
 		)
-		periodic(ctx, period, func(ctx context.Context) {
+		periodic(ctx, opts.Period, func(ctx context.Context) {
 			as.publishRoutes(ctx, client, peers, routes)
 			rounds++
 		})
@@ -631,32 +632,51 @@ func (as *AssembleState) Run(
 		}
 	}()
 
+	// wait for a completion signal
+	wg.Add(1)
+	go func() {
+		defer wg.Done()
+
+		select {
+		case <-as.completed:
+			logger.Debug("assembly complete: discovered all devices with full connectivity")
+			cancel()
+		case <-ctx.Done():
+		}
+	}()
+
 	// wait for context cancellation
 	wg.Wait()
 
 	select {
 	case err := <-serverError:
-		return Routes{}, fmt.Errorf("server failed: %w", err)
+		return nil, Routes{}, fmt.Errorf("server failed: %w", err)
 	default:
 	}
 
+	// at this point, the [Transport] should have shut down and no longer
+	// interface with our data. however, a misbehaving implementation might. in
+	// that case, we need to lock.
+	as.lock.Lock()
+	defer as.lock.Unlock()
+
 	// perform final fingerprint consistency check
 	devices := as.devices.Export()
 	for _, identity := range devices.IDs {
 		if fp, ok := as.fingerprints[identity.RDT]; ok {
 			if fp != identity.FP {
-				return Routes{}, fmt.Errorf("consistency check failed: fingerprint mismatch for device %s", identity.RDT)
+				return nil, Routes{}, fmt.Errorf("consistency check failed: fingerprint mismatch for device %s", identity.RDT)
 			}
 		}
 	}
 
-	sent, received, tx, rx := transport.Stats()
+	stats := transport.Stats()
 	logger.Debugf(
 		"assemble stopped after %d rounds, sent: %d messages (%d bytes), received: %d messages (%d bytes)",
-		rounds, sent, tx, received, rx,
+		rounds, stats.Sent, stats.Tx, stats.Received, stats.Rx,
 	)
 
-	return as.selector.Routes(), nil
+	return devices.IDs, as.selector.Routes(), nil
 }
 
 func periodic(
@@ -676,15 +696,10 @@ func periodic(
 }
 
 // peerHandle is a wrapper over [AssembleState] that enables an authenticated
-// peer report its knowledge of the state of the cluster.
+// peer to report its knowledge of the state of the cluster.
 type peerHandle struct {
-	as   *AssembleState
-	peer DeviceToken
-}
-
-// RDT returns the RDT of the device that this [peerHandle] represents.
-func (h *peerHandle) RDT() DeviceToken {
-	return h.peer
+	as  *AssembleState
+	rdt DeviceToken
 }
 
 // CommitDeviceQueries adds the given devices to the queue of queries for this
@@ -697,10 +712,10 @@ func (h *peerHandle) CommitDeviceQueries(unknown UnknownDevices) error {
 	h.as.lock.Lock()
 	defer h.as.lock.Unlock()
 
-	h.as.devices.RecordIncomingQuery(h.peer, unknown.Devices)
+	h.as.devices.RecordIncomingQuery(h.rdt, unknown.Devices)
 
 	h.as.commit(h.as.export())
-	logger.Debugf("got device queries from %q", h.peer)
+	logger.Debugf("got device queries from %q", h.rdt)
 
 	return nil
 }
@@ -714,19 +729,43 @@ func (h *peerHandle) CommitRoutes(routes Routes) error {
 
 	// if this peer is sending us routes that include these devices, then we
 	// know that they must have identifying information for those devices.
-	h.as.devices.RecordDevicesKnownBy(h.peer, routes.Devices)
+	h.as.devices.RecordDevicesKnownBy(h.rdt, routes.Devices)
 
-	added, total, err := h.as.selector.RecordRoutes(h.peer, routes)
+	added, total, err := h.as.selector.RecordRoutes(h.rdt, routes)
 	if err != nil {
 		return err
 	}
 
+	if err := h.as.shutdownAtExpectedSize(); err != nil {
+		return err
+	}
+
 	h.as.commit(h.as.export())
 
 	// routes are represented by an array of triplets, refer to the doc comment
 	// on [Routes] for more information
 	received := len(routes.Routes) / 3
-	logger.Debugf("got routes update from %s, received: %d, wasted: %d, total: %d", h.peer, received, received-added, total)
+	logger.Debugf("got routes update from %s, received: %d, wasted: %d, total: %d", h.rdt, received, received-added, total)
+
+	return nil
+}
+
+func (as *AssembleState) shutdownAtExpectedSize() error {
+	if as.config.ExpectedSize == 0 {
+		return nil
+	}
+
+	complete, err := as.selector.Complete(as.config.ExpectedSize)
+	if err != nil {
+		return err
+	}
+
+	if complete {
+		select {
+		case as.completed <- struct{}{}:
+		default:
+		}
+	}
 
 	return nil
 }
@@ -744,7 +783,7 @@ func (h *peerHandle) CommitDevices(devices Devices) error {
 
 	for _, id := range devices.Devices {
 		if current, ok := h.as.devices.Lookup(id.RDT); ok {
-			if current != id {
+			if current.RDT != id.RDT || current.FP != id.FP || current.SerialBundle != id.SerialBundle || !bytes.Equal(current.SerialProof, id.SerialProof) {
 				return errors.New("got inconsistent device identity")
 			}
 		}
@@ -756,7 +795,9 @@ func (h *peerHandle) CommitDevices(devices Devices) error {
 	}
 
 	for _, id := range devices.Devices {
-		h.as.devices.RecordIdentity(id)
+		if err := h.as.devices.RecordIdentity(id); err != nil {
+			return err
+		}
 	}
 
 	// TODO: i don't really love the implicit connection of as.devices and
@@ -766,9 +807,13 @@ func (h *peerHandle) CommitDevices(devices Devices) error {
 	// valid to send to our peers
 	h.as.selector.VerifyRoutes()
 
+	if err := h.as.shutdownAtExpectedSize(); err != nil {
+		return err
+	}
+
 	h.as.commit(h.as.export())
 
-	logger.Debugf("got unknown device information from %s, count: %d", h.peer, len(devices.Devices))
+	logger.Debugf("got unknown device information from %s, count: %d", h.rdt, len(devices.Devices))
 
 	return nil
 }
@@ -787,41 +832,100 @@ func CalculateFP(cert []byte) Fingerprint {
 // trustedSend releases the given lock and calls client.Trusted.
 func trustedSend(
 	ctx context.Context, lock *sync.Mutex, client Client,
-	addr string, cert []byte, kind string, data any,
+	addr string, fp Fingerprint, kind string, data any,
 ) error {
 	lock.Unlock()
 	defer lock.Lock()
-	return client.Trusted(ctx, addr, cert, kind, data)
+	return client.Trusted(ctx, addr, fp, kind, data)
 }
 
 // untrustedSend releases the given lock and calls client.Untrusted.
 func untrustedSend(
 	ctx context.Context, lock *sync.Mutex, client Client,
 	addr string, kind string, data any,
-) ([]byte, error) {
+) (Fingerprint, error) {
 	lock.Unlock()
 	defer lock.Lock()
 	return client.Untrusted(ctx, addr, kind, data)
 }
 
-// ensureLocalDevicePresent adds the local device identity to the IDs slice if
-// not present, or validates consistency if already present.
-func ensureLocalDevicePresent(data *DeviceQueryTrackerData, self Identity) error {
-	for _, existing := range data.IDs {
-		if existing.RDT == self.RDT {
-			if existing.FP != self.FP {
+func buildSerialBundle(serial *asserts.Serial, db asserts.RODatabase) (string, error) {
+	retrieve := func(ref *asserts.Ref) (asserts.Assertion, error) {
+		return ref.Resolve(db.Find)
+	}
+
+	buf := bytes.NewBuffer(nil)
+	enc := asserts.NewEncoder(buf)
+
+	fetcher := asserts.NewFetcher(db, retrieve, enc.Encode)
+	if err := fetcher.Save(serial); err != nil {
+		return "", fmt.Errorf("cannot bundle serial prerequisites: %w", err)
+	}
+
+	return buf.String(), nil
+}
+
+// ensureLocalDevicePresent ensures that the local device identity is present in
+// our internal representation of authenticated devices and device identities.
+func ensureLocalDevicePresent(
+	session *validatedSession,
+	db asserts.RODatabase,
+	secret string,
+	self Identity,
+) error {
+	if err := ensureLocalDeviceAuthenticated(session, self); err != nil {
+		return err
+	}
+
+	for _, unverifiedID := range session.devices.IDs {
+		if unverifiedID.RDT == self.RDT {
+			if unverifiedID.FP != self.FP {
 				return fmt.Errorf("fingerprint mismatch for local device %q", self.RDT)
 			}
 
+			// proof signatures will not be byte-identical across runs. thus, to
+			// check for consistency, we need to validate the existing proof
+			// instead of requiring equality
+			serial, err := verifySerialBundle(unverifiedID.SerialBundle, db)
+			if err != nil {
+				return fmt.Errorf("cannot validate serial bundle for local device %q: %w", self.RDT, err)
+			}
+
+			expectedHMAC := CalculateHMAC(unverifiedID.RDT, unverifiedID.FP, secret)
+			if err := asserts.RawVerifyWithKey(expectedHMAC, unverifiedID.SerialProof, serial.DeviceKey()); err != nil {
+				return fmt.Errorf("serial proof mismatch for local device %q: %w", self.RDT, err)
+			}
+
+			// this shouldn't really be possible, since above we've shown that
+			// the existing and new proof were both signed using the same
+			// private key, which is derived from the serial bundle.
+			if unverifiedID.SerialBundle != self.SerialBundle {
+				return fmt.Errorf("serial bundle mismatch for local device %q", self.RDT)
+			}
+
 			return nil
 		}
 	}
 
-	data.IDs = append(data.IDs, self)
+	session.devices.IDs = append(session.devices.IDs, self)
 
 	return nil
 }
 
+func ensureLocalDeviceAuthenticated(session *validatedSession, self Identity) error {
+	if rdt, ok := session.trusted[self.FP]; ok && rdt != self.RDT {
+		return errors.New("current device fingerprint changed when resuming session")
+	}
+
+	if fp, ok := session.fingerprints[self.RDT]; ok && fp != self.FP {
+		return errors.New("current device fingerprint changed when resuming session")
+	}
+
+	session.trusted[self.FP] = self.RDT
+	session.fingerprints[self.RDT] = self.FP
+	return nil
+}
+
 // parseAndValidateFingerprint parses a base64-encoded fingerprint and validates
 // its length
 func parseAndValidateFingerprint(strFP string) (Fingerprint, error) {
@@ -840,7 +944,7 @@ func parseAndValidateFingerprint(strFP string) (Fingerprint, error) {
 // validatedSession contains pre-parsed session data with field names matching
 // [AssembleState].
 type validatedSession struct {
-	trusted      map[Fingerprint]Peer
+	trusted      map[Fingerprint]DeviceToken
 	fingerprints map[DeviceToken]Fingerprint
 	addresses    map[Fingerprint]string
 	discovered   map[string]bool
@@ -877,16 +981,16 @@ func validateSession(session AssembleSession, clock func() time.Time) (validated
 		}
 	}
 
-	trusted := make(map[Fingerprint]Peer, len(session.Trusted))
+	trusted := make(map[Fingerprint]DeviceToken, len(session.Trusted))
 	fingerprints := make(map[DeviceToken]Fingerprint, len(session.Trusted))
-	for strFP, peer := range session.Trusted {
+	for strFP, rdt := range session.Trusted {
 		fp, err := parseAndValidateFingerprint(strFP)
 		if err != nil {
 			return validatedSession{}, fmt.Errorf("invalid fingerprint in trusted peers: %w", err)
 		}
 
-		trusted[fp] = peer
-		fingerprints[peer.RDT] = fp
+		trusted[fp] = rdt
+		fingerprints[rdt] = fp
 	}
 
 	addresses := make(map[Fingerprint]string, len(session.Addresses))
