@@ -30,6 +30,7 @@ import (
 	"github.com/snapcore/snapd/osutil/disks"
 	"github.com/snapcore/snapd/secboot"
 	"github.com/snapcore/snapd/seed"
+	"github.com/snapcore/snapd/snap/integrity"
 	"github.com/snapcore/snapd/testutil"
 	"github.com/snapcore/snapd/timings"
 )
@@ -54,11 +55,11 @@ type OverlayFsOptions = overlayFsOptions
 type DmVerityOptions = dmVerityOptions
 type SystemdMountOptions = systemdMountOptions
 
-type RecoverDegradedState = recoverDegradedState
+type DiskUnlockState = diskUnlockState
 
 type PartitionState = partitionState
 
-func (r *RecoverDegradedState) Degraded(isEncrypted bool) bool {
+func (r *DiskUnlockState) Degraded(isEncrypted bool) bool {
 	m := recoverModeStateMachine{
 		isEncryptedDev: isEncrypted,
 		degradedState:  r,
@@ -257,3 +258,11 @@ func MockOsGetenv(mock func(string) string) (restore func()) {
 		osGetenv = old
 	}
 }
+
+func MockLookupDmVerityDataAndCrossCheck(f func(snapPath string, params *integrity.IntegrityDataParams) (string, error)) (restore func()) {
+	old := lookupDmVerityDataAndCrossCheck
+	lookupDmVerityDataAndCrossCheck = f
+	return func() {
+		lookupDmVerityDataAndCrossCheck = old
+	}
+}
