@@ -296,7 +296,7 @@ func (s *appsSuite) TestGetAppsInfoNames(c *check.C) {
 
 func (s *appsSuite) TestGetAppsInfoServices(c *check.C) {
 	r := daemon.MockNewStatusDecorator(func(ctx context.Context, isGlobal bool, uid string) clientutil.StatusDecorator {
-		c.Check(isGlobal, check.Equals, false)
+		c.Check(isGlobal, check.Equals, true)
 		c.Check(uid, check.Equals, "0")
 		return s
 	})
@@ -1059,9 +1059,8 @@ func (s *appsSuite) TestLogsN(c *check.C) {
 
 	for _, t := range []T{
 		{in: "", out: 10},
-		{in: "0", out: 0},
 		{in: "-1", out: -1},
-		{in: strconv.Itoa(math.MinInt32), out: math.MinInt32},
+		{in: "42", out: 42},
 		{in: strconv.Itoa(math.MaxInt32), out: math.MaxInt32},
 	} {
 
@@ -1088,6 +1087,16 @@ func (s *appsSuite) TestLogsBadN(c *check.C) {
 	c.Assert(rspe.Status, check.Equals, 400)
 }
 
+func (s *appsSuite) TestLogsNegativeN(c *check.C) {
+	s.expectLogsAccess()
+	// -1 is allowed and is tested in TestLogsN.
+	req, err := http.NewRequest("GET", "/v2/logs?n=-2", nil)
+	c.Assert(err, check.IsNil)
+
+	rspe := s.errorReq(c, req, nil, actionIsExpected)
+	c.Assert(rspe.Status, check.Equals, 400)
+}
+
 func (s *appsSuite) TestLogsFollow(c *check.C) {
 	s.expectLogsAccess()
 
