@@ -30,6 +30,7 @@ import (
 	"github.com/snapcore/snapd/boot"
 	"github.com/snapcore/snapd/dirs"
 	"github.com/snapcore/snapd/gadget"
+	"github.com/snapcore/snapd/gadget/device"
 	"github.com/snapcore/snapd/gadget/quantity"
 	"github.com/snapcore/snapd/kernel"
 	"github.com/snapcore/snapd/logger"
@@ -99,10 +100,15 @@ func saveStorageTraits(mod gadget.Model, allVols map[string]*gadget.Volume, opts
 	return nil
 }
 
-func maybeEncryptPartition(dgpair *gadget.OnDiskAndGadgetStructurePair, encryptionType secboot.EncryptionType, sectorSize quantity.Size, perfTimings timings.Measurer) (fsParams *mkfsParams, encryptionKey keys.EncryptionKey, err error) {
+func maybeEncryptPartition(
+	dgpair *gadget.OnDiskAndGadgetStructurePair,
+	encryptionType device.EncryptionType,
+	sectorSize quantity.Size,
+	perfTimings timings.Measurer,
+) (fsParams *mkfsParams, diskEncryptionKey secboot.DiskUnlockKey, err error) {
 	diskPart := dgpair.DiskStructure
 	volStruct := dgpair.GadgetStructure
-	mustEncrypt := (encryptionType != secboot.EncryptionTypeNone)
+	mustEncrypt := (encryptionType != device.EncryptionTypeNone)
 	// fsParams.Device is the kernel device that carries the
 	// filesystem, which is either the raw /dev/<partition>, or
 	// the mapped LUKS device if the structure is encrypted (if
@@ -121,10 +127,12 @@ func maybeEncryptPartition(dgpair *gadget.OnDiskAndGadgetStructurePair, encrypti
 		timings.Run(perfTimings, fmt.Sprintf("make-key-set[%s]", volStruct.Role),
 			fmt.Sprintf("Create encryption key set for %s", volStruct.Role),
 			func(timings.Measurer) {
-				encryptionKey, err = keys.NewEncryptionKey()
-				if err != nil {
-					err = fmt.Errorf("cannot create encryption key: %v", err)
+				encryptionKey, errk := keys.NewEncryptionKey()
+				if errk != nil {
+					err = fmt.Errorf("cannot create encryption key: %v", errk)
+					return
 				}
+				diskEncryptionKey = secboot.DiskUnlockKey(encryptionKey)
 			})
 		if err != nil {
 			return nil, nil, err
@@ -132,11 +140,11 @@ func maybeEncryptPartition(dgpair *gadget.OnDiskAndGadgetStructurePair, encrypti
 		logger.Noticef("encrypting partition device %v", diskPart.Node)
 		var dataPart encryptedDevice
 		switch encryptionType {
-		case secboot.EncryptionTypeLUKS, secboot.EncryptionTypeLUKSWithICE:
+		case device.EncryptionTypeLUKS, device.EncryptionTypeLUKSWithICE:
 			timings.Run(perfTimings, fmt.Sprintf("new-encrypted-device[%s] (%v)", volStruct.Role, encryptionType),
 				fmt.Sprintf("Create encryption device for %s (%s)", volStruct.Role, encryptionType),
 				func(timings.Measurer) {
-					dataPart, err = newEncryptedDeviceLUKS(diskPart.Node, encryptionType, encryptionKey, volStruct.Label, volStruct.Name)
+					dataPart, err = newEncryptedDeviceLUKS(diskPart.Node, encryptionType, diskEncryptionKey, volStruct.Label, volStruct.Name)
 					// TODO close device???
 				})
 			if err != nil {
@@ -157,11 +165,11 @@ func maybeEncryptPartition(dgpair *gadget.OnDiskAndGadgetStructurePair, encrypti
 		fsParams.SectorSize = quantity.Size(fsSectorSizeInt)
 	}
 
-	return fsParams, encryptionKey, nil
+	return fsParams, diskEncryptionKey, nil
 }
 
 // TODO probably we won't need to pass partDisp when we include storage in laidOut
-func createFilesystem(part *gadget.OnDiskStructure, fsParams *mkfsParams, partDisp string, perfTimings timings.Measurer) error {
+func createFilesystem(fsParams *mkfsParams, partDisp string, perfTimings timings.Measurer) error {
 	var err error
 	timings.Run(perfTimings, fmt.Sprintf("make-filesystem[%s]", partDisp),
 		fmt.Sprintf("Create filesystem for %s", fsParams.Device),
@@ -175,12 +183,12 @@ func createFilesystem(part *gadget.OnDiskStructure, fsParams *mkfsParams, partDi
 }
 
 // TODO probably we won't need to pass partDisp when we include storage in laidOut
-func writePartitionContent(laidOut *gadget.LaidOutStructure, fsDevice string, observer gadget.ContentObserver, partDisp string, perfTimings timings.Measurer) error {
+func writePartitionContent(laidOut *gadget.LaidOutStructure, kSnapInfo *KernelSnapInfo, fsDevice string, observer gadget.ContentObserver, partDisp string, perfTimings timings.Measurer) error {
 	var err error
 	timings.Run(perfTimings, fmt.Sprintf("write-content[%s]", partDisp),
 		fmt.Sprintf("Write content for %s", partDisp),
 		func(timings.Measurer) {
-			err = writeFilesystemContent(laidOut, fsDevice, observer)
+			err = writeFilesystemContent(laidOut, kSnapInfo, fsDevice, observer)
 		})
 	if err != nil {
 		return err
@@ -188,9 +196,12 @@ func writePartitionContent(laidOut *gadget.LaidOutStructure, fsDevice string, ob
 	return nil
 }
 
-func installOnePartition(dgpair *gadget.OnDiskAndGadgetStructurePair, kernelInfo *kernel.Info, gadgetRoot, kernelRoot string, encryptionType secboot.EncryptionType, sectorSize quantity.Size, observer gadget.ContentObserver, perfTimings timings.Measurer) (fsDevice string, encryptionKey keys.EncryptionKey, err error) {
+func installOnePartition(dgpair *gadget.OnDiskAndGadgetStructurePair,
+	kernelInfo *kernel.Info, kernelSnapInfo *KernelSnapInfo, gadgetRoot string,
+	encryptionType device.EncryptionType, sectorSize quantity.Size,
+	observer gadget.ContentObserver, perfTimings timings.Measurer,
+) (fsDevice string, encryptionKey secboot.DiskUnlockKey, err error) {
 	// 1. Encrypt
-	diskPart := dgpair.DiskStructure
 	vs := dgpair.GadgetStructure
 	role := vs.Role
 	fsParams, encryptionKey, err := maybeEncryptPartition(dgpair, encryptionType, sectorSize, perfTimings)
@@ -200,21 +211,21 @@ func installOnePartition(dgpair *gadget.OnDiskAndGadgetStructurePair, kernelInfo
 	fsDevice = fsParams.Device
 
 	// 2. Create filesystem
-	if err := createFilesystem(diskPart, fsParams, role, perfTimings); err != nil {
+	if err := createFilesystem(fsParams, role, perfTimings); err != nil {
 		return "", nil, err
 	}
 
 	// 3. Write content
 	opts := &gadget.LayoutOptions{
 		GadgetRootDir: gadgetRoot,
-		KernelRootDir: kernelRoot,
+		KernelRootDir: kernelSnapInfo.MountPoint,
 		EncType:       encryptionType,
 	}
 	los, err := gadget.LayoutVolumeStructure(dgpair, kernelInfo, opts)
 	if err != nil {
 		return "", nil, err
 	}
-	if err := writePartitionContent(los, fsDevice, observer, role, perfTimings); err != nil {
+	if err := writePartitionContent(los, kernelSnapInfo, fsDevice, observer, role, perfTimings); err != nil {
 		return "", nil, err
 	}
 
@@ -227,6 +238,11 @@ func resolveBootDevice(bootDevice string, bootVol *gadget.Volume) (string, error
 	if bootDevice != "" {
 		return bootDevice, nil
 	}
+
+	// XXX: It makes no sense to handle bootVol.DeviceAssignment here and
+	// force the assignment there, as current constraints dictate the boot
+	// device must be on the same disk as ubuntu-seed. Therefore we should
+	// just ensure that the two disk paths don't differ if assigned
 	foundDisk, err := disks.DiskFromMountPoint("/run/mnt/ubuntu-seed", nil)
 	if err != nil {
 		logger.Noticef("Warning: cannot find disk from mounted seed: %s", err)
@@ -237,7 +253,19 @@ func resolveBootDevice(bootDevice string, bootVol *gadget.Volume) (string, error
 	if err != nil {
 		return "", fmt.Errorf("cannot find device to create partitions on: %v", err)
 	}
-
+	if bootVol.AssignedDevice != "" {
+		// disk is assigned for this volume, then it must match the one
+		// that is assigned to system-seed
+		bootDisk, err := disks.DiskFromDeviceName(bootVol.AssignedDevice)
+		if err != nil {
+			return "", err
+		}
+		logger.Noticef("volume %s has been assigned disk %s", bootVol.Name, bootDisk.KernelDeviceNode())
+		if bootDevice != bootDisk.KernelDeviceNode() {
+			return "", fmt.Errorf("volume %s was assigned disk %s, but this does not match the disk for ubuntu-seed %s",
+				bootVol.Name, bootDisk.KernelDeviceNode(), bootDevice)
+		}
+	}
 	return bootDevice, nil
 }
 
@@ -245,11 +273,10 @@ func resolveBootDevice(bootDevice string, bootVol *gadget.Volume) (string, error
 // volume name where partitions have been created, the on disk
 // structures after that, the laidout volumes, and the disk sector
 // size.
-func createPartitions(model gadget.Model, info *gadget.Info, gadgetRoot, kernelRoot, bootDevice string, options Options,
-	perfTimings timings.Measurer) (
+func createPartitions(volumes map[string]*gadget.Volume, gadgetRoot, bootDevice string, perfTimings timings.Measurer) (
 	bootVolGadgetName string, created []*gadget.OnDiskAndGadgetStructurePair, bootVolSectorSize quantity.Size, err error) {
 	// Find boot volume
-	bootVol, err := gadget.FindBootVolume(info.Volumes)
+	bootVol, err := gadget.FindBootVolume(volumes)
 	if err != nil {
 		return "", nil, 0, err
 	}
@@ -271,7 +298,12 @@ func createPartitions(model gadget.Model, info *gadget.Info, gadgetRoot, kernelR
 	}
 
 	// remove partitions added during a previous install attempt
-	if err := removeCreatedPartitions(gadgetRoot, bootVol, diskVolume); err != nil {
+	// TODO we probably do not need to do this, as we are re-creating the
+	// partitions with the same sizes as the ones removed. We even check
+	// that labels are the same in gadget.EnsureVolumeCompatibility, so no
+	// data for the partition will actually change.
+	deletedOffsetSize, err := removeCreatedPartitions(gadgetRoot, bootVol, diskVolume)
+	if err != nil {
 		return "", nil, 0, fmt.Errorf("cannot remove partitions from previous install: %v", err)
 	}
 	// at this point we removed any existing partition, nuke any
@@ -288,7 +320,7 @@ func createPartitions(model gadget.Model, info *gadget.Info, gadgetRoot, kernelR
 		opts := &CreateOptions{
 			GadgetRootDir: gadgetRoot,
 		}
-		created, err = createMissingPartitions(diskVolume, bootVol, opts)
+		created, err = createMissingPartitions(diskVolume, bootVol, opts, deletedOffsetSize)
 	})
 	if err != nil {
 		return "", nil, 0, fmt.Errorf("cannot create the partitions: %v", err)
@@ -299,9 +331,9 @@ func createPartitions(model gadget.Model, info *gadget.Info, gadgetRoot, kernelR
 	return bootVolGadgetName, created, bootVolSectorSize, nil
 }
 
-func createEncryptionParams(encTyp secboot.EncryptionType) gadget.StructureEncryptionParameters {
+func createEncryptionParams(encTyp device.EncryptionType) gadget.StructureEncryptionParameters {
 	switch encTyp {
-	case secboot.EncryptionTypeLUKS, secboot.EncryptionTypeLUKSWithICE:
+	case device.EncryptionTypeLUKS, device.EncryptionTypeLUKSWithICE:
 		return gadget.StructureEncryptionParameters{
 			// TODO:ICE: remove "Method" entirely, there is only LUKS
 			Method: gadget.EncryptionLUKS,
@@ -322,7 +354,7 @@ func onDiskStructsSortedIdx(vss map[int]*gadget.OnDiskStructure) []int {
 
 // Run creates partitions, encrypts them when expected, creates
 // filesystems, and finally writes content on them.
-func Run(model gadget.Model, gadgetRoot, kernelRoot, bootDevice string, options Options, observer gadget.ContentObserver, perfTimings timings.Measurer) (*InstalledSystemSideData, error) {
+func Run(model gadget.Model, gadgetRoot string, kernelSnapInfo *KernelSnapInfo, bootDevice string, options Options, observer gadget.ContentObserver, perfTimings timings.Measurer) (*InstalledSystemSideData, error) {
 	logger.Noticef("installing a new system")
 	logger.Noticef("        gadget data from: %v", gadgetRoot)
 	logger.Noticef("        encryption: %v", options.EncryptionType)
@@ -339,15 +371,29 @@ func Run(model gadget.Model, gadgetRoot, kernelRoot, bootDevice string, options
 		return nil, err
 	}
 
+	volumes, explicitAssignment, err := gadget.VolumesForCurrentDevice(info)
+	if err != nil {
+		return nil, err
+	}
+	if explicitAssignment {
+		// in case of volume assignments let us list those for
+		// information
+		logger.Noticef("volume assignments:")
+		for name, vol := range volumes {
+			logger.Noticef("        %s => %s", name, vol.AssignedDevice)
+		}
+	}
+
 	// Step 1: create partitions
-	bootVolGadgetName, created, bootVolSectorSize, err :=
-		createPartitions(model, info, gadgetRoot, kernelRoot, bootDevice, options, perfTimings)
+	kernelRoot := kernelSnapInfo.MountPoint
+	bootVolumeName, created, bootVolSectorSize, err :=
+		createPartitions(volumes, gadgetRoot, bootDevice, perfTimings)
 	if err != nil {
 		return nil, err
 	}
 
 	// Step 2: layout content in the created partitions
-	var keyForRole map[string]keys.EncryptionKey
+	var installKeyForRole map[string]secboot.BootstrappedContainer
 	devicesForRoles := map[string]string{}
 	partsEncrypted := map[string]gadget.StructureEncryptionParameters{}
 	kernelInfo, err := kernel.ReadInfo(kernelRoot)
@@ -379,23 +425,23 @@ func Run(model gadget.Model, gadgetRoot, kernelRoot, bootDevice string, options
 		// for encrypted device the filesystem device it will point to
 		// the mapper device otherwise it's the raw device node
 		fsDevice, encryptionKey, err := installOnePartition(dgpair,
-			kernelInfo, gadgetRoot, kernelRoot, options.EncryptionType,
+			kernelInfo, kernelSnapInfo, gadgetRoot, options.EncryptionType,
 			bootVolSectorSize, observer, perfTimings)
 		if err != nil {
 			return nil, err
 		}
 
 		if encryptionKey != nil {
-			if keyForRole == nil {
-				keyForRole = map[string]keys.EncryptionKey{}
+			if installKeyForRole == nil {
+				installKeyForRole = map[string]secboot.BootstrappedContainer{}
 			}
-			keyForRole[vs.Role] = encryptionKey
+			installKeyForRole[vs.Role] = secboot.CreateBootstrappedContainer(encryptionKey, diskPart.Node)
 			partsEncrypted[vs.Name] = createEncryptionParams(options.EncryptionType)
 		}
 		if options.Mount && vs.Label != "" && vs.HasFilesystem() {
 			// fs is taken from gadget, as on disk one might be displayed as
 			// crypto_LUKS, which is not useful for formatting.
-			if err := mountFilesystem(fsDevice, vs.LinuxFilesystem(), getMntPointForPart(vs)); err != nil {
+			if err := mountFilesystem(fsDevice, vs.LinuxFilesystem(), getMntPointForPart(vs), mntParamsForPartRole(vs.Role)); err != nil {
 				return nil, err
 			}
 		}
@@ -406,19 +452,19 @@ func Run(model gadget.Model, gadgetRoot, kernelRoot, bootDevice string, options
 	optsPerVol := map[string]*gadget.DiskVolumeValidationOptions{
 		// this assumes that the encrypted partitions above are always only on the
 		// system-boot volume, this assumption may change
-		bootVolGadgetName: {
+		bootVolumeName: {
 			ExpectedStructureEncryption: partsEncrypted,
 		},
 	}
 
 	// save the traits to ubuntu-data host and optionally to ubuntu-save if it exists
-	if err := saveStorageTraits(model, info.Volumes, optsPerVol, hasSavePartition); err != nil {
+	if err := saveStorageTraits(model, volumes, optsPerVol, hasSavePartition); err != nil {
 		return nil, err
 	}
 
 	return &InstalledSystemSideData{
-		KeyForRole:    keyForRole,
-		DeviceForRole: devicesForRoles,
+		BootstrappedContainerForRole: installKeyForRole,
+		DeviceForRole:                devicesForRoles,
 	}, nil
 }
 
@@ -493,7 +539,7 @@ func deviceForMaybeEncryptedVolume(volStruct *gadget.VolumeStructure, encSetupDa
 
 // WriteContent writes gadget content to the devices specified in
 // onVolumes. It returns the resolved on disk volumes.
-func WriteContent(onVolumes map[string]*gadget.Volume, allLaidOutVols map[string]*gadget.LaidOutVolume, encSetupData *EncryptionSetupData, observer gadget.ContentObserver, perfTimings timings.Measurer) ([]*gadget.OnDiskVolume, error) {
+func WriteContent(onVolumes map[string]*gadget.Volume, allLaidOutVols map[string]*gadget.LaidOutVolume, encSetupData *EncryptionSetupData, kSnapInfo *KernelSnapInfo, observer gadget.ContentObserver, perfTimings timings.Measurer) ([]*gadget.OnDiskVolume, error) {
 	// TODO this taking onVolumes and allLaidOutVols is odd,
 	// we should try to avoid this when we have partial
 
@@ -529,7 +575,7 @@ func WriteContent(onVolumes map[string]*gadget.Volume, allLaidOutVols map[string
 			device := deviceForMaybeEncryptedVolume(&volStruct, encSetupData)
 			logger.Debugf("writing content on partition %s", device)
 			partDisp := roleOrLabelOrName(laidOut.Role(), &laidOut.OnDiskStructure)
-			if err := writePartitionContent(laidOut, device, observer, partDisp, perfTimings); err != nil {
+			if err := writePartitionContent(laidOut, kSnapInfo, device, observer, partDisp, perfTimings); err != nil {
 				return nil, err
 			}
 		}
@@ -538,6 +584,18 @@ func WriteContent(onVolumes map[string]*gadget.Volume, allLaidOutVols map[string
 	return onDiskVols, nil
 }
 
+// mntParamsForPartRole decides mount flags for a given structure role.
+func mntParamsForPartRole(role string) (mntParams mntfsParams) {
+	var p mntfsParams
+	switch role {
+	case gadget.SystemSeed, gadget.SystemSave:
+		p.NoDev = true
+		p.NoExec = true
+		p.NoSuid = true
+	}
+	return p
+}
+
 // getMntPointForPart tells us where to mount a given structure so we
 // match what the functions that write something expect.
 func getMntPointForPart(part *gadget.VolumeStructure) (mntPt string) {
@@ -587,7 +645,7 @@ func MountVolumes(onVolumes map[string]*gadget.Volume, encSetupData *EncryptionS
 			// Device might have been encrypted
 			device := deviceForMaybeEncryptedVolume(&part, encSetupData)
 
-			if err := mountFilesystem(device, part.LinuxFilesystem(), mntPt); err != nil {
+			if err := mountFilesystem(device, part.LinuxFilesystem(), mntPt, mntParamsForPartRole(part.Role)); err != nil {
 				defer unmount()
 				return "", nil, fmt.Errorf("cannot mount %q at %q: %v", device, mntPt, err)
 			}
@@ -622,9 +680,20 @@ func SaveStorageTraits(model gadget.Model, vols map[string]*gadget.Volume, encry
 	return nil
 }
 
-func EncryptPartitions(onVolumes map[string]*gadget.Volume, encryptionType secboot.EncryptionType, model *asserts.Model, gadgetRoot, kernelRoot string, perfTimings timings.Measurer) (*EncryptionSetupData, error) {
+func EncryptPartitions(
+	onVolumes map[string]*gadget.Volume,
+	volumesAuth *device.VolumesAuthOptions,
+	encryptionType device.EncryptionType,
+	checkContext *secboot.PreinstallCheckContext,
+	model *asserts.Model,
+	gadgetRoot,
+	kernelRoot string,
+	perfTimings timings.Measurer,
+) (*EncryptionSetupData, error) {
 	setupData := &EncryptionSetupData{
-		parts: make(map[string]partEncryptionData),
+		parts:                  make(map[string]partEncryptionData),
+		volumesAuth:            volumesAuth,
+		preinstallCheckContext: checkContext,
 	}
 	for volName, vol := range onVolumes {
 		onDiskVol, err := gadget.OnDiskVolumeFromGadgetVol(vol)
@@ -662,7 +731,7 @@ func EncryptPartitions(onVolumes map[string]*gadget.Volume, encryptionType secbo
 				// EncryptedDevice will be /dev/mapper/ubuntu-data, etc.
 				encryptedDevice:     fsParams.Device,
 				volName:             volName,
-				encryptionKey:       encryptionKey,
+				installKey:          secboot.CreateBootstrappedContainer(encryptionKey, device),
 				encryptedSectorSize: fsParams.SectorSize,
 				encryptionParams:    createEncryptionParams(encryptionType),
 			}
@@ -671,15 +740,15 @@ func EncryptPartitions(onVolumes map[string]*gadget.Volume, encryptionType secbo
 	return setupData, nil
 }
 
-func KeysForRole(setupData *EncryptionSetupData) map[string]keys.EncryptionKey {
-	keyForRole := make(map[string]keys.EncryptionKey)
+func BootstrappedContainersForRole(setupData *EncryptionSetupData) map[string]secboot.BootstrappedContainer {
+	installKeyForRole := make(map[string]secboot.BootstrappedContainer)
 	for _, p := range setupData.parts {
-		keyForRole[p.role] = p.encryptionKey
+		installKeyForRole[p.role] = p.installKey
 	}
-	return keyForRole
+	return installKeyForRole
 }
 
-func FactoryReset(model gadget.Model, gadgetRoot, kernelRoot, bootDevice string, options Options, observer gadget.ContentObserver, perfTimings timings.Measurer) (*InstalledSystemSideData, error) {
+func FactoryReset(model gadget.Model, gadgetRoot string, kernelSnapInfo *KernelSnapInfo, bootDevice string, options Options, observer gadget.ContentObserver, perfTimings timings.Measurer) (*InstalledSystemSideData, error) {
 	logger.Noticef("performing factory reset on an installed system")
 	logger.Noticef("        gadget data from: %v", gadgetRoot)
 	logger.Noticef("        encryption: %v", options.EncryptionType)
@@ -696,10 +765,25 @@ func FactoryReset(model gadget.Model, gadgetRoot, kernelRoot, bootDevice string,
 	if err != nil {
 		return nil, err
 	}
-	bootVol, err := gadget.FindBootVolume(info.Volumes)
+
+	volumes, explicitAssignment, err := gadget.VolumesForCurrentDevice(info)
 	if err != nil {
 		return nil, err
 	}
+	if explicitAssignment {
+		// in case of volume assignments let us list those for
+		// information
+		logger.Noticef("volume assignments:")
+		for name, vol := range volumes {
+			logger.Noticef("        %s => %s", name, vol.AssignedDevice)
+		}
+	}
+
+	bootVol, err := gadget.FindBootVolume(volumes)
+	if err != nil {
+		return nil, err
+	}
+
 	bootDevice, err = resolveBootDevice(bootDevice, bootVol)
 	if err != nil {
 		return nil, err
@@ -714,10 +798,10 @@ func FactoryReset(model gadget.Model, gadgetRoot, kernelRoot, bootDevice string,
 		AssumeCreatablePartitionsCreated: true,
 		ExpectedStructureEncryption:      map[string]gadget.StructureEncryptionParameters{},
 	}
-	if options.EncryptionType != secboot.EncryptionTypeNone {
+	if options.EncryptionType != device.EncryptionTypeNone {
 		var encryptionParam gadget.StructureEncryptionParameters
 		switch options.EncryptionType {
-		case secboot.EncryptionTypeLUKS, secboot.EncryptionTypeLUKSWithICE:
+		case device.EncryptionTypeLUKS, device.EncryptionTypeLUKSWithICE:
 			encryptionParam = gadget.StructureEncryptionParameters{Method: gadget.EncryptionLUKS}
 		default:
 			// XXX what about ICE?
@@ -737,11 +821,11 @@ func FactoryReset(model gadget.Model, gadgetRoot, kernelRoot, bootDevice string,
 		return nil, fmt.Errorf("gadget and system-boot device %v partition table not compatible: %v", bootDevice, err)
 	}
 
-	kernelInfo, err := kernel.ReadInfo(kernelRoot)
+	kernelInfo, err := kernel.ReadInfo(kernelSnapInfo.MountPoint)
 	if err != nil {
 		return nil, err
 	}
-	var keyForRole map[string]keys.EncryptionKey
+	var installKeyForRole map[string]secboot.BootstrappedContainer
 	deviceForRole := map[string]string{}
 	var hasSavePartition bool
 	rolesToReset := []string{gadget.SystemBoot, gadget.SystemData}
@@ -769,21 +853,21 @@ func FactoryReset(model gadget.Model, gadgetRoot, kernelRoot, bootDevice string,
 		fsDevice, encryptionKey, err := installOnePartition(
 			&gadget.OnDiskAndGadgetStructurePair{
 				DiskStructure: onDiskStruct, GadgetStructure: vs},
-			kernelInfo, gadgetRoot, kernelRoot, options.EncryptionType,
+			kernelInfo, kernelSnapInfo, gadgetRoot, options.EncryptionType,
 			diskLayout.SectorSize, observer, perfTimings)
 		if err != nil {
 			return nil, err
 		}
 		if encryptionKey != nil {
-			if keyForRole == nil {
-				keyForRole = map[string]keys.EncryptionKey{}
+			if installKeyForRole == nil {
+				installKeyForRole = map[string]secboot.BootstrappedContainer{}
 			}
-			keyForRole[vs.Role] = encryptionKey
+			installKeyForRole[vs.Role] = secboot.CreateBootstrappedContainer(encryptionKey, onDiskStruct.Node)
 		}
 		if options.Mount && vs.Label != "" && vs.HasFilesystem() {
 			// fs is taken from gadget, as on disk one might be displayed as
 			// crypto_LUKS, which is not useful for formatting.
-			if err := mountFilesystem(fsDevice, vs.LinuxFilesystem(), getMntPointForPart(vs)); err != nil {
+			if err := mountFilesystem(fsDevice, vs.LinuxFilesystem(), getMntPointForPart(vs), mntParamsForPartRole(vs.Role)); err != nil {
 				return nil, err
 			}
 		}
@@ -798,14 +882,15 @@ func FactoryReset(model gadget.Model, gadgetRoot, kernelRoot, bootDevice string,
 			ExpectedStructureEncryption: volCompatOps.ExpectedStructureEncryption,
 		},
 	}
+
 	// save the traits to ubuntu-data host and optionally to ubuntu-save if it exists
-	if err := saveStorageTraits(model, info.Volumes, optsPerVol, hasSavePartition); err != nil {
+	if err := saveStorageTraits(model, volumes, optsPerVol, hasSavePartition); err != nil {
 		return nil, err
 	}
 
 	return &InstalledSystemSideData{
-		KeyForRole:    keyForRole,
-		DeviceForRole: deviceForRole,
+		BootstrappedContainerForRole: installKeyForRole,
+		DeviceForRole:                deviceForRole,
 	}, nil
 }
 
