@@ -210,7 +210,7 @@ func (s *partitionTestSuite) TestBuildPartitionList(c *C) {
 
 	// the expected expanded writable partition size is:
 	// start offset = (2M + 1200M), expanded size in sectors = (8388575*512 - start offset)/512
-	sfdiskInput, create, err := install.BuildPartitionList(dl, pv.Volume, nil)
+	sfdiskInput, create, err := install.BuildPartitionList(dl, pv.Volume, nil, nil)
 	c.Assert(err, IsNil)
 	c.Assert(sfdiskInput.String(), Equals,
 		`/dev/node3 : start=     2461696, size=      262144, type=0FC63DAF-8483-4772-8E79-3D69D8477DE4, name="Save"
@@ -250,7 +250,7 @@ func (s *partitionTestSuite) TestBuildPartitionListPartsNotInGadget(c *C) {
 	// offset = (2M + 1200M), expanded size in sectors =
 	// (8388575*512 - start offset)/512
 	sfdiskInput, create, err := install.BuildPartitionList(dl, pv.Volume,
-		&install.CreateOptions{})
+		&install.CreateOptions{}, nil)
 	c.Assert(err, IsNil)
 	c.Assert(sfdiskInput.String(), Equals,
 		`/dev/node3 : start=     2461696, size=      262144, type=0FC63DAF-8483-4772-8E79-3D69D8477DE4, name="Save"
@@ -291,7 +291,7 @@ func (s *partitionTestSuite) TestBuildPartitionListOnlyCreatablePartitions(c *C)
 	dl, err := gadget.OnDiskVolumeFromDevice("/dev/node")
 	c.Assert(err, IsNil)
 
-	_, _, err = install.BuildPartitionList(dl, pv.Volume, nil)
+	_, _, err = install.BuildPartitionList(dl, pv.Volume, nil, nil)
 	c.Assert(err, ErrorMatches, `gadget and boot device /dev/node partition table not compatible: cannot find gadget structure "BIOS Boot" on disk`)
 }
 
@@ -317,7 +317,7 @@ func (s *partitionTestSuite) TestBuildPartitionListExistingPartsInSizeRange(c *C
 
 	// the expected expanded writable partition size is:
 	// start offset = (2M + 1200M), expanded size in sectors = (8388575*512 - start offset)/512
-	sfdiskInput, create, err := install.BuildPartitionList(dl, pv.Volume, nil)
+	sfdiskInput, create, err := install.BuildPartitionList(dl, pv.Volume, nil, nil)
 	c.Assert(err, IsNil)
 	c.Assert(sfdiskInput.String(), Equals,
 		`/dev/node3 : start=     2461696, size=      262144, type=0FC63DAF-8483-4772-8E79-3D69D8477DE4, name="Save"
@@ -336,6 +336,17 @@ func (s *partitionTestSuite) TestBuildPartitionListExistingPartsInSizeRange(c *C
 	})
 }
 
+func (s *partitionTestSuite) TestBuildPartitionListEMMCIsEmptyButNoError(c *C) {
+	sfdiskInput, create, err := install.BuildPartitionList(&gadget.OnDiskVolume{
+		SectorSize: 512,
+	}, &gadget.Volume{
+		Schema: "emmc",
+	}, nil, nil)
+	c.Assert(err, IsNil)
+	c.Assert(sfdiskInput, IsNil)
+	c.Assert(create, IsNil)
+}
+
 func (s *partitionTestSuite) TestCreatePartitions(c *C) {
 	cmdSfdisk := testutil.MockCommand(c, "sfdisk", "")
 	defer cmdSfdisk.Restore()
@@ -369,7 +380,7 @@ func (s *partitionTestSuite) TestCreatePartitions(c *C) {
 	opts := &install.CreateOptions{
 		GadgetRootDir: s.gadgetRoot,
 	}
-	created, err := install.TestCreateMissingPartitions(dl, pv.Volume, opts)
+	created, err := install.TestCreateMissingPartitions(dl, pv.Volume, opts, nil)
 	c.Assert(err, IsNil)
 	c.Assert(created, DeepEquals, []*gadget.OnDiskAndGadgetStructurePair{
 		{
@@ -430,7 +441,7 @@ func (s *partitionTestSuite) TestCreatePartitionsNonRolePartitions(c *C) {
 		GadgetRootDir:              s.gadgetRoot,
 		CreateAllMissingPartitions: true,
 	}
-	created, err := install.TestCreateMissingPartitions(dl, pv.Volume, opts)
+	created, err := install.TestCreateMissingPartitions(dl, pv.Volume, opts, nil)
 	c.Assert(err, IsNil)
 	c.Assert(created, HasLen, 3)
 	c.Assert(calls, Equals, 1)
@@ -453,8 +464,9 @@ func (s *partitionTestSuite) TestRemovePartitionsTrivial(c *C) {
 	dl, err := gadget.OnDiskVolumeFromDevice("/dev/node")
 	c.Assert(err, IsNil)
 
-	err = install.RemoveCreatedPartitions(s.gadgetRoot, gInfo.Volumes["pc"], dl)
+	deletedOffsetSize, err := install.RemoveCreatedPartitions(s.gadgetRoot, gInfo.Volumes["pc"], dl)
 	c.Assert(err, IsNil)
+	c.Assert(deletedOffsetSize, DeepEquals, map[int]install.StructOffsetSize{})
 }
 
 func (s *partitionTestSuite) TestRemovePartitions(c *C) {
@@ -530,8 +542,10 @@ func (s *partitionTestSuite) TestRemovePartitions(c *C) {
 	gInfo, err := gadget.ReadInfoAndValidate(s.gadgetRoot, uc20Mod, nil)
 	c.Assert(err, IsNil)
 
-	err = install.RemoveCreatedPartitions(s.gadgetRoot, gInfo.Volumes["pc"], dl)
+	deletedOffsetSize, err := install.RemoveCreatedPartitions(s.gadgetRoot, gInfo.Volumes["pc"], dl)
 	c.Assert(err, IsNil)
+	c.Assert(deletedOffsetSize, DeepEquals, map[int]install.StructOffsetSize{
+		3: {StartOffset: 1260388352, Size: 2457600 * 512}})
 
 	c.Assert(cmdSfdisk.Calls(), DeepEquals, [][]string{
 		{"sfdisk", "--no-reread", "--delete", "/dev/node", "3"},
@@ -652,8 +666,10 @@ func (s *partitionTestSuite) TestRemovePartitionsWithDeviceRescan(c *C) {
 	gInfo, err := gadget.ReadInfoAndValidate(s.gadgetRoot, uc20Mod, nil)
 	c.Assert(err, IsNil)
 
-	err = install.RemoveCreatedPartitions(s.gadgetRoot, gInfo.Volumes["pc"], dl)
+	deletedOffsetSize, err := install.RemoveCreatedPartitions(s.gadgetRoot, gInfo.Volumes["pc"], dl)
 	c.Assert(err, IsNil)
+	c.Assert(deletedOffsetSize, DeepEquals, map[int]install.StructOffsetSize{
+		3: {StartOffset: 1260388352, Size: 2457600 * 512}})
 
 	c.Assert(cmdSfdisk.Calls(), DeepEquals, [][]string{
 		{"sfdisk", "--no-reread", "--delete", "/dev/node", "3"},
@@ -793,8 +809,10 @@ func (s *partitionTestSuite) TestRemovePartitionsNonAdjacent(c *C) {
 	gInfo, err := gadget.ReadInfoAndValidate(s.gadgetRoot, uc20Mod, nil)
 	c.Assert(err, IsNil)
 
-	err = install.RemoveCreatedPartitions(s.gadgetRoot, gInfo.Volumes["pc"], dl)
+	deletedOffsetSize, err := install.RemoveCreatedPartitions(s.gadgetRoot, gInfo.Volumes["pc"], dl)
 	c.Assert(err, IsNil)
+	c.Assert(deletedOffsetSize, DeepEquals, map[int]install.StructOffsetSize{
+		2: {StartOffset: 1024*1024 + 1024*1024, Size: 2457600 * 512}})
 
 	c.Assert(cmdSfdisk.Calls(), DeepEquals, [][]string{
 		{"sfdisk", "--no-reread", "--delete", "/dev/node", "2"},
@@ -1005,6 +1023,22 @@ const gptGadgetContentWithRangeForSeed = `volumes:
         size: 1200M
 `
 
+// createdDuringInstall returns a list of partitions created during the
+// install process.
+func createdDuringInstall(gv *gadget.Volume, layout *gadget.OnDiskVolume) (created []string, yamlIdxes []int) {
+	created = make([]string, 0, len(layout.Structure))
+	startFromIdx := 0
+	for _, s := range layout.Structure {
+		gadgetIndexes := install.IndexIfCreatedDuringInstall(gv, s, startFromIdx)
+		if gadgetIndexes.YamlIdx >= 0 {
+			yamlIdxes = append(yamlIdxes, gadgetIndexes.YamlIdx)
+			startFromIdx = gadgetIndexes.OrderIdx + 1
+			created = append(created, s.Node)
+		}
+	}
+	return created, yamlIdxes
+}
+
 func (s *partitionTestSuite) TestCreatedDuringInstallGPT(c *C) {
 	m := map[string]*disks.MockDiskMapping{
 		"node": {
@@ -1084,9 +1118,10 @@ func (s *partitionTestSuite) TestCreatedDuringInstallGPT(c *C) {
 	dl, err := gadget.OnDiskVolumeFromDevice("node")
 	c.Assert(err, IsNil)
 
-	list := install.CreatedDuringInstall(pv.Volume, dl)
+	list, indexes := createdDuringInstall(pv.Volume, dl)
 	// only save and writable should show up
 	c.Check(list, DeepEquals, []string{"/dev/node3", "/dev/node4"})
+	c.Check(indexes, DeepEquals, []int{3, 4})
 
 	// min-size for ubuntu-save for this gadget will match the third partition size
 	// (but size wouldn't)
@@ -1098,9 +1133,10 @@ func (s *partitionTestSuite) TestCreatedDuringInstallGPT(c *C) {
 	dl, err = gadget.OnDiskVolumeFromDevice("node")
 	c.Assert(err, IsNil)
 
-	list = install.CreatedDuringInstall(pv.Volume, dl)
+	list, indexes = createdDuringInstall(pv.Volume, dl)
 	// only save and writable should show up
 	c.Check(list, DeepEquals, []string{"/dev/node3", "/dev/node4"})
+	c.Check(indexes, DeepEquals, []int{3, 4})
 }
 
 // this is an mbr gadget like the pi, but doesn't have the amd64 mbr structure
@@ -1220,6 +1256,206 @@ func (s *partitionTestSuite) TestCreatedDuringInstallMBR(c *C) {
 	pv, err := gadgettest.MustLayOutSingleVolumeFromGadget(s.gadgetRoot, "", uc20Mod)
 	c.Assert(err, IsNil)
 
-	list := install.CreatedDuringInstall(pv.Volume, dl)
+	list, indexes := createdDuringInstall(pv.Volume, dl)
 	c.Assert(list, DeepEquals, []string{"/dev/node2", "/dev/node3", "/dev/node4"})
+	c.Check(indexes, DeepEquals, []int{1, 2, 3})
+}
+
+const gptGadgetWithMinSize = `volumes:
+  pc:
+    schema: gpt
+    bootloader: grub
+    structure:
+      - name: Recovery
+        role: system-seed
+        filesystem: vfat
+        type: EF,C12A7328-F81F-11D2-BA4B-00A0C93EC93B
+        offset: 1M
+        min-size: 99
+        size: 999M
+      - name: Save
+        role: system-save
+        filesystem: ext4
+        type: 83,0FC63DAF-8483-4772-8E79-3D69D8477DE4
+        min-size: 100M
+        size: 1000M
+      - name: Writable
+        role: system-data
+        filesystem: ext4
+        type: 83,0FC63DAF-8483-4772-8E79-3D69D8477DE4
+        min-size: 100M
+        size: 1000M
+`
+
+func (s *partitionTestSuite) TestCreatedDuringInstallOverlaps(c *C) {
+	const oneMeg = 1 * 1024 * 1024
+	m := map[string]*disks.MockDiskMapping{
+		"node": {
+			DevNum:              "42:0",
+			DevNode:             "/dev/node",
+			DiskSizeInBytes:     (8388574 + 34) * 512,
+			DiskUsableSectorEnd: 8388574 + 1,
+			DiskSchema:          "gpt",
+			ID:                  "9151F25B-CDF0-48F1-9EDE-68CBD616E2CA",
+			SectorSizeBytes:     512,
+			Structure: []disks.Partition{
+				{
+					KernelDeviceNode: "/dev/node1",
+					StartInBytes:     oneMeg,
+					SizeInBytes:      399 * oneMeg,
+					PartitionType:    "0a",
+					PartitionLabel:   "Recovery",
+					Major:            42,
+					Minor:            1,
+					DiskIndex:        1,
+					FilesystemType:   "vfat",
+					FilesystemUUID:   "A644-B807",
+					FilesystemLabel:  "ubuntu-seed",
+				},
+				{
+					KernelDeviceNode: "/dev/node2",
+					StartInBytes:     400 * oneMeg,
+					SizeInBytes:      600 * oneMeg,
+					PartitionType:    "c",
+					PartitionLabel:   "Save",
+					Major:            42,
+					Minor:            2,
+					DiskIndex:        2,
+					FilesystemType:   "ext4",
+					FilesystemUUID:   "8781-433a",
+					FilesystemLabel:  "ubuntu-save",
+				},
+				{
+					KernelDeviceNode: "/dev/node3",
+					StartInBytes:     1000 * oneMeg,
+					SizeInBytes:      200 * oneMeg,
+					PartitionType:    "0d",
+					PartitionLabel:   "Data",
+					Major:            42,
+					Minor:            3,
+					DiskIndex:        3,
+					FilesystemType:   "ext4",
+					FilesystemUUID:   "8123-433a",
+					FilesystemLabel:  "ubuntu-data",
+				},
+			},
+		},
+	}
+
+	restore := disks.MockDeviceNameToDiskMapping(m)
+	defer restore()
+
+	dl, err := gadget.OnDiskVolumeFromDevice("node")
+	c.Assert(err, IsNil)
+
+	err = gadgettest.MakeMockGadget(s.gadgetRoot, gptGadgetWithMinSize)
+	c.Assert(err, IsNil)
+	pv, err := gadgettest.MustLayOutSingleVolumeFromGadget(s.gadgetRoot, "", uc20Mod)
+	c.Assert(err, IsNil)
+
+	list, indexes := createdDuringInstall(pv.Volume, dl)
+	c.Assert(list, DeepEquals, []string{"/dev/node2", "/dev/node3"})
+	c.Check(indexes, DeepEquals, []int{1, 2})
+}
+
+func (s *partitionTestSuite) TestRemovePartitionsMinSize(c *C) {
+	const oneMeg = 1 * 1024 * 1024
+	m := map[string]*disks.MockDiskMapping{
+		"/dev/node": {
+			DevNum:              "42:0",
+			DevNode:             "/dev/node",
+			DiskSizeInBytes:     (8388574 + 34) * 512,
+			DiskUsableSectorEnd: 8388574 + 1,
+			DiskSchema:          "gpt",
+			ID:                  "9151F25B-CDF0-48F1-9EDE-68CBD616E2CA",
+			SectorSizeBytes:     512,
+			Structure: []disks.Partition{
+				{
+					KernelDeviceNode: "/dev/node1",
+					StartInBytes:     oneMeg,
+					SizeInBytes:      399 * oneMeg,
+					PartitionType:    "0a",
+					PartitionLabel:   "Recovery",
+					Major:            42,
+					Minor:            1,
+					DiskIndex:        1,
+					FilesystemType:   "vfat",
+					FilesystemUUID:   "A644-B807",
+					FilesystemLabel:  "ubuntu-seed",
+				},
+				{
+					KernelDeviceNode: "/dev/node2",
+					StartInBytes:     400 * oneMeg,
+					SizeInBytes:      600 * oneMeg,
+					PartitionType:    "c",
+					PartitionLabel:   "Save",
+					Major:            42,
+					Minor:            2,
+					DiskIndex:        2,
+					FilesystemType:   "ext4",
+					FilesystemUUID:   "8781-433a",
+					FilesystemLabel:  "ubuntu-save",
+				},
+				{
+					KernelDeviceNode: "/dev/node3",
+					StartInBytes:     1000 * oneMeg,
+					SizeInBytes:      200 * oneMeg,
+					PartitionType:    "0d",
+					PartitionLabel:   "Data",
+					Major:            42,
+					Minor:            3,
+					DiskIndex:        3,
+					FilesystemType:   "ext4",
+					FilesystemUUID:   "8123-433a",
+					FilesystemLabel:  "ubuntu-data",
+				},
+			},
+		},
+	}
+
+	restore := disks.MockDeviceNameToDiskMapping(m)
+	defer restore()
+
+	cmdSfdisk := testutil.MockCommand(c, "sfdisk", "")
+	defer cmdSfdisk.Restore()
+
+	cmdUdevadm := testutil.MockCommand(c, "udevadm", "")
+	defer cmdUdevadm.Restore()
+
+	dl, err := gadget.OnDiskVolumeFromDevice("/dev/node")
+	c.Assert(err, IsNil)
+
+	err = gadgettest.MakeMockGadget(s.gadgetRoot, gptGadgetWithMinSize)
+	c.Assert(err, IsNil)
+	gInfo, err := gadget.ReadInfoAndValidate(s.gadgetRoot, uc20Mod, nil)
+	c.Assert(err, IsNil)
+
+	deletedOffsetSize, err := install.RemoveCreatedPartitions(s.gadgetRoot, gInfo.Volumes["pc"], dl)
+	c.Assert(err, IsNil)
+	c.Assert(deletedOffsetSize, DeepEquals, map[int]install.StructOffsetSize{
+		1: {StartOffset: 400 * oneMeg, Size: 600 * oneMeg},
+		2: {StartOffset: 1000 * oneMeg, Size: 200 * oneMeg},
+	})
+
+	c.Assert(cmdSfdisk.Calls(), DeepEquals, [][]string{
+		{"sfdisk", "--no-reread", "--delete", "/dev/node", "2", "3"},
+	})
+
+	c.Assert(s.cmdPartx.Calls(), DeepEquals, [][]string{
+		{"partx", "-u", "/dev/node"},
+	})
+
+	// check that the OnDiskVolume was updated as expected
+	c.Assert(dl.Structure, DeepEquals, []gadget.OnDiskStructure{
+		{
+			PartitionFSLabel: "ubuntu-seed",
+			Name:             "Recovery",
+			Type:             "0a",
+			PartitionFSType:  "vfat",
+			StartOffset:      oneMeg,
+			DiskIndex:        1,
+			Node:             "/dev/node1",
+			Size:             399 * oneMeg,
+		},
+	})
 }
