@@ -23,6 +23,7 @@ import (
 	"fmt"
 	"os"
 	"path/filepath"
+	"strings"
 
 	. "gopkg.in/check.v1"
 
@@ -129,11 +130,11 @@ func (s *preseedSuite) TestRunPreseedHappy(c *C) {
 	restoreSystemSnapFromSeed := preseed.MockSystemSnapFromSeed(func(string, string) (string, string, error) { return "/a/core.snap", "", nil })
 	defer restoreSystemSnapFromSeed()
 
-	mockTargetSnapd := testutil.MockCommand(c, filepath.Join(targetSnapdRoot, "usr/lib/snapd/snapd"), `#!/bin/sh
-	if [ "$SNAPD_PRESEED" != "1" ]; then
-		exit 1
-	fi
-`)
+	mockTargetSnapd := testutil.MockCommand(c, filepath.Join(targetSnapdRoot, "usr/lib/snapd/snapd"), fmt.Sprintf(`#!/bin/sh
+	set -eu
+	[ -L %s/snap/snapd/current ]
+	[ "${SNAPD_PRESEED}" = "1" ]
+`, tmpDir))
 	defer mockTargetSnapd.Restore()
 
 	mockSnapdFromDeb := testutil.MockCommand(c, filepath.Join(tmpDir, "usr/lib/snapd/snapd"), `#!/bin/sh
@@ -391,6 +392,28 @@ func (s *preseedSuite) TestReset(c *C) {
 			}
 		}
 
+		emptiedDirs := []string{
+			dirs.SnapDataDir,
+			dirs.SnapCacheDir,
+			apparmor_sandbox.CacheDir,
+			dirs.SnapDesktopFilesDir,
+			dirs.SnapDBusSessionServicesDir,
+			dirs.SnapDBusSystemServicesDir,
+			dirs.SnapMountDir,
+		}
+
+		removedDirs := []string{
+			dirs.SnapAssertsDBDir,
+			dirs.FeaturesDir,
+			dirs.SnapDesktopIconsDir,
+			dirs.SnapDeviceDir,
+			dirs.SnapCookieDir,
+			dirs.SnapMountPolicyDir,
+			dirs.SnapAppArmorDir,
+			dirs.SnapSeqDir,
+			dirs.SnapSeccompBase,
+		}
+
 		checkArtifacts := func(exists bool) {
 			for _, art := range artifacts {
 				fullPath := filepath.Join(tmpDir, art.path)
@@ -399,6 +422,23 @@ func (s *preseedSuite) TestReset(c *C) {
 				} else {
 					c.Check(osutil.FileExists(fullPath), Equals, exists, Commentf("offending file: %s", fullPath))
 				}
+				if exists {
+					continue
+				}
+				dir := filepath.Dir(art.path)
+				for _, d := range emptiedDirs {
+					if strings.HasPrefix(dir, d) {
+						c.Check(osutil.IsDirectory(filepath.Join(tmpDir, d)), Equals, true)
+						entries, err := os.ReadDir(filepath.Join(tmpDir, d))
+						c.Assert(err, IsNil)
+						c.Check(len(entries), Equals, 0, Commentf("directory %s should be empty", d))
+					}
+				}
+				for _, d := range removedDirs {
+					if strings.HasPrefix(dir, d) {
+						c.Check(osutil.IsDirectory(filepath.Join(tmpDir, d)), Equals, false)
+					}
+				}
 			}
 		}
 
