@@ -1,7 +1,7 @@
 // -*- Mode: Go; indent-tabs-mode: t -*-
 
 /*
- * Copyright (C) 2017-2018 Canonical Ltd
+ * Copyright (C) 2017-2024 Canonical Ltd
  *
  * This program is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License version 3 as
@@ -23,10 +23,13 @@ import (
 	"bytes"
 	"fmt"
 	"path/filepath"
+	"regexp"
 	"sort"
 	"strings"
 
 	"github.com/snapcore/snapd/interfaces"
+	"github.com/snapcore/snapd/logger"
+	apparmor_sandbox "github.com/snapcore/snapd/sandbox/apparmor"
 	"github.com/snapcore/snapd/snap"
 	"github.com/snapcore/snapd/strutil"
 )
@@ -42,6 +45,110 @@ const (
 	UnconfinedEnabled
 )
 
+type prioritizedSnippets struct {
+	priority uint
+	list     []string
+}
+
+// SnippetKey is an opaque string identifying a class of snippets.
+//
+// Some APIs require the use of snippet keys to allow adding many different snippets
+// with the same key but possibly different priority.
+type SnippetKey struct {
+	key string
+}
+
+func (pk *SnippetKey) String() string {
+	return pk.key
+}
+
+func newSnippetKey(key string) SnippetKey {
+	return SnippetKey{key: key}
+}
+
+// registeredKeys is a list of allowed keys for prioritized snippets.
+// Trying to add a prioritized snippet with an unregistered key is
+// an error. Trying to register the same key twice is also an error.
+var registeredKeys map[SnippetKey]bool = make(map[SnippetKey]bool)
+
+// RegisterSnippetKey adds a key to the list of valid keys
+func RegisterSnippetKey(key string) SnippetKey {
+	snippetKey := newSnippetKey(key)
+	if _, ok := registeredKeys[snippetKey]; ok {
+		logger.Panicf("priority key %s is already registered", key)
+	}
+	registeredKeys[snippetKey] = true
+	return snippetKey
+}
+
+// GetSnippetKey retrieves all the current valid keys
+func RegisteredSnippetKeys() []string {
+	keylist := make([]string, 0, len(registeredKeys))
+	for k := range registeredKeys {
+		keylist = append(keylist, k.key)
+	}
+	return keylist
+}
+
+// MetadataTag is an opaque string used to tag a snippet.
+//
+// Metadata tags can be added to AppArmor rules or snippets to encode additional
+// information which the kernel can then send back to snapd when the kernel
+// sends a message notification.
+//
+// When each metadata tag is registered, it associated with a particular
+// interface. Once registered in association with an interface, any attempt to
+// register it in association with a different interface will result in a panic,
+// though a tag may be repeatedly registered in association with the same
+// interface with which it was originally registered.
+type MetadataTag struct {
+	tag string
+}
+
+func (mt MetadataTag) String() string {
+	return mt.tag
+}
+
+var validMetadataTagRegexp = regexp.MustCompile("^[a-z][a-z0-9_-]*$")
+
+func newMetadataTag(tag string) MetadataTag {
+	if !validMetadataTagRegexp.MatchString(tag) {
+		logger.Panicf("cannot register invalid metadata tag: %q", tag)
+	}
+	return MetadataTag{tag: tag}
+}
+
+// registeredMetadataTags is a mapping from metadata tag to the interface with
+// which it was associated when registered.
+var registeredMetadataTags map[MetadataTag]string = make(map[MetadataTag]string)
+
+// RegisterMetadataTagWithInterface marks the given tag as being associated
+// with the given interface and returns the tag as a MetadataTag. If the tag
+// is already associated with a different interface, this function panics.
+func RegisterMetadataTagWithInterface(tag string, iface string) MetadataTag {
+	if iface == "" {
+		logger.Panicf("cannot register metadata tag with missing interface: %q", tag)
+	}
+	metadataTag := newMetadataTag(tag)
+	if val, ok := registeredMetadataTags[metadataTag]; ok && val != iface {
+		logger.Panicf("cannot register metadata tag %q to two different interfaces: %q and %q", tag, val, iface)
+	}
+	registeredMetadataTags[metadataTag] = iface
+	return metadataTag
+}
+
+// InterfaceForMetadataTag retrieves the interface associated with the given
+// tag, if the tag was registered as associated with an interface. If the tag
+// was not registered, returns false.
+func InterfaceForMetadataTag(tag string) (string, bool) {
+	metadataTag := newMetadataTag(tag)
+	iface, ok := registeredMetadataTags[metadataTag]
+	if !ok {
+		return "", false
+	}
+	return iface, true
+}
+
 // Specification assists in collecting apparmor entries associated with an interface.
 type Specification struct {
 	// appSet is the set of snap applications and hooks that the specification
@@ -56,6 +163,15 @@ type Specification struct {
 	// of the application or hook.
 	snippets map[string][]string
 
+	// prioritizedSnippets are just like snippets, but they have a priority value
+	// and a snippet key. An interface can add a snippet with a specific key and a priority.
+	// If there doesn't exist any snippet with that key, the passed snippet will be
+	// added as-is. But if it does exist, the new snippet will replace the old one if
+	// the new priority is bigger than the old one; will be appended if the new
+	// priority is the same as the old one, and will be discarded if the new priority
+	// is smaller than the old one.
+	prioritizedSnippets map[string]map[SnippetKey]prioritizedSnippets
+
 	// dedupSnippets are just like snippets but are added only once to the
 	// resulting policy in an effort to avoid certain expensive to de-duplicate
 	// rules by apparmor_parser.
@@ -108,6 +224,9 @@ type Specification struct {
 	// Same as the above, but for the pycache deny rule which breaks docker
 	suppressPycacheDeny bool
 
+	// Include prompt prefix for relevant rules when generating security profiles.
+	usePromptPrefix bool
+
 	// Unconfined profile mode allows a profile to be applied without any
 	// real confinement
 	unconfined UnconfinedMode
@@ -132,6 +251,42 @@ func (spec *Specification) setScope(securityTags []string) (restore func()) {
 	}
 }
 
+var metadataTagsSupported = apparmor_sandbox.MetadataTagsSupported
+
+// MetadataTagSnippet wraps the given AppArmor rule snippet in the given
+// metadata tags if tagging is supported, and returns the resulting snippet.
+// If tagging is not supported, returns the snippet unchanged.
+func MetadataTagSnippet(snippet string, tags []MetadataTag) string {
+	if len(tags) == 0 || !metadataTagsSupported() {
+		return snippet
+	}
+
+	var b strings.Builder
+
+	// Put a blank line before the tagged block and open the tags set
+	b.WriteString("\ntags=(")
+
+	// Write the tags, separated by spaces
+	for i, tag := range tags {
+		if i > 0 {
+			b.WriteString(" ")
+		}
+		b.WriteString(tag.String())
+	}
+
+	// Close the tags, open the rules bracket, and prepare to write the snippet
+	// on a new line
+	b.WriteString(") {\n")
+
+	// Write the snippet itself
+	b.WriteString(snippet)
+
+	// Write the closing bracket, and add an extra newline afterwards
+	b.WriteString("\n}\n")
+
+	return b.String()
+}
+
 // AddSnippet adds a new apparmor snippet to all applications and hooks using the interface.
 func (spec *Specification) AddSnippet(snippet string) {
 	if len(spec.securityTags) == 0 {
@@ -146,6 +301,51 @@ func (spec *Specification) AddSnippet(snippet string) {
 	}
 }
 
+// AddPrioritizedSnippet adds a new apparmor snippet to all applications and hooks using the interface,
+// but identified with a key and a priority. If no other snippet exists with that key, the snippet is
+// added like with AddSnippet, but if there is already another snippet with that key, the priority of
+// both will be taken into account to decide whether the new snippet replaces the old one, is appended
+// to it, or is just ignored. The key must have been previously registered using RegisterSnippetKey().
+func (spec *Specification) AddPrioritizedSnippet(snippet string, key SnippetKey, priority uint) {
+	if _, ok := registeredKeys[key]; !ok {
+		logger.Panicf("priority key %s is not registered", key.String())
+	}
+	if len(spec.securityTags) == 0 {
+		return
+	}
+	if spec.prioritizedSnippets == nil {
+		spec.prioritizedSnippets = make(map[string]map[SnippetKey]prioritizedSnippets)
+	}
+
+	for _, tag := range spec.securityTags {
+		if _, exists := spec.prioritizedSnippets[tag]; !exists {
+			spec.prioritizedSnippets[tag] = make(map[SnippetKey]prioritizedSnippets)
+		}
+		snippets := spec.prioritizedSnippets[tag][key]
+		// If the entry doesn't exist, it will return a snippet with an empty
+		// snippet string and zero priority.
+		if snippets.priority == priority {
+			// if the priority is the same, just append the snippet to the snippets already there
+			snippets.list = append(snippets.list, snippet)
+		} else if snippets.priority < priority {
+			// if the priority is bigger, replace the snippets with the new one
+			snippets.list = append([]string(nil), snippet)
+			snippets.priority = priority
+		} // smaller priority, discard
+		spec.prioritizedSnippets[tag][key] = snippets
+	}
+}
+
+func (spec *Specification) composeSnippetsForTag(tag string) []string {
+	// Compose the normal and the prioritized snippets in a single string array
+	composedSnippets := append([]string(nil), spec.snippets[tag]...)
+	for key := range spec.prioritizedSnippets[tag] {
+		composedSnippets = append(composedSnippets, spec.prioritizedSnippets[tag][key].list...)
+	}
+	sort.Strings(composedSnippets)
+	return composedSnippets
+}
+
 // AddDeduplicatedSnippet adds a new apparmor snippet to all applications and hooks using the interface.
 //
 // Certain combinations of snippets may be computationally expensive for
@@ -258,7 +458,7 @@ func (spec *Specification) AddUpdateNS(snippet string) {
 }
 
 // AddUpdateNSf formats and adds a new apparmor snippet for the snap-update-ns program.
-func (spec *Specification) AddUpdateNSf(f string, args ...interface{}) {
+func (spec *Specification) AddUpdateNSf(f string, args ...any) {
 	spec.AddUpdateNS(fmt.Sprintf(f, args...))
 }
 
@@ -322,7 +522,8 @@ func (spec *Specification) emitLayout(si *snap.Info, layout *snap.Layout) {
 //
 // Importantly, the above mount operations are happening within the per-snap
 // mount namespace.
-func (spec *Specification) AddLayout(snapInfo *snap.Info) {
+func (spec *Specification) AddLayout(appSet *interfaces.SnapAppSet) {
+	snapInfo := appSet.Info()
 	if len(snapInfo.Layout) == 0 {
 		return
 	}
@@ -334,13 +535,11 @@ func (spec *Specification) AddLayout(snapInfo *snap.Info) {
 	}
 	sort.Strings(paths)
 
-	// Get tags describing all apps and hooks.
-	tags := make([]string, 0, len(snapInfo.Apps)+len(snapInfo.Hooks))
-	for _, app := range snapInfo.Apps {
-		tags = append(tags, app.SecurityTag())
-	}
-	for _, hook := range snapInfo.Hooks {
-		tags = append(tags, hook.SecurityTag())
+	// Get tags describing all runnables (apps, hooks, component hooks)
+	runnables := appSet.Runnables()
+	tags := make([]string, 0, len(runnables))
+	for _, r := range runnables {
+		tags = append(tags, r.SecurityTag)
 	}
 
 	// Append layout snippets to all tags; the layout applies equally to the
@@ -410,7 +609,7 @@ func isProbablyPresent(path string) bool {
 }
 
 // GenWritableMimicProfile generates apparmor rules for a writable mimic at the given path.
-func GenWritableMimicProfile(emit func(f string, args ...interface{}), path string, assumedPrefixDepth int) {
+func GenWritableMimicProfile(emit func(f string, args ...any), path string, assumedPrefixDepth int) {
 	emit("  # Writable mimic %s\n", path)
 
 	iter, err := strutil.NewPathIterator(path)
@@ -422,7 +621,8 @@ func GenWritableMimicProfile(emit func(f string, args ...interface{}), path stri
 	emit("  # .. permissions for traversing the prefix that is assumed to exist\n")
 	for iter.Next() {
 		if iter.Depth() < assumedPrefixDepth {
-			emit("  \"%s\" r,\n", iter.CurrentPath())
+			cp := iter.CurrentPathPlusSlash()
+			emit("  \"%s\" r,\n", cp)
 		}
 	}
 
@@ -436,7 +636,7 @@ func GenWritableMimicProfile(emit func(f string, args ...interface{}), path stri
 		// full mimic path. This is called a mimic "variant". Both of the paths
 		// must end with a slash as this is important for apparmor file vs
 		// directory path semantics.
-		mimicPath := filepath.Join(iter.CurrentBaseNoSlash(), iter.CurrentNameNoSlash()) + "/"
+		mimicPath := iter.CurrentPathPlusSlash()
 		mimicAuxPath := filepath.Join("/tmp/.snap", iter.CurrentPath()) + "/"
 		emit("  # .. variant with mimic at %s\n", mimicPath)
 		emit("  # Allow reading the mimic directory, it must exist in the first place.\n")
@@ -471,7 +671,7 @@ func GenWritableMimicProfile(emit func(f string, args ...interface{}), path stri
 }
 
 // GenWritableFileProfile writes a profile for snap-update-ns for making given file writable.
-func GenWritableFileProfile(emit func(f string, args ...interface{}), path string, assumedPrefixDepth int) {
+func GenWritableFileProfile(emit func(f string, args ...any), path string, assumedPrefixDepth int) {
 	if path == "/" {
 		return
 	}
@@ -488,7 +688,7 @@ func GenWritableFileProfile(emit func(f string, args ...interface{}), path strin
 }
 
 // GenWritableProfile generates a profile for snap-update-ns for making given directory writable.
-func GenWritableProfile(emit func(f string, args ...interface{}), path string, assumedPrefixDepth int) {
+func GenWritableProfile(emit func(f string, args ...any), path string, assumedPrefixDepth int) {
 	if path == "/" {
 		return
 	}
@@ -535,6 +735,12 @@ func (spec *Specification) SecurityTags() []string {
 		tags = append(tags, t)
 		seen[t] = true
 	}
+	for t := range spec.prioritizedSnippets {
+		if !seen[t] {
+			tags = append(tags, t)
+			seen[t] = true
+		}
+	}
 	for t := range spec.dedupSnippets {
 		if !seen[t] {
 			tags = append(tags, t)
@@ -550,7 +756,7 @@ func (spec *Specification) SecurityTags() []string {
 }
 
 func (spec *Specification) snippetsForTag(tag string) []string {
-	snippets := append([]string(nil), spec.snippets[tag]...)
+	snippets := append([]string(nil), spec.composeSnippetsForTag(tag)...)
 	// First add any deduplicated snippets
 	if bag := spec.dedupSnippets[tag]; bag != nil {
 		snippets = append(snippets, bag.Items()...)
@@ -628,7 +834,7 @@ func emitEnsureDir(spec *Specification, ifaceName string, ensureDirSpec *interfa
 		return
 	}
 	for iter.Next() {
-		if iter.CurrentPathNoSlash() == mustExistDir {
+		if iter.CurrentPath() == mustExistDir {
 			emit("  # Allow the %s interface to create potentially missing directories", ifaceName)
 			emit("  owner %s rw,", appArmorDir(replacePrefixHome(mustExistDir)))
 			break
@@ -637,7 +843,7 @@ func emitEnsureDir(spec *Specification, ifaceName string, ensureDirSpec *interfa
 
 	// Create entries for the remaining directories after MustExistDir up to and including EnsureDir
 	for iter.Next() {
-		emit("  owner %s/ rw,", replacePrefixHome(iter.CurrentPathNoSlash()))
+		emit("  owner %s rw,", replacePrefixHome(iter.CurrentPathPlusSlash()))
 	}
 }
 
@@ -758,6 +964,12 @@ func (spec *Specification) SuppressPtraceTrace() bool {
 	return spec.suppressPtraceTrace
 }
 
+// UsePromptPrefix returns whether the prompt prefix should be included for
+// relevant rules when generating security profiles.
+func (spec *Specification) UsePromptPrefix() bool {
+	return spec.usePromptPrefix
+}
+
 // SetUsesSysModuleCapability records that some interface has granted the
 // sys_module capability
 func (spec *Specification) SetUsesSysModuleCapability() {
