@@ -79,6 +79,12 @@ var templateCommon = `
 # SNAP_NAME so the security policy will allow writing to both locations (since
 # they are equivalent).
 
+# The template is applied to all the snaps regardless of their base snaps. Some base
+# snaps, notably core26, chose to ship uutils-coreutils as their native installation
+# location used by cargo.
+# The usr/bin/gnu rule deliberately doesn't have a trailing slash
+# This is to support paths like /usr/bin/ls -> /usr/bin/gnuls
+@{SNAP_COREUTIL_DIRS}="/{bin/,usr/bin/,usr/bin/gnu,usr/lib/cargo/bin/coreutils/}"
 ###VAR###
 
 ###PROFILEATTACH### ###FLAGS### {
@@ -179,7 +185,6 @@ var templateCommon = `
   /usr/lib/snapd/snap-exec m,
 
   # For gdb support
-  /usr/lib/snapd/snap-gdb-shim ixr,
   /usr/lib/snapd/snap-gdbserver-shim ixr,
 
   # For in-snap tab completion
@@ -495,6 +500,10 @@ var templateCommon = `
   /run/credentials/ r,
   /run/credentials/snap.@{SNAP_INSTANCE_NAME}.*.service/ r,
   /run/credentials/snap.@{SNAP_INSTANCE_NAME}.*.service/** r,
+
+  # Work around for https://gitlab.com/apparmor/apparmor/-/issues/571
+  # which prevents access to mmap MAP_HUGETLB.
+  allow file / rwm,
   
   ###DEVMODE_SNAP_CONFINE###
 `
@@ -537,52 +546,54 @@ var defaultCoreRuntimeTemplateRules = `
   /usr/share/terminfo/** r,
 
   # Common utilities for shell scripts
-  /{,usr/}bin/arch ixr,
+  # Support coreutils paths (LP: #2123870)
+  @{SNAP_COREUTIL_DIRS}arch ixr,
   /{,usr/}bin/{,g,m}awk ixr,
-  /{,usr/}bin/base32 ixr,
-  /{,usr/}bin/base64 ixr,
-  /{,usr/}bin/basename ixr,
+  @{SNAP_COREUTIL_DIRS}base32 ixr,
+  @{SNAP_COREUTIL_DIRS}base64 ixr,
+  @{SNAP_COREUTIL_DIRS}basename ixr,
   /{,usr/}bin/bunzip2 ixr,
   /{,usr/}bin/busctl ixr,
   /{,usr/}bin/bzcat ixr,
   /{,usr/}bin/bzdiff ixr,
   /{,usr/}bin/bzgrep ixr,
   /{,usr/}bin/bzip2 ixr,
-  /{,usr/}bin/cat ixr,
-  /{,usr/}bin/chgrp ixr,
-  /{,usr/}bin/chmod ixr,
-  /{,usr/}bin/chown ixr,
+  @{SNAP_COREUTIL_DIRS}cat ixr,
+  @{SNAP_COREUTIL_DIRS}chgrp ixr,
+  @{SNAP_COREUTIL_DIRS}chmod ixr,
+  @{SNAP_COREUTIL_DIRS}chown ixr,
   /{,usr/}bin/clear ixr,
   /{,usr/}bin/cmp ixr,
-  /{,usr/}bin/cp ixr,
+  @{SNAP_COREUTIL_DIRS}cp ixr,
   /{,usr/}bin/cpio ixr,
-  /{,usr/}bin/cut ixr,
-  /{,usr/}bin/date ixr,
+  @{SNAP_COREUTIL_DIRS}cut ixr,
+  @{SNAP_COREUTIL_DIRS}date ixr,
   /{,usr/}bin/dbus-daemon ixr,
   /{,usr/}bin/dbus-run-session ixr,
   /{,usr/}bin/dbus-send ixr,
-  /{,usr/}bin/dd ixr,
+  @{SNAP_COREUTIL_DIRS}dd ixr,
   /{,usr/}bin/diff{,3} ixr,
-  /{,usr/}bin/dir ixr,
-  /{,usr/}bin/dirname ixr,
-  /{,usr/}bin/du ixr,
-  /{,usr/}bin/echo ixr,
+  @{SNAP_COREUTIL_DIRS}dir ixr,
+  @{SNAP_COREUTIL_DIRS}dirname ixr,
+  @{SNAP_COREUTIL_DIRS}du ixr,
+  @{SNAP_COREUTIL_DIRS}echo ixr,
   /{,usr/}bin/{,e,f,r}grep ixr,
-  /{,usr/}bin/env ixr,
-  /{,usr/}bin/expr ixr,
-  /{,usr/}bin/false ixr,
+  @{SNAP_COREUTIL_DIRS}env ixr,
+  @{SNAP_COREUTIL_DIRS}expr ixr,
+  @{SNAP_COREUTIL_DIRS}false ixr,
   /{,usr/}bin/find ixr,
   /{,usr/}bin/flock ixr,
-  /{,usr/}bin/fmt ixr,
-  /{,usr/}bin/fold ixr,
+  @{SNAP_COREUTIL_DIRS}fmt ixr,
+  @{SNAP_COREUTIL_DIRS}fold ixr,
+  /{,usr/}bin/free ixr,
   /{,usr/}bin/getconf ixr,
   /{,usr/}bin/getent ixr,
   /{,usr/}bin/getopt ixr,
-  /{,usr/}bin/groups ixr,
+  @{SNAP_COREUTIL_DIRS}groups ixr,
   /{,usr/}bin/gzip ixr,
-  /{,usr/}bin/head ixr,
+  @{SNAP_COREUTIL_DIRS}head ixr,
   /{,usr/}bin/hostname ixr,
-  /{,usr/}bin/id ixr,
+  @{SNAP_COREUTIL_DIRS}id ixr,
   /{,usr/}bin/igawk ixr,
   /{,usr/}bin/infocmp ixr,
   /{,usr/}bin/kill ixr,
@@ -590,74 +601,74 @@ var defaultCoreRuntimeTemplateRules = `
   /{usr/,}lib{,32,64}/ld{,32,64}-*.so ix,
   /{usr/,}lib/@{multiarch}/ld{,32,64}-*.so* ix,
   /{,usr/}bin/less{,file,pipe} ixr,
-  /{,usr/}bin/ln ixr,
   /{,usr/}bin/line ixr,
-  /{,usr/}bin/link ixr,
+  @{SNAP_COREUTIL_DIRS}link ixr,
+  @{SNAP_COREUTIL_DIRS}ln ixr,
   /{,usr/}bin/locale ixr,
   /{,usr/}bin/logger ixr,
-  /{,usr/}bin/ls ixr,
-  /{,usr/}bin/md5sum ixr,
-  /{,usr/}bin/mkdir ixr,
-  /{,usr/}bin/mkfifo ixr,
-  /{,usr/}bin/mknod ixr,
-  /{,usr/}bin/mktemp ixr,
+  @{SNAP_COREUTIL_DIRS}ls ixr,
+  @{SNAP_COREUTIL_DIRS}md5sum ixr,
+  @{SNAP_COREUTIL_DIRS}mkdir ixr,
+  @{SNAP_COREUTIL_DIRS}mkfifo ixr,
+  @{SNAP_COREUTIL_DIRS}mknod ixr,
+  @{SNAP_COREUTIL_DIRS}mktemp ixr,
   /{,usr/}bin/more ixr,
-  /{,usr/}bin/mv ixr,
-  /{,usr/}bin/nice ixr,
-  /{,usr/}bin/nohup ixr,
-  /{,usr/}bin/numfmt ixr,
-  /{,usr/}bin/od ixr,
+  @{SNAP_COREUTIL_DIRS}mv ixr,
+  @{SNAP_COREUTIL_DIRS}nice ixr,
+  @{SNAP_COREUTIL_DIRS}nohup ixr,
+  @{SNAP_COREUTIL_DIRS}numfmt ixr,
+  @{SNAP_COREUTIL_DIRS}od ixr,
   /{,usr/}bin/openssl ixr, # may cause harmless capability block_suspend denial
-  /{,usr/}bin/paste ixr,
+  @{SNAP_COREUTIL_DIRS}paste ixr,
   /{,usr/}bin/pgrep ixr,
-  /{,usr/}bin/printenv ixr,
-  /{,usr/}bin/printf ixr,
+  @{SNAP_COREUTIL_DIRS}printenv ixr,
+  @{SNAP_COREUTIL_DIRS}printf ixr,
   /{,usr/}bin/ps ixr,
-  /{,usr/}bin/pwd ixr,
-  /{,usr/}bin/readlink ixr,
-  /{,usr/}bin/realpath ixr,
+  @{SNAP_COREUTIL_DIRS}pwd ixr,
+  @{SNAP_COREUTIL_DIRS}readlink ixr,
+  @{SNAP_COREUTIL_DIRS}realpath ixr,
   /{,usr/}bin/rev ixr,
-  /{,usr/}bin/rm ixr,
-  /{,usr/}bin/rmdir ixr,
+  @{SNAP_COREUTIL_DIRS}rm ixr,
+  @{SNAP_COREUTIL_DIRS}rmdir ixr,
   /{,usr/}bin/run-parts ixr,
   /{,usr/}bin/sed ixr,
-  /{,usr/}bin/seq ixr,
+  @{SNAP_COREUTIL_DIRS}seq ixr,
   /{,usr/}bin/setpriv ixr,
-  /{,usr/}bin/sha{1,224,256,384,512}sum ixr,
-  /{,usr/}bin/shuf ixr,
-  /{,usr/}bin/sleep ixr,
-  /{,usr/}bin/sort ixr,
-  /{,usr/}bin/stat ixr,
-  /{,usr/}bin/stdbuf ixr,
-  /{,usr/}bin/stty ixr,
-  /{,usr/}bin/sync ixr,
+  @{SNAP_COREUTIL_DIRS}sha{1,224,256,384,512}sum ixr,
+  @{SNAP_COREUTIL_DIRS}shuf ixr,
+  @{SNAP_COREUTIL_DIRS}sleep ixr,
+  @{SNAP_COREUTIL_DIRS}sort ixr,
+  @{SNAP_COREUTIL_DIRS}stat ixr,
+  @{SNAP_COREUTIL_DIRS}stdbuf ixr,
+  @{SNAP_COREUTIL_DIRS}stty ixr,
+  @{SNAP_COREUTIL_DIRS}sync ixr,
   /{,usr/}bin/systemd-cat ixr,
   /{,usr/}bin/systemd-creds ixr,
-  /{,usr/}bin/tac ixr,
-  /{,usr/}bin/tail ixr,
+  @{SNAP_COREUTIL_DIRS}tac ixr,
+  @{SNAP_COREUTIL_DIRS}tail ixr,
   /{,usr/}bin/tar ixr,
-  /{,usr/}bin/tee ixr,
-  /{,usr/}bin/test ixr,
+  @{SNAP_COREUTIL_DIRS}tee ixr,
+  @{SNAP_COREUTIL_DIRS}test ixr,
   /{,usr/}bin/tempfile ixr,
   /{,usr/}bin/tset ixr,
-  /{,usr/}bin/touch ixr,
+  @{SNAP_COREUTIL_DIRS}touch ixr,
   /{,usr/}bin/tput ixr,
-  /{,usr/}bin/tr ixr,
-  /{,usr/}bin/true ixr,
-  /{,usr/}bin/tty ixr,
-  /{,usr/}bin/uname ixr,
-  /{,usr/}bin/uniq ixr,
-  /{,usr/}bin/unlink ixr,
+  @{SNAP_COREUTIL_DIRS}tr ixr,
+  @{SNAP_COREUTIL_DIRS}true ixr,
+  @{SNAP_COREUTIL_DIRS}tty ixr,
+  @{SNAP_COREUTIL_DIRS}uname ixr,
+  @{SNAP_COREUTIL_DIRS}uniq ixr,
+  @{SNAP_COREUTIL_DIRS}unlink ixr,
   /{,usr/}bin/unxz ixr,
   /{,usr/}bin/unzip ixr,
   /{,usr/}bin/uptime ixr,
-  /{,usr/}bin/vdir ixr,
+  @{SNAP_COREUTIL_DIRS}vdir ixr,
   /{,usr/}bin/vim.tiny ixr,
-  /{,usr/}bin/wc ixr,
+  @{SNAP_COREUTIL_DIRS}wc ixr,
   /{,usr/}bin/which{,.debianutils} ixr,
   /{,usr/}bin/xargs ixr,
   /{,usr/}bin/xz ixr,
-  /{,usr/}bin/yes ixr,
+  @{SNAP_COREUTIL_DIRS}yes ixr,
   /{,usr/}bin/zcat ixr,
   /{,usr/}bin/z{,e,f}grep ixr,
   /{,usr/}bin/zip ixr,
@@ -672,7 +683,13 @@ var defaultCoreRuntimeTemplateRules = `
   /{,usr/}sbin/ldconfig{,.real} ixr,
 
   # Allow all snaps to chroot
+  # chroot can be on either coreutil paths or sbin depending on the Ubuntu release
+  @{SNAP_COREUTIL_DIRS}chroot ixr,
   /{,usr/}sbin/chroot ixr,
+
+  # Allow pidof (and killall5, as pidof can be a symlink to killall5 in some distros)
+  /{,usr/}bin/pidof ixr,
+  /{,usr/}sbin/killall5 ixr,
 `
 
 // defaultCoreRuntimeTemplate contains the default apparmor template for core* bases. It
