@@ -21,12 +21,16 @@ package builtin_test
 
 import (
 	"fmt"
+	"path/filepath"
 
 	. "gopkg.in/check.v1"
 
+	"github.com/snapcore/snapd/dirs"
 	"github.com/snapcore/snapd/interfaces"
 	"github.com/snapcore/snapd/interfaces/builtin"
+	"github.com/snapcore/snapd/interfaces/configfiles"
 	"github.com/snapcore/snapd/interfaces/ldconfig"
+	"github.com/snapcore/snapd/osutil"
 	"github.com/snapcore/snapd/snap"
 	"github.com/snapcore/snapd/testutil"
 )
@@ -60,7 +64,8 @@ apps:
 const cudaDriverLibsProvider = `name: cuda-provider
 version: 0
 slots:
-  cuda-driver-libs:
+  cuda-slot:
+    interface: cuda-driver-libs
     compatibility: cuda-(9..12)-ubuntu-2404
     library-source:
       - $SNAP/lib1
@@ -73,7 +78,7 @@ func (s *CudaDriverLibsInterfaceSuite) SetUpTest(c *C) {
 	s.plug, s.plugInfo = MockConnectedPlug(c, cudaDriverLibsConsumerYaml,
 		&snap.SideInfo{Revision: snap.R(3)}, "cuda")
 	s.slot, s.slotInfo = MockConnectedSlot(c, cudaDriverLibsProvider,
-		&snap.SideInfo{Revision: snap.R(5)}, "cuda-driver-libs")
+		&snap.SideInfo{Revision: snap.R(5)}, "cuda-slot")
 }
 
 func (s *CudaDriverLibsInterfaceSuite) TestName(c *C) {
@@ -95,7 +100,7 @@ slots:
       - /snap/cuda-provider/current/lib1
 `, nil, "cuda")
 	c.Assert(interfaces.BeforePrepareSlot(s.iface, slot), ErrorMatches,
-		`cuda-driver-libs source directory .* must start with \$SNAP/ or \$\{SNAP\}/`)
+		`cuda-driver-libs library-source directory .* must start with \$SNAP/ or \$\{SNAP\}/`)
 
 	slot = MockSlot(c, `name: cuda-provider
 version: 0
@@ -140,7 +145,7 @@ func (s *CudaDriverLibsInterfaceSuite) TestSanitizeSlotAPIversion(c *C) {
 		{"cuda-9", `compatibility label "cuda-9": unexpected number of strings \(should be 2\)`},
 		{"other-9", `compatibility label "other-9": unexpected number of strings \(should be 2\)`},
 		{"cuda-10-2-ubuntu-2510", `compatibility label "cuda-10-2-ubuntu-2510": unexpected number of integers \(should be 1 for "cuda"\)`},
-		{"cuda 5", `compatibility label "cuda 5": bad string "cuda 5"`},
+		{"cuda 5", `compatibility label "cuda 5": while parsing: unexpected rune: 5`},
 	} {
 		slot := MockSlot(c, fmt.Sprintf(`name: cuda-provider
 version: 0
@@ -169,8 +174,20 @@ func (s *CudaDriverLibsInterfaceSuite) TestLdconfigSpec(c *C) {
 	spec := &ldconfig.Specification{}
 	c.Assert(spec.AddConnectedPlug(s.iface, s.plug, s.slot), IsNil)
 	c.Check(spec.LibDirs(), DeepEquals, map[ldconfig.SnapSlot][]string{
-		{SnapName: "cuda-provider", SlotName: "cuda-driver-libs"}: {"/snap/cuda-provider/5/lib1",
-			"/snap/cuda-provider/5/lib2"}})
+		{SnapName: "cuda-provider", SlotName: "cuda-slot"}: {filepath.Join(dirs.SnapMountDir, "cuda-provider/5/lib1"),
+			filepath.Join(dirs.SnapMountDir, "cuda-provider/5/lib2")}})
+}
+
+func (s *CudaDriverLibsInterfaceSuite) TestConfigfilesSpec(c *C) {
+	spec := &configfiles.Specification{}
+	c.Assert(spec.AddConnectedPlug(s.iface, s.plug, s.slot), IsNil)
+	c.Check(spec.PathContent(), DeepEquals, map[string]osutil.FileState{
+		"/var/lib/snapd/export/system_cuda-provider_cuda-slot_cuda-driver-libs.library-source": &osutil.MemoryFileState{
+			Content: []byte(
+				filepath.Join(dirs.SnapMountDir, "cuda-provider/5/lib1") + "\n" +
+					filepath.Join(dirs.SnapMountDir, "cuda-provider/5/lib2") + "\n"),
+			Mode: 0644},
+	})
 }
 
 func (s *CudaDriverLibsInterfaceSuite) TestStaticInfo(c *C) {
