@@ -119,23 +119,26 @@ apps:
 }
 
 func (s *DbusInterfaceSuite) SetUpTest(c *C) {
+	appSet, err := interfaces.NewSnapAppSet(s.snapInfo, nil)
+	c.Assert(err, IsNil)
+
 	s.sessionSlotInfo = s.snapInfo.Slots["test-session-slot"]
-	s.sessionSlot = interfaces.NewConnectedSlot(s.sessionSlotInfo, nil, nil)
+	s.sessionSlot = interfaces.NewConnectedSlot(s.sessionSlotInfo, appSet, nil, nil)
 	s.systemSlotInfo = s.snapInfo.Slots["test-system-slot"]
-	s.systemSlot = interfaces.NewConnectedSlot(s.systemSlotInfo, nil, nil)
+	s.systemSlot = interfaces.NewConnectedSlot(s.systemSlotInfo, appSet, nil, nil)
 	s.connectedSessionSlotInfo = s.snapInfo.Slots["test-session-connected-slot"]
-	s.connectedSessionSlot = interfaces.NewConnectedSlot(s.connectedSessionSlotInfo, nil, nil)
+	s.connectedSessionSlot = interfaces.NewConnectedSlot(s.connectedSessionSlotInfo, appSet, nil, nil)
 	s.connectedSystemSlotInfo = s.snapInfo.Slots["test-system-connected-slot"]
-	s.connectedSystemSlot = interfaces.NewConnectedSlot(s.connectedSystemSlotInfo, nil, nil)
+	s.connectedSystemSlot = interfaces.NewConnectedSlot(s.connectedSystemSlotInfo, appSet, nil, nil)
 
 	s.sessionPlugInfo = s.snapInfo.Plugs["test-session-plug"]
-	s.sessionPlug = interfaces.NewConnectedPlug(s.sessionPlugInfo, nil, nil)
+	s.sessionPlug = interfaces.NewConnectedPlug(s.sessionPlugInfo, appSet, nil, nil)
 	s.systemPlugInfo = s.snapInfo.Plugs["test-system-plug"]
-	s.systemPlug = interfaces.NewConnectedPlug(s.systemPlugInfo, nil, nil)
+	s.systemPlug = interfaces.NewConnectedPlug(s.systemPlugInfo, appSet, nil, nil)
 	s.connectedSessionPlugInfo = s.snapInfo.Plugs["test-session-connected-plug"]
-	s.connectedSessionPlug = interfaces.NewConnectedPlug(s.connectedSessionPlugInfo, nil, nil)
+	s.connectedSessionPlug = interfaces.NewConnectedPlug(s.connectedSessionPlugInfo, appSet, nil, nil)
 	s.connectedSystemPlugInfo = s.snapInfo.Plugs["test-system-connected-plug"]
-	s.connectedSystemPlug = interfaces.NewConnectedPlug(s.connectedSystemPlugInfo, nil, nil)
+	s.connectedSystemPlug = interfaces.NewConnectedPlug(s.connectedSystemPlugInfo, appSet, nil, nil)
 }
 
 func (s *DbusInterfaceSuite) TestName(c *C) {
@@ -289,7 +292,7 @@ plugs:
 }
 
 func (s *DbusInterfaceSuite) TestPermanentSlotAppArmorSession(c *C) {
-	apparmorSpec := apparmor.NewSpecification(interfaces.NewSnapAppSet(s.sessionSlotInfo.Snap))
+	apparmorSpec := apparmor.NewSpecification(s.sessionSlot.AppSet())
 	err := apparmorSpec.AddPermanentSlot(s.iface, s.sessionSlotInfo)
 	c.Assert(err, IsNil)
 	c.Assert(apparmorSpec.SecurityTags(), DeepEquals, []string{"snap.test-dbus.test-session-provider"})
@@ -316,7 +319,7 @@ func (s *DbusInterfaceSuite) TestPermanentSlotAppArmorSessionNative(c *C) {
 	restore := release.MockOnClassic(false)
 	defer restore()
 
-	apparmorSpec := apparmor.NewSpecification(interfaces.NewSnapAppSet(s.sessionSlotInfo.Snap))
+	apparmorSpec := apparmor.NewSpecification(s.sessionSlot.AppSet())
 	err := apparmorSpec.AddPermanentSlot(s.iface, s.sessionSlotInfo)
 	c.Assert(err, IsNil)
 	c.Assert(apparmorSpec.SecurityTags(), DeepEquals, []string{"snap.test-dbus.test-session-provider"})
@@ -329,7 +332,7 @@ func (s *DbusInterfaceSuite) TestPermanentSlotAppArmorSessionClassic(c *C) {
 	restore := release.MockOnClassic(true)
 	defer restore()
 
-	apparmorSpec := apparmor.NewSpecification(interfaces.NewSnapAppSet(s.sessionSlotInfo.Snap))
+	apparmorSpec := apparmor.NewSpecification(s.sessionSlot.AppSet())
 	err := apparmorSpec.AddPermanentSlot(s.iface, s.sessionSlotInfo)
 	c.Assert(err, IsNil)
 	c.Assert(apparmorSpec.SecurityTags(), DeepEquals, []string{"snap.test-dbus.test-session-provider"})
@@ -339,7 +342,7 @@ func (s *DbusInterfaceSuite) TestPermanentSlotAppArmorSessionClassic(c *C) {
 }
 
 func (s *DbusInterfaceSuite) TestPermanentSlotAppArmorSystem(c *C) {
-	apparmorSpec := apparmor.NewSpecification(interfaces.NewSnapAppSet(s.systemSlotInfo.Snap))
+	apparmorSpec := apparmor.NewSpecification(s.systemSlot.AppSet())
 	err := apparmorSpec.AddPermanentSlot(s.iface, s.systemSlotInfo)
 	c.Assert(err, IsNil)
 	c.Assert(apparmorSpec.SecurityTags(), DeepEquals, []string{"snap.test-dbus.test-system-provider"})
@@ -363,14 +366,14 @@ func (s *DbusInterfaceSuite) TestPermanentSlotAppArmorSystem(c *C) {
 }
 
 func (s *DbusInterfaceSuite) TestPermanentSlotDBusSession(c *C) {
-	dbusSpec := dbus.NewSpecification(interfaces.NewSnapAppSet(s.sessionSlotInfo.Snap))
+	dbusSpec := dbus.NewSpecification(s.sessionSlot.AppSet())
 	err := dbusSpec.AddPermanentSlot(s.iface, s.sessionSlotInfo)
 	c.Assert(err, IsNil)
 	c.Assert(dbusSpec.SecurityTags(), HasLen, 0)
 }
 
 func (s *DbusInterfaceSuite) TestPermanentSlotDBusSystem(c *C) {
-	dbusSpec := dbus.NewSpecification(interfaces.NewSnapAppSet(s.systemSlotInfo.Snap))
+	dbusSpec := dbus.NewSpecification(s.systemSlot.AppSet())
 	err := dbusSpec.AddPermanentSlot(s.iface, s.systemSlotInfo)
 	c.Assert(err, IsNil)
 	c.Assert(dbusSpec.SecurityTags(), DeepEquals, []string{"snap.test-dbus.test-system-provider"})
@@ -380,7 +383,7 @@ func (s *DbusInterfaceSuite) TestPermanentSlotDBusSystem(c *C) {
 }
 
 func (s *DbusInterfaceSuite) TestPermanentSlotSecCompSystem(c *C) {
-	seccompSpec := seccomp.NewSpecification(interfaces.NewSnapAppSet(s.systemSlotInfo.Snap))
+	seccompSpec := seccomp.NewSpecification(s.systemSlot.AppSet())
 	err := seccompSpec.AddPermanentSlot(s.iface, s.systemSlotInfo)
 	c.Assert(err, IsNil)
 	c.Assert(seccompSpec.SecurityTags(), DeepEquals, []string{"snap.test-dbus.test-system-provider"})
@@ -389,7 +392,7 @@ func (s *DbusInterfaceSuite) TestPermanentSlotSecCompSystem(c *C) {
 }
 
 func (s *DbusInterfaceSuite) TestPermanentSlotSecCompSession(c *C) {
-	seccompSpec := seccomp.NewSpecification(interfaces.NewSnapAppSet(s.sessionSlotInfo.Snap))
+	seccompSpec := seccomp.NewSpecification(s.sessionSlot.AppSet())
 	err := seccompSpec.AddPermanentSlot(s.iface, s.sessionSlotInfo)
 	c.Assert(err, IsNil)
 	c.Assert(seccompSpec.SecurityTags(), DeepEquals, []string{"snap.test-dbus.test-session-provider"})
@@ -398,7 +401,7 @@ func (s *DbusInterfaceSuite) TestPermanentSlotSecCompSession(c *C) {
 }
 
 func (s *DbusInterfaceSuite) TestConnectedSlotAppArmorSession(c *C) {
-	apparmorSpec := apparmor.NewSpecification(interfaces.NewSnapAppSet(s.connectedSessionSlot.Snap()))
+	apparmorSpec := apparmor.NewSpecification(s.connectedSessionSlot.AppSet())
 	err := apparmorSpec.AddConnectedSlot(s.iface, s.connectedSessionPlug, s.connectedSessionSlot)
 	c.Assert(err, IsNil)
 	c.Assert(apparmorSpec.SecurityTags(), DeepEquals, []string{"snap.test-dbus.test-session-consumer", "snap.test-dbus.test-session-provider", "snap.test-dbus.test-system-consumer", "snap.test-dbus.test-system-provider"})
@@ -418,7 +421,7 @@ func (s *DbusInterfaceSuite) TestConnectedSlotAppArmorSession(c *C) {
 }
 
 func (s *DbusInterfaceSuite) TestConnectedSlotAppArmorSystem(c *C) {
-	apparmorSpec := apparmor.NewSpecification(interfaces.NewSnapAppSet(s.connectedSystemSlot.Snap()))
+	apparmorSpec := apparmor.NewSpecification(s.connectedSessionPlug.AppSet())
 	err := apparmorSpec.AddConnectedSlot(s.iface, s.connectedSystemPlug, s.connectedSystemSlot)
 	c.Assert(err, IsNil)
 	c.Assert(apparmorSpec.SecurityTags(), DeepEquals, []string{"snap.test-dbus.test-session-consumer", "snap.test-dbus.test-session-provider", "snap.test-dbus.test-system-consumer", "snap.test-dbus.test-system-provider"})
@@ -438,7 +441,7 @@ func (s *DbusInterfaceSuite) TestConnectedSlotAppArmorSystem(c *C) {
 }
 
 func (s *DbusInterfaceSuite) TestConnectedPlugAppArmorSession(c *C) {
-	apparmorSpec := apparmor.NewSpecification(interfaces.NewSnapAppSet(s.connectedSessionPlug.Snap()))
+	apparmorSpec := apparmor.NewSpecification(s.connectedSessionPlug.AppSet())
 	err := apparmorSpec.AddConnectedPlug(s.iface, s.connectedSessionPlug, s.connectedSessionSlot)
 	c.Assert(err, IsNil)
 	c.Assert(apparmorSpec.SecurityTags(), DeepEquals, []string{"snap.test-dbus.test-session-consumer", "snap.test-dbus.test-session-provider", "snap.test-dbus.test-system-consumer", "snap.test-dbus.test-system-provider"})
@@ -463,7 +466,7 @@ func (s *DbusInterfaceSuite) TestConnectedPlugAppArmorSession(c *C) {
 }
 
 func (s *DbusInterfaceSuite) TestConnectedPlugAppArmorSystem(c *C) {
-	apparmorSpec := apparmor.NewSpecification(interfaces.NewSnapAppSet(s.connectedSystemPlug.Snap()))
+	apparmorSpec := apparmor.NewSpecification(s.connectedSystemPlug.AppSet())
 	err := apparmorSpec.AddConnectedPlug(s.iface, s.connectedSystemPlug, s.connectedSystemSlot)
 	c.Assert(err, IsNil)
 	c.Assert(apparmorSpec.SecurityTags(), DeepEquals, []string{"snap.test-dbus.test-session-consumer", "snap.test-dbus.test-session-provider", "snap.test-dbus.test-system-consumer", "snap.test-dbus.test-system-provider"})
@@ -510,14 +513,13 @@ slots:
   name: org.slotter.other-session
 `
 
-	plugInfo := snaptest.MockInfo(c, plugYaml, nil)
-	matchingPlug := interfaces.NewConnectedPlug(plugInfo.Plugs["this"], nil, nil)
+	matchingPlug, _ := MockConnectedPlug(c, plugYaml, nil, "this")
 
-	slotInfo := snaptest.MockInfo(c, slotYaml, nil)
-	matchingSlot := interfaces.NewConnectedSlot(slotInfo.Slots["this"], nil, nil)
-	nonmatchingSlot := interfaces.NewConnectedSlot(slotInfo.Slots["that"], nil, nil)
+	matchingSlot, _ := MockConnectedSlot(c, slotYaml, nil, "this")
+	slotInfo := matchingSlot.Snap()
+	nonmatchingSlot := interfaces.NewConnectedSlot(slotInfo.Slots["that"], matchingSlot.AppSet(), nil, nil)
 
-	apparmorSpec := apparmor.NewSpecification(interfaces.NewSnapAppSet(matchingPlug.Snap()))
+	apparmorSpec := apparmor.NewSpecification(matchingPlug.AppSet())
 	err := apparmorSpec.AddConnectedPlug(s.iface, matchingPlug, matchingSlot)
 	c.Assert(err, IsNil)
 	c.Assert(apparmorSpec.SecurityTags(), DeepEquals, []string{"snap.plugger.app"})
@@ -528,7 +530,7 @@ slots:
 	c.Check(snippet, Not(testutil.Contains), "org.slotter.other-session")
 	c.Check(snippet, Not(testutil.Contains), "bus=system")
 
-	apparmorSpec = apparmor.NewSpecification(interfaces.NewSnapAppSet(matchingPlug.Snap()))
+	apparmorSpec = apparmor.NewSpecification(matchingPlug.AppSet())
 	err = apparmorSpec.AddConnectedPlug(s.iface, matchingPlug, nonmatchingSlot)
 	c.Assert(err, IsNil)
 	c.Assert(apparmorSpec.SecurityTags(), HasLen, 0)
@@ -559,14 +561,13 @@ slots:
   name: org.slotter.other-session
 `
 
-	plugInfo := snaptest.MockInfo(c, plugYaml, nil)
-	matchingPlug := interfaces.NewConnectedPlug(plugInfo.Plugs["that"], nil, nil)
+	matchingPlug, _ := MockConnectedPlug(c, plugYaml, nil, "that")
 
-	slotInfo := snaptest.MockInfo(c, slotYaml, nil)
-	matchingSlot := interfaces.NewConnectedSlot(slotInfo.Slots["that"], nil, nil)
-	nonmatchingSlot := interfaces.NewConnectedSlot(slotInfo.Slots["this"], nil, nil)
+	matchingSlot, _ := MockConnectedSlot(c, slotYaml, nil, "that")
+	slotInfo := matchingSlot.Snap()
+	nonmatchingSlot := interfaces.NewConnectedSlot(slotInfo.Slots["this"], matchingSlot.AppSet(), nil, nil)
 
-	apparmorSpec := apparmor.NewSpecification(interfaces.NewSnapAppSet(matchingPlug.Snap()))
+	apparmorSpec := apparmor.NewSpecification(matchingPlug.AppSet())
 	err := apparmorSpec.AddConnectedPlug(s.iface, matchingPlug, matchingSlot)
 	c.Assert(err, IsNil)
 	c.Assert(apparmorSpec.SecurityTags(), DeepEquals, []string{"snap.plugger.app"})
@@ -577,7 +578,7 @@ slots:
 	c.Check(snippet, Not(testutil.Contains), "org.slotter.session")
 	c.Check(snippet, Not(testutil.Contains), "bus=session")
 
-	apparmorSpec = apparmor.NewSpecification(interfaces.NewSnapAppSet(matchingPlug.Snap()))
+	apparmorSpec = apparmor.NewSpecification(matchingPlug.AppSet())
 	err = apparmorSpec.AddConnectedPlug(s.iface, matchingPlug, nonmatchingSlot)
 	c.Assert(err, IsNil)
 	c.Assert(apparmorSpec.SecurityTags(), HasLen, 0)
@@ -612,15 +613,15 @@ slots:
   name: org.slotter.other-session
 `
 
+	matchingPlug1, _ := MockConnectedPlug(c, plugYaml, nil, "this")
 	plugInfo := snaptest.MockInfo(c, plugYaml, nil)
-	matchingPlug1 := interfaces.NewConnectedPlug(plugInfo.Plugs["this"], nil, nil)
-	matchingPlug2 := interfaces.NewConnectedPlug(plugInfo.Plugs["that"], nil, nil)
+	matchingPlug2 := interfaces.NewConnectedPlug(plugInfo.Plugs["that"], matchingPlug1.AppSet(), nil, nil)
 
+	matchingSlot1, _ := MockConnectedSlot(c, slotYaml, nil, "this")
 	slotInfo := snaptest.MockInfo(c, slotYaml, nil)
-	matchingSlot1 := interfaces.NewConnectedSlot(slotInfo.Slots["this"], nil, nil)
-	matchingSlot2 := interfaces.NewConnectedSlot(slotInfo.Slots["that"], nil, nil)
+	matchingSlot2 := interfaces.NewConnectedSlot(slotInfo.Slots["that"], matchingSlot1.AppSet(), nil, nil)
 
-	apparmorSpec := apparmor.NewSpecification(interfaces.NewSnapAppSet(matchingPlug1.Snap()))
+	apparmorSpec := apparmor.NewSpecification(matchingPlug1.AppSet())
 	err := apparmorSpec.AddConnectedPlug(s.iface, matchingPlug1, matchingSlot1)
 	c.Assert(err, IsNil)
 	c.Assert(apparmorSpec.SecurityTags(), DeepEquals, []string{"snap.plugger.app"})
@@ -628,7 +629,7 @@ slots:
 	c.Check(snippet, testutil.Contains, "org.slotter.session")
 	c.Check(snippet, testutil.Contains, "bus=session")
 
-	apparmorSpec = apparmor.NewSpecification(interfaces.NewSnapAppSet(matchingPlug2.Snap()))
+	apparmorSpec = apparmor.NewSpecification(matchingPlug2.AppSet())
 	err = apparmorSpec.AddConnectedPlug(s.iface, matchingPlug2, matchingSlot2)
 	c.Assert(err, IsNil)
 	c.Assert(apparmorSpec.SecurityTags(), DeepEquals, []string{"snap.plugger.app"})
@@ -655,13 +656,10 @@ slots:
   name: org.slotter.session
 `
 
-	plugInfo := snaptest.MockInfo(c, plugYaml, nil)
-	plug := interfaces.NewConnectedPlug(plugInfo.Plugs["this"], nil, nil)
+	slot, _ := MockConnectedSlot(c, slotYaml, nil, "this")
+	plug, _ := MockConnectedPlug(c, plugYaml, nil, "this")
 
-	slotInfo := snaptest.MockInfo(c, slotYaml, nil)
-	slot := interfaces.NewConnectedSlot(slotInfo.Slots["this"], nil, nil)
-
-	apparmorSpec := apparmor.NewSpecification(interfaces.NewSnapAppSet(plug.Snap()))
+	apparmorSpec := apparmor.NewSpecification(plug.AppSet())
 	err := apparmorSpec.AddConnectedPlug(s.iface, plug, slot)
 	c.Assert(err, IsNil)
 	c.Assert(apparmorSpec.SecurityTags(), HasLen, 0)
@@ -685,13 +683,10 @@ slots:
   name: org.slotter.nomatch
 `
 
-	plugInfo := snaptest.MockInfo(c, plugYaml, nil)
-	plug := interfaces.NewConnectedPlug(plugInfo.Plugs["this"], nil, nil)
+	slot, _ := MockConnectedSlot(c, slotYaml, nil, "this")
+	plug, _ := MockConnectedPlug(c, plugYaml, nil, "this")
 
-	slotInfo := snaptest.MockInfo(c, slotYaml, nil)
-	slot := interfaces.NewConnectedSlot(slotInfo.Slots["this"], nil, nil)
-
-	apparmorSpec := apparmor.NewSpecification(interfaces.NewSnapAppSet(plug.Snap()))
+	apparmorSpec := apparmor.NewSpecification(plug.AppSet())
 	err := apparmorSpec.AddConnectedPlug(s.iface, plug, slot)
 	c.Assert(err, IsNil)
 	c.Assert(apparmorSpec.SecurityTags(), HasLen, 0)
