@@ -20,15 +20,21 @@
 package builtin
 
 import (
+	"encoding/json"
 	"fmt"
 	"math"
 	"path/filepath"
+	"strings"
 
+	"github.com/snapcore/snapd/dirs"
 	"github.com/snapcore/snapd/interfaces"
 	"github.com/snapcore/snapd/interfaces/compatibility"
+	"github.com/snapcore/snapd/interfaces/configfiles"
 	"github.com/snapcore/snapd/interfaces/ldconfig"
 	"github.com/snapcore/snapd/interfaces/symlinks"
+	"github.com/snapcore/snapd/release"
 	"github.com/snapcore/snapd/snap"
+	"github.com/snapcore/snapd/systemd"
 )
 
 const eglDriverLibsSummary = `allows exposing EGL driver libraries to the system`
@@ -69,32 +75,25 @@ func (iface *eglDriverLibsInterface) BeforePrepareSlot(slot *snap.SlotInfo) erro
 		return fmt.Errorf("priority must be a positive integer")
 	}
 
-	var icdDir string
-	if err := slot.Attr("icd-source", &icdDir); err != nil {
-		return fmt.Errorf("invalid icd-source: %w", err)
-	}
-	// We want a directory in icd-source, should start with $SNAP
-	if err := validateSnapDir(icdDir); err != nil {
-		return err
-	}
-
 	var compatField string
 	if err := slot.Attr("compatibility", &compatField); err != nil {
 		return err
 	}
 	// Validate format of compatibility field - we don't actually need to
 	// do anything else with it until we start to support regular snaps.
-	_, err := compatibility.DecodeCompatField(compatField,
+	if err := compatibility.IsValidExpression(compatField,
 		&compatibility.CompatSpec{Dimensions: []compatibility.CompatDimension{
 			{Tag: "egl", Values: []compatibility.CompatRange{{Min: 1, Max: 1}, {Min: 5, Max: 5}}},
 			{Tag: "ubuntu", Values: []compatibility.CompatRange{{Min: 0, Max: math.MaxUint}}},
-		}})
-	if err != nil {
+		}}); err != nil {
 		return err
 	}
 
-	// Validate directories
-	return validateLdconfigLibDirs(slot)
+	// Validate *-source directories
+	if err := validateSourceDirs(slot, sourceDirAttr{attrName: "icd-source", isOptional: false}); err != nil {
+		return err
+	}
+	return validateSourceDirs(slot, sourceDirAttr{attrName: "library-source", isOptional: false})
 }
 
 func (iface *eglDriverLibsInterface) LdconfigConnectedPlug(spec *ldconfig.Specification, plug *interfaces.ConnectedPlug, slot *interfaces.ConnectedSlot) error {
@@ -103,30 +102,68 @@ func (iface *eglDriverLibsInterface) LdconfigConnectedPlug(spec *ldconfig.Specif
 }
 
 var _ = symlinks.ConnectedPlugCallback(&eglDriverLibsInterface{})
+var _ = interfaces.ConfigfilesUser(&eglDriverLibsInterface{})
 
-const eglVendorPath = "/etc/glvnd/egl_vendor.d"
+const (
+	eglDriverLibs = "egl-driver-libs"
+	eglVendorPath = "/etc/glvnd/egl_vendor.d"
+)
 
 func (iface *eglDriverLibsInterface) TrackedDirectories() []string {
 	return []string{eglVendorPath}
 }
 
+func checkEglIcdFile(slot *interfaces.ConnectedSlot, icdContent []byte) error {
+	var icdJson struct {
+		Icd struct {
+			LibraryPath string `json:"library_path"`
+		} `json:"ICD"`
+	}
+	err := json.Unmarshal(icdContent, &icdJson)
+	if err != nil {
+		return fmt.Errorf("while unmarshalling: %w", err)
+	}
+	if icdJson.Icd.LibraryPath == "" {
+		return fmt.Errorf("no library_path value found")
+	}
+	// Here we are implicitly limiting library_path to be a file
+	// name instead of a full path.
+	_, err = filePathInLibDirs(slot, icdJson.Icd.LibraryPath)
+	if err != nil {
+		return err
+	}
+	return nil
+}
+
 func (iface *eglDriverLibsInterface) SymlinksConnectedPlug(spec *symlinks.Specification, plug *interfaces.ConnectedPlug, slot *interfaces.ConnectedSlot) error {
 	var priority int64
 	if err := slot.Attr("priority", &priority); err != nil {
 		return fmt.Errorf("invalid priority: %w", err)
 	}
 
-	icdFiles, err := icdDirFilesCheck(slot)
+	icdPaths, err := sourceDirsCheck(slot,
+		sourceDirAttr{attrName: "icd-source", isOptional: false}, checkEglIcdFile)
 	if err != nil {
 		return fmt.Errorf("invalid icd-source: %w", err)
 	}
 
 	// Create symlinks to snap content (which is fine as this is a super-privileged slot)
-	for _, icdFile := range icdFiles {
-		// Note that icdDirFilesCheck already ensures a .json suffix
+	for _, icdPath := range icdPaths {
+		// Strip out mount dir and snap name and revision
+		relIcdPath, err := filepath.Rel(dirs.SnapMountDir, icdPath)
+		if err != nil {
+			return err
+		}
+		dirs := strings.SplitN(relIcdPath, "/", 3)
+		if len(dirs) < 3 {
+			return fmt.Errorf("internal error: wrong icd file path: %s", relIcdPath)
+		}
+		// Make path an easier to handle name
+		escapedRelPath := systemd.EscapeUnitNamePath(filepath.Join(dirs[2]))
+		// Note that icdFilePathsCheck already ensures a .json suffix
 		linkPath := filepath.Join(eglVendorPath, fmt.Sprintf("%d_snap_%s_%s_%s",
-			priority, slot.Snap().InstanceName(), slot.Name(), filepath.Base(icdFile)))
-		if err := spec.AddSymlink(icdFile, linkPath); err != nil {
+			priority, slot.Snap().InstanceName(), slot.Name(), escapedRelPath))
+		if err := spec.AddSymlink(icdPath, linkPath); err != nil {
 			return err
 		}
 	}
@@ -134,6 +171,20 @@ func (iface *eglDriverLibsInterface) SymlinksConnectedPlug(spec *symlinks.Specif
 	return nil
 }
 
+func (t *eglDriverLibsInterface) PathPatterns() []string {
+	return []string{systemLibrarySourcePath("*", "*", eglDriverLibs)}
+}
+
+func (iface *eglDriverLibsInterface) ConfigfilesConnectedPlug(spec *configfiles.Specification, plug *interfaces.ConnectedPlug, slot *interfaces.ConnectedSlot) error {
+	// Files used by snap-confine on classic
+	if release.OnClassic {
+		if err := addConfigfilesForSystemLibrarySourcePaths(eglDriverLibs, spec, slot); err != nil {
+			return err
+		}
+	}
+	return nil
+}
+
 func (iface *eglDriverLibsInterface) AutoConnect(*snap.PlugInfo, *snap.SlotInfo) bool {
 	// TODO This might need changes when we support plugs in non-system
 	// snaps for this interface.
@@ -143,7 +194,7 @@ func (iface *eglDriverLibsInterface) AutoConnect(*snap.PlugInfo, *snap.SlotInfo)
 func init() {
 	registerIface(&eglDriverLibsInterface{
 		commonInterface: commonInterface{
-			name:                 "egl-driver-libs",
+			name:                 eglDriverLibs,
 			summary:              eglDriverLibsSummary,
 			baseDeclarationPlugs: eglDriverLibsBaseDeclarationPlugs,
 			baseDeclarationSlots: eglDriverLibsBaseDeclarationSlots,
