@@ -20,6 +20,7 @@
 package builtin_test
 
 import (
+	"fmt"
 	"os"
 	"path/filepath"
 
@@ -28,8 +29,10 @@ import (
 	"github.com/snapcore/snapd/dirs"
 	"github.com/snapcore/snapd/interfaces"
 	"github.com/snapcore/snapd/interfaces/builtin"
+	"github.com/snapcore/snapd/interfaces/configfiles"
 	"github.com/snapcore/snapd/interfaces/ldconfig"
 	"github.com/snapcore/snapd/interfaces/symlinks"
+	"github.com/snapcore/snapd/osutil"
 	"github.com/snapcore/snapd/snap"
 	"github.com/snapcore/snapd/testutil"
 )
@@ -64,10 +67,14 @@ apps:
 const eglDriverLibsProvider = `name: egl-provider
 version: 0
 slots:
-  egl-driver-libs:
+  egl-slot:
+    interface: egl-driver-libs
     priority: 10
     compatibility: egl-1-5-ubuntu-2404
-    icd-source: $SNAP/egl.d/
+    icd-source:
+      - $SNAP/egl.d/
+      - $SNAP/egl_alt.d/
+      - $SNAP/egl_empty.d/
     library-source:
       - $SNAP/lib1
       - ${SNAP}/lib2
@@ -77,13 +84,14 @@ func (s *EglDriverLibsInterfaceSuite) SetUpTest(c *C) {
 	s.BaseTest.SetUpTest(c)
 
 	s.testRoot = c.MkDir()
+	os.MkdirAll(filepath.Join(s.testRoot, dirs.DefaultSnapMountDir), 0755)
 	dirs.SetRootDir(s.testRoot)
 	s.AddCleanup(func() { dirs.SetRootDir("/") })
 
 	s.plug, s.plugInfo = MockConnectedPlug(c, eglDriverLibsConsumerYaml,
 		&snap.SideInfo{Revision: snap.R(3)}, "egl")
 	s.slot, s.slotInfo = MockConnectedSlot(c, eglDriverLibsProvider,
-		&snap.SideInfo{Revision: snap.R(5)}, "egl-driver-libs")
+		&snap.SideInfo{Revision: snap.R(5)}, "egl-slot")
 }
 
 func (s *EglDriverLibsInterfaceSuite) TestName(c *C) {
@@ -91,7 +99,7 @@ func (s *EglDriverLibsInterfaceSuite) TestName(c *C) {
 }
 
 func (s *EglDriverLibsInterfaceSuite) TestSanitizeSlot(c *C) {
-	libDir1 := filepath.Join(dirs.GlobalRootDir, "snap/egl-provider/5/lib1")
+	libDir1 := filepath.Join(dirs.SnapMountDir, "egl-provider/5/lib1")
 	c.Assert(os.MkdirAll(libDir1, 0755), IsNil)
 	c.Assert(os.WriteFile(filepath.Join(libDir1, "libEGL_nvidia.so.0"), []byte(``), 0644), IsNil)
 	c.Assert(interfaces.BeforePrepareSlot(s.iface, s.slotInfo), IsNil)
@@ -105,12 +113,13 @@ slots:
     interface: egl-driver-libs
     priority: 10
     compatibility: egl-1-5-ubuntu-2404
-    icd-source: $SNAP/egl.d/
+    icd-source:
+      - $SNAP/egl.d/
     library-source:
       - /snap/egl-provider/current/lib1
 `, nil, "egl")
 	c.Assert(interfaces.BeforePrepareSlot(s.iface, slot), ErrorMatches,
-		`egl-driver-libs source directory .* must start with \$SNAP/ or \$\{SNAP\}/`)
+		`egl-driver-libs library-source directory .* must start with \$SNAP/ or \$\{SNAP\}/`)
 
 	slot = MockSlot(c, `name: egl-provider
 version: 0
@@ -118,7 +127,8 @@ slots:
   egl:
     priority: 10
     compatibility: egl-1-5-ubuntu-2404
-    icd-source: $SNAP/egl.d/
+    icd-source:
+      - $SNAP/egl.d/
     interface: egl-driver-libs
 `, nil, "egl")
 	c.Assert(interfaces.BeforePrepareSlot(s.iface, slot), ErrorMatches,
@@ -131,7 +141,8 @@ slots:
     interface: egl-driver-libs
     priority: 10
     compatibility: egl-1-5-ubuntu-2404
-    icd-source: $SNAP/egl.d/
+    icd-source:
+      - $SNAP/egl.d/
     library-source: $SNAP/lib1
 `, nil, "egl")
 	c.Assert(interfaces.BeforePrepareSlot(s.iface, slot), ErrorMatches,
@@ -143,7 +154,8 @@ slots:
   egl:
     interface: egl-driver-libs
     compatibility: egl-ubuntu-2404
-    icd-source: $SNAP/egl.d/
+    icd-source:
+      - $SNAP/egl.d/
 `, nil, "egl")
 	c.Assert(interfaces.BeforePrepareSlot(s.iface, slot), ErrorMatches,
 		`invalid priority: snap "egl-provider" does not have attribute "priority" for interface "egl-driver-libs"`)
@@ -157,7 +169,7 @@ slots:
     compatibility: egl-1-5-ubuntu-2404
 `, nil, "egl")
 	c.Assert(interfaces.BeforePrepareSlot(s.iface, slot), ErrorMatches,
-		`invalid icd-source: snap "egl-provider" does not have attribute "icd-source" for interface "egl-driver-libs"`)
+		`snap "egl-provider" does not have attribute "icd-source" for interface "egl-driver-libs"`)
 
 	slot = MockSlot(c, `name: egl-provider
 version: 0
@@ -166,7 +178,8 @@ slots:
     interface: egl-driver-libs
     priority: 0
     compatibility: egl-1-5-ubuntu-2404
-    icd-source: $SNAP/egl.d/
+    icd-source:
+      - $SNAP/egl.d/
 `, nil, "egl")
 	c.Assert(interfaces.BeforePrepareSlot(s.iface, slot), ErrorMatches,
 		`priority must be a positive integer`)
@@ -178,10 +191,11 @@ slots:
     interface: egl-driver-libs
     priority: 15
     compatibility: egl-1-5-ubuntu-2404
-    icd-source: /abs/path/egl.d/
+    icd-source:
+      - /abs/path/egl.d/
 `, nil, "egl")
 	c.Assert(interfaces.BeforePrepareSlot(s.iface, slot), ErrorMatches,
-		`source directory "/abs/path/egl.d/" must start with \$SNAP/ or \$\{SNAP\}/`)
+		`egl-driver-libs icd-source directory "/abs/path/egl.d/" must start with \$SNAP/ or \$\{SNAP\}/`)
 
 	slot = MockSlot(c, `name: egl-provider
 version: 0
@@ -189,12 +203,11 @@ slots:
   egl:
     interface: egl-driver-libs
     priority: 15
-    compatibility: egl-ubuntu-2404
-    icd-source:
-      - $SNAP/egl.d/
+    compatibility: egl-1-5-ubuntu-2404
+    icd-source: $SNAP/egl.d/
 `, nil, "egl")
 	c.Assert(interfaces.BeforePrepareSlot(s.iface, slot), ErrorMatches,
-		`invalid icd-source: snap "egl-provider" has interface "egl-driver-libs" with invalid value type \[\]interface {} for "icd-source" attribute: \*string`)
+		`snap "egl-provider" has interface "egl-driver-libs" with invalid value type string for "icd-source" attribute: \*\[\]string`)
 
 	slot = MockSlot(c, `name: egl-provider
 version: 0
@@ -203,7 +216,8 @@ slots:
     interface: egl-driver-libs
     priority: 15
     compatibility: ubuntu
-    icd-source: $SNAP/egl.d/
+    icd-source:
+      - $SNAP/egl.d/
 `, nil, "egl")
 	c.Assert(interfaces.BeforePrepareSlot(s.iface, slot), ErrorMatches,
 		`compatibility label "ubuntu": unexpected number of strings \(should be 2\)`)
@@ -214,7 +228,8 @@ slots:
   egl:
     interface: egl-driver-libs
     priority: 15
-    icd-source: $SNAP/egl.d/
+    icd-source:
+      - $SNAP/egl.d/
 `, nil, "egl")
 	c.Assert(interfaces.BeforePrepareSlot(s.iface, slot), ErrorMatches,
 		`snap "egl-provider" does not have attribute "compatibility" for interface "egl-driver-libs"`)
@@ -229,42 +244,58 @@ func (s *EglDriverLibsInterfaceSuite) TestLdconfigSpec(c *C) {
 	spec := &ldconfig.Specification{}
 	c.Assert(spec.AddConnectedPlug(s.iface, s.plug, s.slot), IsNil)
 	c.Check(spec.LibDirs(), DeepEquals, map[ldconfig.SnapSlot][]string{
-		{SnapName: "egl-provider", SlotName: "egl-driver-libs"}: {
-			filepath.Join(dirs.GlobalRootDir, "snap/egl-provider/5/lib1"),
-			filepath.Join(dirs.GlobalRootDir, "snap/egl-provider/5/lib2")}})
+		{SnapName: "egl-provider", SlotName: "egl-slot"}: {
+			filepath.Join(dirs.SnapMountDir, "egl-provider/5/lib1"),
+			filepath.Join(dirs.SnapMountDir, "egl-provider/5/lib2")}})
 }
 
 func (s *EglDriverLibsInterfaceSuite) TestSymlinksSpec(c *C) {
-	// Write ICD file
-	icdDir := filepath.Join(dirs.GlobalRootDir, "snap/egl-provider/5/egl.d")
-	c.Assert(os.MkdirAll(icdDir, 0755), IsNil)
-	icdPath := filepath.Join(icdDir, "nvidia.json")
-	os.WriteFile(icdPath, []byte(`{
+	// Write ICD files
+	expected := symlinks.SymlinkToTarget{}
+	for _, icdData := range []struct {
+		gpu    string
+		subDir string
+	}{{"mesa", "egl.d"}, {"nvidia", "egl.d"}, {"radeon", "egl_alt.d"}} {
+		icdDir := filepath.Join(dirs.SnapMountDir, "egl-provider/5", icdData.subDir)
+		c.Assert(os.MkdirAll(icdDir, 0755), IsNil)
+		icdPath := filepath.Join(icdDir, icdData.gpu+".json")
+		os.WriteFile(icdPath, []byte(fmt.Sprintf(`{
     "file_format_version" : "1.0.0",
     "ICD" : {
-        "library_path" : "libEGL_nvidia.so.0"
+        "library_path" : "libEGL_%s.so.0"
     }
 }
-`), 0655)
-	libDir := filepath.Join(dirs.GlobalRootDir, "snap/egl-provider/5/lib2")
-	c.Assert(os.MkdirAll(libDir, 0755), IsNil)
-	libPath := filepath.Join(libDir, "libEGL_nvidia.so.0")
-	os.WriteFile(libPath, []byte{}, 0655)
+`, icdData.gpu)), 0655)
+		libDir := filepath.Join(dirs.SnapMountDir, "egl-provider/5/lib2")
+		c.Assert(os.MkdirAll(libDir, 0755), IsNil)
+		libPath := filepath.Join(libDir, "libEGL_"+icdData.gpu+".so.0")
+		os.WriteFile(libPath, []byte{}, 0655)
 
-	// Ignored file
-	otherPath := filepath.Join(icdDir, "foo.bar")
-	os.WriteFile(otherPath, []byte{}, 0655)
+		// Ignored file
+		otherPath := filepath.Join(icdDir, "foo.bar")
+		os.WriteFile(otherPath, []byte{}, 0655)
 
-	// Ignored symlink
-	os.Symlink("not_exists", filepath.Join(icdDir, "foo.json"))
+		// Ignored symlink
+		os.Symlink("not_exists", filepath.Join(icdDir, "foo.json"))
+
+		expected["10_snap_egl-provider_egl-slot_"+icdData.subDir+"-"+icdData.gpu+".json"] = icdPath
+	}
 
 	// Now check symlinks to be created
 	spec := &symlinks.Specification{}
 	c.Assert(spec.AddConnectedPlug(s.iface, s.plug, s.slot), IsNil)
 	c.Check(spec.Symlinks(), DeepEquals, map[string]symlinks.SymlinkToTarget{
-		"/etc/glvnd/egl_vendor.d": {
-			"10_snap_egl-provider_egl-driver-libs_nvidia.json": icdPath,
-		},
+		"/etc/glvnd/egl_vendor.d": expected,
+	})
+}
+
+func (s *EglDriverLibsInterfaceSuite) TestConfigfilesSpec(c *C) {
+	spec := &configfiles.Specification{}
+	c.Assert(spec.AddConnectedPlug(s.iface, s.plug, s.slot), IsNil)
+	c.Check(spec.PathContent(), DeepEquals, map[string]osutil.FileState{
+		filepath.Join(dirs.GlobalRootDir, "/var/lib/snapd/export/system_egl-provider_egl-slot_egl-driver-libs.library-source"): &osutil.MemoryFileState{
+			Content: []byte(filepath.Join(dirs.SnapMountDir, "egl-provider/5/lib1") + "\n" +
+				filepath.Join(dirs.SnapMountDir, "egl-provider/5/lib2") + "\n"), Mode: 0644},
 	})
 }
 
@@ -276,7 +307,7 @@ func (s *EglDriverLibsInterfaceSuite) TestTrackedDirectories(c *C) {
 
 func (s *EglDriverLibsInterfaceSuite) TestSymlinksSpecNoLibrary(c *C) {
 	// Write ICD file
-	icdDir := filepath.Join(dirs.GlobalRootDir, "snap/egl-provider/5/egl.d")
+	icdDir := filepath.Join(dirs.SnapMountDir, "egl-provider/5/egl.d")
 	c.Assert(os.MkdirAll(icdDir, 0755), IsNil)
 	icdPath := filepath.Join(icdDir, "nvidia.json")
 	os.WriteFile(icdPath, []byte(`{
@@ -290,12 +321,12 @@ func (s *EglDriverLibsInterfaceSuite) TestSymlinksSpecNoLibrary(c *C) {
 	// Now check symlinks to be created
 	spec := &symlinks.Specification{}
 	c.Assert(spec.AddConnectedPlug(s.iface, s.plug, s.slot), ErrorMatches,
-		`invalid icd-source: "libEGL_nvidia.so.0" not found in the library-source directories`)
+		`invalid icd-source: nvidia.json: "libEGL_nvidia.so.0" not found in the library-source directories`)
 }
 
 func (s *EglDriverLibsInterfaceSuite) TestSymlinksSpecBadJson(c *C) {
 	// Write ICD file
-	icdDir := filepath.Join(dirs.GlobalRootDir, "snap/egl-provider/5/egl.d")
+	icdDir := filepath.Join(dirs.SnapMountDir, "egl-provider/5/egl.d")
 	c.Assert(os.MkdirAll(icdDir, 0755), IsNil)
 	icdPath := filepath.Join(icdDir, "nvidia.json")
 	os.WriteFile(icdPath, []byte(`libEGL_nvidia.so.0`), 0655)
@@ -303,7 +334,7 @@ func (s *EglDriverLibsInterfaceSuite) TestSymlinksSpecBadJson(c *C) {
 	// Now check symlinks to be created
 	spec := &symlinks.Specification{}
 	c.Assert(spec.AddConnectedPlug(s.iface, s.plug, s.slot), ErrorMatches,
-		`invalid icd-source: while unmarshalling nvidia.json: invalid character 'l' looking for beginning of value`)
+		`invalid icd-source: nvidia.json: while unmarshalling: invalid character 'l' looking for beginning of value`)
 }
 
 func (s *EglDriverLibsInterfaceSuite) TestStaticInfo(c *C) {
