@@ -149,6 +149,7 @@ func (s *baseDeclSuite) TestAutoConnection(c *C) {
 		"desktop":                true,
 		"egl-driver-libs":        true,
 		"gbm-driver-libs":        true,
+		"vulkan-driver-libs":     true,
 		"home":                   true,
 		"lxd-support":            true,
 		"microstack-support":     true,
@@ -831,6 +832,7 @@ var (
 		"hidraw":                    {"core", "gadget"},
 		"i2c":                       {"core", "gadget"},
 		"iio":                       {"core", "gadget"},
+		"iscsi-initiator":           {"core"},
 		"kernel-module-load":        {"core"},
 		"kubernetes-support":        {"core"},
 		"location-control":          {"app"},
@@ -843,10 +845,12 @@ var (
 		"modem-manager":             {"app", "core"},
 		"mount-control":             {"core"},
 		"mpris":                     {"app"},
+		"dm-multipath":              {"core"},
 		"netlink-driver":            {"core", "gadget"},
 		"network-manager":           {"app", "core"},
 		"network-manager-observe":   {"app", "core"},
 		"network-status":            {"core"},
+		"nvme-control":              {"core"},
 		"ofono":                     {"app", "core"},
 		"online-accounts-service":   {"app"},
 		"power-control":             {"core"},
@@ -873,6 +877,7 @@ var (
 		"unity8-calendar":           {"app"},
 		"unity8-contacts":           {"app"},
 		"upower-observe":            {"app", "core"},
+		"usb-gadget":                {"core"},
 		"userns":                    {"core"},
 		"wayland":                   {"app", "core"},
 		"x11":                       {"app", "core"},
@@ -883,6 +888,7 @@ var (
 		"docker":               nil,
 		"egl-driver-libs":      nil,
 		"gbm-driver-libs":      nil,
+		"vulkan-driver-libs":   nil,
 		"lxd":                  nil,
 		"microceph":            nil,
 		"microceph-support":    nil,
@@ -1053,9 +1059,11 @@ func (s *baseDeclSuite) TestPlugInstallation(c *C) {
 		"docker-support":                   true,
 		"egl-driver-libs":                  true,
 		"gbm-driver-libs":                  true,
+		"vulkan-driver-libs":               true,
 		"greengrass-support":               true,
 		"gpio-control":                     true,
 		"ion-memory-control":               true,
+		"iscsi-initiator":                  true,
 		"kernel-firmware-control":          true,
 		"kernel-module-control":            true,
 		"kernel-module-load":               true,
@@ -1065,8 +1073,10 @@ func (s *baseDeclSuite) TestPlugInstallation(c *C) {
 		"microstack-support":               true,
 		"mount-control":                    true,
 		"multipass-support":                true,
+		"dm-multipath":                     true,
 		"nomad-support":                    true,
 		"nvidia-drivers-support":           true,
+		"nvme-control":                     true,
 		"opengl-driver-libs":               true,
 		"opengles-driver-libs":             true,
 		"packagekit-control":               true,
@@ -1364,10 +1374,12 @@ func (s *baseDeclSuite) TestValidity(c *C) {
 		"dm-crypt":                         true,
 		"docker-support":                   true,
 		"egl-driver-libs":                  true,
+		"vulkan-driver-libs":               true,
 		"gbm-driver-libs":                  true,
 		"greengrass-support":               true,
 		"gpio-control":                     true,
 		"ion-memory-control":               true,
+		"iscsi-initiator":                  true,
 		"kernel-firmware-control":          true,
 		"kernel-module-control":            true,
 		"kernel-module-load":               true,
@@ -1377,8 +1389,10 @@ func (s *baseDeclSuite) TestValidity(c *C) {
 		"microstack-support":               true,
 		"mount-control":                    true,
 		"multipass-support":                true,
+		"dm-multipath":                     true,
 		"nomad-support":                    true,
 		"nvidia-drivers-support":           true,
+		"nvme-control":                     true,
 		"opengl-driver-libs":               true,
 		"opengles-driver-libs":             true,
 		"packagekit-control":               true,
@@ -1406,6 +1420,7 @@ func (s *baseDeclSuite) TestValidity(c *C) {
 		"udisks2":                          true,
 		"uinput":                           true,
 		"unity8":                           true,
+		"usb-gadget":                       true,
 		"userns":                           true,
 		"wayland":                          true,
 		"xilinx-dma":                       true,
