@@ -75,29 +75,21 @@ type Prompt struct {
 
 // jsonPrompt defines the marshalled json structure of a Prompt.
 type jsonPrompt struct {
-	ID          prompting.IDType       `json:"id"`
-	Timestamp   time.Time              `json:"timestamp"`
-	Snap        string                 `json:"snap"`
-	PID         int32                  `json:"pid"`
-	Cgroup      string                 `json:"cgroup"`
-	Interface   string                 `json:"interface"`
-	Constraints *jsonPromptConstraints `json:"constraints"`
-}
-
-// jsonPromptConstraints defines the marshalled json structure of promptConstraints.
-type jsonPromptConstraints struct {
-	Path                 string   `json:"path"`
-	RequestedPermissions []string `json:"requested-permissions"`
-	AvailablePermissions []string `json:"available-permissions"`
+	ID          prompting.IDType `json:"id"`
+	Timestamp   time.Time        `json:"timestamp"`
+	Snap        string           `json:"snap"`
+	PID         int32            `json:"pid"`
+	Cgroup      string           `json:"cgroup"`
+	Interface   string           `json:"interface"`
+	Constraints json.RawMessage  `json:"constraints"`
 }
 
 // MarshalJSON marshals the Prompt to JSON.
 // TODO: consider having instead a MarshalForClient -> json.RawMessage method
 func (p *Prompt) MarshalJSON() ([]byte, error) {
-	constraints := &jsonPromptConstraints{
-		Path:                 p.Constraints.path,
-		RequestedPermissions: p.Constraints.outstandingPermissions,
-		AvailablePermissions: p.Constraints.availablePermissions,
+	constraintsJSON, err := p.Constraints.marshalForInterface(p.Interface)
+	if err != nil {
+		return nil, err
 	}
 	toMarshal := &jsonPrompt{
 		ID:          p.ID,
@@ -106,7 +98,7 @@ func (p *Prompt) MarshalJSON() ([]byte, error) {
 		PID:         p.PID,
 		Cgroup:      p.Cgroup,
 		Interface:   p.Interface,
-		Constraints: constraints,
+		Constraints: constraintsJSON,
 	}
 	return json.Marshal(toMarshal)
 }
@@ -138,7 +130,7 @@ func (p *Prompt) containsListenerRequestID(requestID uint64) bool {
 	})
 }
 
-// TODO: replace this with slices.ContainsFunc once on go 1.21+
+// TODO:GOVERSION: replace this with slices.ContainsFunc once on go 1.21+
 func slicesContainsFunc(s []*listener.Request, f func(r *listener.Request) bool) bool {
 	for _, element := range s {
 		if f(element) {
@@ -215,6 +207,49 @@ type promptConstraints struct {
 	originalPermissions []string
 }
 
+// promptConstraintsJSONHome defines the marshalled json structure of
+// promptConstraints for the home interface.
+type promptConstraintsJSONHome struct {
+	Path                 string   `json:"path"`
+	RequestedPermissions []string `json:"requested-permissions"`
+	AvailablePermissions []string `json:"available-permissions"`
+}
+
+// promptConstraintsJSONCamera defines the marshalled json structure of
+// promptConstraints for the camera interface.
+type promptConstraintsJSONCamera struct {
+	RequestedPermissions []string `json:"requested-permissions"`
+	AvailablePermissions []string `json:"available-permissions"`
+}
+
+func (pc *promptConstraints) MarshalJSON() ([]byte, error) {
+	panic("programmer error: cannot marshal promptConstraints directly; must use marshalForInterface with a given interface")
+}
+
+// marshalForInterface marshals the prompt constraints into JSON with fields
+// corresponding to the given interface.
+func (pc *promptConstraints) marshalForInterface(iface string) ([]byte, error) {
+	switch iface {
+	case "home":
+		constraintsJSON := &promptConstraintsJSONHome{
+			Path:                 pc.path,
+			RequestedPermissions: pc.outstandingPermissions,
+			AvailablePermissions: pc.availablePermissions,
+		}
+		return json.Marshal(constraintsJSON)
+	case "camera":
+		constraintsJSON := &promptConstraintsJSONCamera{
+			RequestedPermissions: pc.outstandingPermissions,
+			AvailablePermissions: pc.availablePermissions,
+		}
+		return json.Marshal(constraintsJSON)
+	default:
+		// This should never occur, as prompts can only be created with known
+		// good interfaces.
+		return nil, fmt.Errorf("internal error: invalid interface: %q", iface)
+	}
+}
+
 // equals returns true if the two prompt constraints apply to the same path and
 // were created with the same originally requested permissions. That implies
 // that the request which triggered the creation of the two prompts were
@@ -419,7 +454,7 @@ func (udb *userPromptDB) timeoutCallback(pdb *PromptDB, user uint32) {
 	expiredPrompts := udb.prompts
 	// Clear all outstanding prompts for the user
 	udb.prompts = nil
-	udb.ids = make(map[prompting.IDType]int) // TODO: clear() once we're on Go 1.21+
+	udb.ids = make(map[prompting.IDType]int) // TODO:GOVERSION: clear() once we're on Go 1.21+
 
 	// Remove the request ID mappings now before unlocking the prompt DB
 	for _, p := range expiredPrompts {
