@@ -19,7 +19,11 @@
 
 package disks
 
-import "fmt"
+import (
+	"errors"
+	"fmt"
+	"strings"
+)
 
 // Options is a set of options used when querying information about
 // partition and disk devices.
@@ -31,6 +35,11 @@ type Options struct {
 
 // Disk is a single physical disk device that contains partitions.
 type Disk interface {
+	// FindMatchingPartitionWithUUID finds a partition with a matching
+	// partition UUID on the disk. If no matching partition is found,
+	// a PartitionNotFoundError will be returned.
+	FindMatchingPartitionWithPartUUID(string) (Partition, error)
+
 	// FindMatchingPartitionWithFsLabel finds the partition with a matching
 	// filesystem label on the disk. Note that for non-ascii labels like
 	// "Some label", the label will be encoded using \x<hex> for potentially
@@ -169,6 +178,13 @@ type Partition struct {
 	// from?
 }
 
+func (p *Partition) hasFilesystemLabel(encodedLabel string) bool {
+	if p.FilesystemType == "vfat" {
+		return strings.EqualFold(p.FilesystemLabel, encodedLabel)
+	}
+	return p.FilesystemLabel == encodedLabel
+}
+
 // MountPointsForPartitionRoot returns all mounts from the mount table which are
 // for the root directory of the specified partition and have matching mount
 // options as specified. Options not specified in the map argument can have any
@@ -196,6 +212,8 @@ func (e PartitionNotFoundError) Error() string {
 		t = "partition label"
 	case "filesystem-label":
 		t = "filesystem label"
+	case "partition-uuid":
+		t = "partition uuid"
 	default:
 		return fmt.Sprintf("searching with unknown search type %q and search query %q did not return a partition", e.SearchType, e.SearchQuery)
 	}
@@ -206,6 +224,7 @@ var (
 	_ = error(PartitionNotFoundError{})
 )
 
+// TODO: simplify this away as now we have only LUKS?
 var deviceMapperBackResolvers = map[string]func(dmUUID, dmName []byte) (dev string, ok bool){}
 
 // RegisterDeviceMapperBackResolver takes a callback function which is used when
@@ -226,3 +245,23 @@ func RegisterDeviceMapperBackResolver(name string, f func(dmUUID, dmName []byte)
 func unregisterDeviceMapperBackResolver(name string) {
 	delete(deviceMapperBackResolvers, name)
 }
+
+// ErrNoDmUUID is returned by DMCryptUUIDFromMountPoint when the device
+// at the mount point is not a device mapper device.
+var ErrNoDmUUID = errors.New("device has no DM_UUID")
+
+// ErrMountPointNotFound is returned by DMCryptUUIDFromMountPoint a path
+// is not a mount point.
+var ErrMountPointNotFound = errors.New("cannot find mount point")
+
+type errMountPointNotFoundImpl struct {
+	path string
+}
+
+func (e errMountPointNotFoundImpl) Error() string {
+	return fmt.Sprintf("cannot find mountpoint %q", e.path)
+}
+
+func (e errMountPointNotFoundImpl) Unwrap() error {
+	return ErrMountPointNotFound
+}
