@@ -268,7 +268,7 @@ func (s *confdbTestSuite) TestUnsetView(c *C) {
 	c.Assert(err, IsNil)
 
 	val, err := bag.Get(parsePath(c, "wifi.ssid"))
-	c.Assert(err, FitsTypeOf, confdb.PathError(""))
+	c.Assert(err, testutil.ErrorIs, &confdb.NoDataError{})
 	c.Assert(val, Equals, nil)
 }
 
@@ -1079,7 +1079,7 @@ func (s *confdbTestSuite) mockConfdbHooks(c *C) (*[]string, func()) {
 			return nil, err
 		}
 
-		if !confdbstate.IsConfdbHook(ctx) {
+		if !confdbstate.IsConfdbHookCtx(ctx) {
 			// ignore non-confdb hooks
 			return nil, nil
 		}
