@@ -2,7 +2,7 @@
 //go:build !nomanagers
 
 /*
- * Copyright (C) 2020-2022 Canonical Ltd
+ * Copyright (C) 2020-2024 Canonical Ltd
  *
  * This program is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License version 3 as
@@ -66,16 +66,28 @@ func init() {
 
 	// kernel.{,dangerous-}cmdline-append
 	addWithStateHandler(validateCmdlineAppend, handleCmdlineAppend, &flags{modeenvOnlyConfig: true})
+
+	// debug.snapd.log
+	addWithStateHandler(validateDebugSnapdLogSetting, handleDebugSnapdLogConfiguration, nil)
+
+	// debug.systemd.log-level
+	addWithStateHandler(validateDebugSystemdLogLevelSetting, handleDebugSystemdLogLevelConfiguration, coreOnly)
+
+	// experimental.apparmor-prompting
+	addWithStateHandler(nil, doExperimentalApparmorPromptingDaemonRestart, nil)
+
+	// interface.*.allow-auto-connection
+	addWithStateHandler(validateAllowAutoConnectionValue, nil, &flags{validatedOnlyStateConfig: true})
 }
 
 // RunTransaction is an interface describing how to access
 // the system configuration state and transaction.
 type RunTransaction interface {
-	Get(snapName, key string, result interface{}) error
-	GetMaybe(snapName, key string, result interface{}) error
-	GetPristine(snapName, key string, result interface{}) error
+	Get(snapName, key string, result any) error
+	GetMaybe(snapName, key string, result any) error
+	GetPristine(snapName, key string, result any) error
 	Task() *state.Task
-	Set(snapName, key string, value interface{}) error
+	Set(snapName, key string, value any) error
 	Changes() []string
 	State() *state.State
 	Commit()
@@ -160,6 +172,10 @@ func applyHandlers(dev sysconfig.Device, cfg RunTransaction, handlers []configHa
 			if release.OnClassic {
 				return fmt.Errorf("cannot set netplan configuration on classic")
 			}
+		case isInterfaceChange(k):
+			if err := validateInterfaceChange(k); err != nil {
+				return err
+			}
 		case !supportedConfigurations[k]:
 			return fmt.Errorf("cannot set %q: unsupported system option", k)
 		}
@@ -185,7 +201,7 @@ func applyHandlers(dev sysconfig.Device, cfg RunTransaction, handlers []configHa
 	return nil
 }
 
-func Early(dev sysconfig.Device, cfg RunTransaction, values map[string]interface{}) error {
+func Early(dev sysconfig.Device, cfg RunTransaction, values map[string]any) error {
 	early, relevant := applyFilters(func(f flags) filterFunc {
 		return f.earlyConfigFilter
 	}, values)
