@@ -1,7 +1,7 @@
 // -*- Mode: Go; indent-tabs-mode: t -*-
 
 /*
- * Copyright (C) 2016-2022 Canonical Ltd
+ * Copyright (C) 2016-2024 Canonical Ltd
  *
  * This program is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License version 3 as
@@ -24,6 +24,7 @@ import (
 	"fmt"
 	"os"
 	"path/filepath"
+	"strconv"
 	"strings"
 	"testing"
 	"time"
@@ -59,6 +60,7 @@ import (
 	"github.com/snapcore/snapd/overlord/state"
 	"github.com/snapcore/snapd/overlord/storecontext"
 	"github.com/snapcore/snapd/release"
+	"github.com/snapcore/snapd/secboot"
 	"github.com/snapcore/snapd/secboot/keys"
 	"github.com/snapcore/snapd/seed"
 	"github.com/snapcore/snapd/seed/seedtest"
@@ -215,7 +217,7 @@ func (s *deviceMgrBaseSuite) setupBaseTest(c *C, classic bool) {
 	s.AddCleanup(sysdb.MockGenericClassicModel(s.storeSigning.GenericClassicModel))
 
 	s.brands = assertstest.NewSigningAccounts(s.storeSigning)
-	s.brands.Register("my-brand", brandPrivKey, map[string]interface{}{
+	s.brands.Register("my-brand", brandPrivKey, map[string]any{
 		"display-name": "fancy model publisher",
 		"validation":   "certified",
 	})
@@ -310,7 +312,7 @@ func (s *deviceMgrBaseSuite) seeding() {
 	s.state.Set("seeded", false)
 }
 
-func (s *deviceMgrBaseSuite) makeModelAssertionInState(c *C, brandID, model string, extras map[string]interface{}) *asserts.Model {
+func (s *deviceMgrBaseSuite) makeModelAssertionInState(c *C, brandID, model string, extras map[string]any) *asserts.Model {
 	modelAs := s.brands.Model(brandID, model, extras)
 
 	s.setupBrands()
@@ -321,7 +323,7 @@ func (s *deviceMgrBaseSuite) makeModelAssertionInState(c *C, brandID, model stri
 func (s *deviceMgrBaseSuite) setPCModelInState(c *C) {
 	s.state.Lock()
 	defer s.state.Unlock()
-	s.makeModelAssertionInState(c, "canonical", "pc", map[string]interface{}{
+	s.makeModelAssertionInState(c, "canonical", "pc", map[string]any{
 		"architecture": "amd64",
 		"kernel":       "pc-kernel",
 		"gadget":       "pc",
@@ -337,19 +339,19 @@ func (s *deviceMgrBaseSuite) setPCModelInState(c *C) {
 func (s *deviceMgrBaseSuite) setUC20PCModelInState(c *C) {
 	s.state.Lock()
 	defer s.state.Unlock()
-	s.makeModelAssertionInState(c, "canonical", "pc-20", map[string]interface{}{
+	s.makeModelAssertionInState(c, "canonical", "pc-20", map[string]any{
 		"architecture": "amd64",
 		// UC20
 		"grade": "dangerous",
 		"base":  "core20",
-		"snaps": []interface{}{
-			map[string]interface{}{
+		"snaps": []any{
+			map[string]any{
 				"name":            "pc-kernel",
 				"id":              snaptest.AssertedSnapID("pc-kernel"),
 				"type":            "kernel",
 				"default-channel": "20",
 			},
-			map[string]interface{}{
+			map[string]any{
 				"name":            "pc",
 				"id":              snaptest.AssertedSnapID("pc"),
 				"type":            "gadget",
@@ -367,14 +369,14 @@ func (s *deviceMgrBaseSuite) setUC20PCModelInState(c *C) {
 
 func (s *deviceMgrBaseSuite) setupBrands() {
 	assertstatetest.AddMany(s.state, s.brands.AccountsAndKeys("my-brand")...)
-	otherAcct := assertstest.NewAccount(s.storeSigning, "other-brand", map[string]interface{}{
+	otherAcct := assertstest.NewAccount(s.storeSigning, "other-brand", map[string]any{
 		"account-id": "other-brand",
 	}, "")
 	assertstatetest.AddMany(s.state, otherAcct)
 }
 
 func (s *deviceMgrBaseSuite) setupSnapDeclForNameAndID(c *C, name, snapID, publisherID string) {
-	snapDecl, err := s.storeSigning.Sign(asserts.SnapDeclarationType, map[string]interface{}{
+	snapDecl, err := s.storeSigning.Sign(asserts.SnapDeclarationType, map[string]any{
 		"series":       "16",
 		"snap-name":    name,
 		"snap-id":      snapID,
@@ -385,6 +387,36 @@ func (s *deviceMgrBaseSuite) setupSnapDeclForNameAndID(c *C, name, snapID, publi
 	assertstatetest.AddMany(s.state, snapDecl)
 }
 
+func (s *deviceMgrBaseSuite) setupSnapResourcePair(c *C, comp, snapID, publisherID string, resRev, snapRev snap.Revision) {
+	assertion, err := s.storeSigning.Sign(asserts.SnapResourcePairType, map[string]any{
+		"snap-id":           snapID,
+		"resource-name":     comp,
+		"resource-revision": strconv.Itoa(resRev.N),
+		"snap-revision":     strconv.Itoa(snapRev.N),
+		"developer-id":      publisherID,
+		"timestamp":         time.Now().UTC().Format(time.RFC3339),
+	}, nil, "")
+	c.Assert(err, IsNil)
+	assertstatetest.AddMany(s.state, assertion)
+}
+
+func (s *deviceMgrBaseSuite) setupSnapResourceRevision(c *C, file string, comp, snapID, publisherID string, rev snap.Revision) {
+	sha, size, err := asserts.SnapFileSHA3_384(file)
+	c.Assert(err, IsNil)
+
+	assertion, err := s.storeSigning.Sign(asserts.SnapResourceRevisionType, map[string]any{
+		"snap-id":           snapID,
+		"resource-name":     comp,
+		"resource-sha3-384": sha,
+		"resource-size":     fmt.Sprint(size),
+		"resource-revision": strconv.Itoa(rev.N),
+		"developer-id":      publisherID,
+		"timestamp":         time.Now().UTC().Format(time.RFC3339),
+	}, nil, "")
+	c.Assert(err, IsNil)
+	assertstatetest.AddMany(s.state, assertion)
+}
+
 func (s *deviceMgrBaseSuite) setupSnapDecl(c *C, info *snap.Info, publisherID string) {
 	s.setupSnapDeclForNameAndID(c, info.SnapName(), info.SnapID, publisherID)
 }
@@ -393,7 +425,7 @@ func (s *deviceMgrBaseSuite) setupSnapRevisionForFileAndID(c *C, file, snapID, p
 	sha3_384, size, err := asserts.SnapFileSHA3_384(file)
 	c.Assert(err, IsNil)
 
-	snapRev, err := s.storeSigning.Sign(asserts.SnapRevisionType, map[string]interface{}{
+	snapRev, err := s.storeSigning.Sign(asserts.SnapRevisionType, map[string]any{
 		"snap-sha3-384": sha3_384,
 		"snap-size":     fmt.Sprintf("%d", size),
 		"snap-id":       snapID,
@@ -412,7 +444,7 @@ func (s *deviceMgrBaseSuite) setupSnapRevision(c *C, info *snap.Info, publisherI
 func makeSerialAssertionInState(c *C, brands *assertstest.SigningAccounts, st *state.State, brandID, model, serialN string) *asserts.Serial {
 	encDevKey, err := asserts.EncodePublicKey(devKey.PublicKey())
 	c.Assert(err, IsNil)
-	serial, err := brands.Signing(brandID).Sign(asserts.SerialType, map[string]interface{}{
+	serial, err := brands.Signing(brandID).Sign(asserts.SerialType, map[string]any{
 		"brand-id":            brandID,
 		"model":               model,
 		"serial":              serialN,
@@ -430,6 +462,22 @@ func (s *deviceMgrBaseSuite) makeSerialAssertionInState(c *C, brandID, model, se
 	return makeSerialAssertionInState(c, s.brands, s.state, brandID, model, serialN)
 }
 
+func (s *deviceMgrBaseSuite) addKeyToManagerInState(c *C) {
+	device, err := devicestatetest.Device(s.state)
+	c.Assert(err, IsNil)
+
+	err = devicestate.KeypairManager(s.mgr).Put(devKey)
+	c.Assert(err, IsNil)
+
+	err = devicestatetest.SetDevice(s.state, &auth.DeviceState{
+		Brand:  device.Brand,
+		Model:  device.Model,
+		Serial: device.Serial,
+		KeyID:  devKey.PublicKey().ID(),
+	})
+	c.Assert(err, IsNil)
+}
+
 func (s *deviceMgrSuite) SetUpTest(c *C) {
 	classic := false
 	s.setupBaseTest(c, classic)
@@ -725,8 +773,8 @@ func (s *deviceMgrSuite) TestDeviceManagerEnsureBootOkError(c *C) {
 	c.Assert(err, ErrorMatches, "devicemgr: cannot mark boot successful: bootloader err")
 }
 
-func fakeMyModel(extra map[string]interface{}) *asserts.Model {
-	model := map[string]interface{}{
+func fakeMyModel(extra map[string]any) *asserts.Model {
+	model := map[string]any{
 		"type":         "model",
 		"authority-id": "my-brand",
 		"series":       "16",
@@ -744,7 +792,7 @@ func (s *deviceMgrSuite) TestCheckGadget(c *C) {
 
 	s.setupBrands()
 	// model assertion in device context
-	model := fakeMyModel(map[string]interface{}{
+	model := fakeMyModel(map[string]any{
 		"architecture": "amd64",
 		"gadget":       "gadget",
 		"kernel":       "krnl",
@@ -802,7 +850,7 @@ func (s *deviceMgrSuite) TestCheckGadgetOnClassic(c *C) {
 
 	s.setupBrands()
 	// model assertion in device context
-	model := fakeMyModel(map[string]interface{}{
+	model := fakeMyModel(map[string]any{
 		"classic": "true",
 		"gadget":  "gadget",
 	})
@@ -854,7 +902,7 @@ func (s *deviceMgrSuite) TestCheckGadgetOnClassicGadgetNotSpecified(c *C) {
 
 	s.setupBrands()
 	// model assertion in device context
-	model := fakeMyModel(map[string]interface{}{
+	model := fakeMyModel(map[string]any{
 		"classic": "true",
 	})
 	deviceCtx := &snapstatetest.TrivialDeviceContext{DeviceModel: model}
@@ -868,7 +916,7 @@ func (s *deviceMgrSuite) TestCheckGadgetValid(c *C) {
 	defer s.state.Unlock()
 
 	// model assertion in device context
-	model := fakeMyModel(map[string]interface{}{
+	model := fakeMyModel(map[string]any{
 		"architecture": "amd64",
 		"gadget":       "gadget",
 		"kernel":       "krnl",
@@ -901,7 +949,7 @@ func (s *deviceMgrSuite) TestCheckKernel(c *C) {
 	// not on classic without modes
 	release.OnClassic = true
 	// model assertion in device context
-	model := fakeMyModel(map[string]interface{}{
+	model := fakeMyModel(map[string]any{
 		"classic": "true",
 	})
 	deviceCtx := &snapstatetest.TrivialDeviceContext{DeviceModel: model}
@@ -911,7 +959,7 @@ func (s *deviceMgrSuite) TestCheckKernel(c *C) {
 
 	s.setupBrands()
 	// model assertion in device context
-	model = fakeMyModel(map[string]interface{}{
+	model = fakeMyModel(map[string]any{
 		"architecture": "amd64",
 		"gadget":       "gadget",
 		"kernel":       "krnl",
@@ -967,20 +1015,20 @@ func (s *deviceMgrSuite) TestCheckKernelOnClassicWithModes(c *C) {
 	kernelInfo := snaptest.MockInfo(c, "{type: kernel, name: pc-kernel, version: 0}", nil)
 
 	// model assertion in device context
-	model := fakeMyModel(map[string]interface{}{
+	model := fakeMyModel(map[string]any{
 		"architecture": "amd64",
 		"classic":      "true",
 		"grade":        "dangerous",
 		"distribution": "ubuntu",
 		"base":         "core22",
-		"snaps": []interface{}{
-			map[string]interface{}{
+		"snaps": []any{
+			map[string]any{
 				"name":            "pc-kernel",
 				"id":              snaptest.AssertedSnapID("pc-kernel"),
 				"type":            "kernel",
 				"default-channel": "22",
 			},
-			map[string]interface{}{
+			map[string]any{
 				"name":            "pc",
 				"id":              snaptest.AssertedSnapID("pc"),
 				"type":            "gadget",
@@ -1013,7 +1061,7 @@ func (s *deviceMgrSuite) TestCanAutoRefreshOnCore(c *C) {
 		Brand: "canonical",
 		Model: "pc",
 	})
-	s.makeModelAssertionInState(c, "canonical", "pc", map[string]interface{}{
+	s.makeModelAssertionInState(c, "canonical", "pc", map[string]any{
 		"architecture": "amd64",
 		"kernel":       "pc-kernel",
 		"gadget":       "pc",
@@ -1053,7 +1101,7 @@ func (s *deviceMgrSuite) TestCanAutoRefreshNoSerialFallback(c *C) {
 		Brand: "canonical",
 		Model: "pc",
 	})
-	s.makeModelAssertionInState(c, "canonical", "pc", map[string]interface{}{
+	s.makeModelAssertionInState(c, "canonical", "pc", map[string]any{
 		"architecture": "amd64",
 		"kernel":       "pc-kernel",
 		"gadget":       "pc",
@@ -1093,7 +1141,7 @@ func (s *deviceMgrSuite) TestCanAutoRefreshOnClassic(c *C) {
 		Brand: "canonical",
 		Model: "pc",
 	})
-	s.makeModelAssertionInState(c, "canonical", "pc", map[string]interface{}{
+	s.makeModelAssertionInState(c, "canonical", "pc", map[string]any{
 		"classic": "true",
 	})
 	c.Check(canAutoRefresh(), Equals, false)
@@ -1179,10 +1227,19 @@ func makeMockRepoWithConnectedSnaps(c *C, st *state.State, info11, core11 *snap.
 		err := repo.AddInterface(iface)
 		c.Assert(err, IsNil)
 	}
-	err := repo.AddSnap(info11)
+
+	info11AppSet, err := interfaces.NewSnapAppSet(info11, nil)
 	c.Assert(err, IsNil)
-	err = repo.AddSnap(core11)
+
+	err = repo.AddAppSet(info11AppSet)
 	c.Assert(err, IsNil)
+
+	core11AppSet, err := interfaces.NewSnapAppSet(core11, nil)
+	c.Assert(err, IsNil)
+
+	err = repo.AddAppSet(core11AppSet)
+	c.Assert(err, IsNil)
+
 	_, err = repo.Connect(&interfaces.ConnRef{
 		PlugRef: interfaces.PlugRef{Snap: info11.InstanceName(), Name: ifname},
 		SlotRef: interfaces.SlotRef{Snap: core11.InstanceName(), Name: ifname},
@@ -1404,7 +1461,7 @@ func (s *deviceMgrSuite) TestDeviceManagerSystemModeInfoUC18(c *C) {
 	defer s.state.Unlock()
 
 	// have a model
-	s.makeModelAssertionInState(c, "canonical", "pc", map[string]interface{}{
+	s.makeModelAssertionInState(c, "canonical", "pc", map[string]any{
 		"architecture": "amd64",
 		"kernel":       "pc-kernel",
 		"gadget":       "pc",
@@ -1748,12 +1805,12 @@ hooks:
  fde-setup:
 `
 
-func (s *deviceMgrSuite) TestHasFdeSetupHook(c *C) {
+func (s *deviceMgrSuite) TestHookKeyProtectorFactory(c *C) {
 	st := s.state
 	st.Lock()
 	defer st.Unlock()
 
-	s.makeModelAssertionInState(c, "canonical", "pc", map[string]interface{}{
+	s.makeModelAssertionInState(c, "canonical", "pc", map[string]any{
 		"architecture": "amd64",
 		"kernel":       "pc-kernel",
 		"gadget":       "pc",
@@ -1772,9 +1829,12 @@ func (s *deviceMgrSuite) TestHasFdeSetupHook(c *C) {
 	} {
 		makeInstalledMockKernelSnap(c, st, tc.kernelYaml)
 
-		hasHook, err := devicestate.DeviceManagerHasFDESetupHook(s.mgr, nil)
-		c.Assert(err, IsNil)
-		c.Check(hasHook, Equals, tc.hasFdeSetupHook)
+		_, err := devicestate.DeviceManagerHookKeyProtectorFactory(s.mgr, nil)
+		if tc.hasFdeSetupHook {
+			c.Assert(err, IsNil)
+		} else {
+			c.Assert(err, testutil.ErrorIs, secboot.ErrNoKeyProtector)
+		}
 	}
 }
 
@@ -1783,7 +1843,7 @@ func (s *deviceMgrSuite) TestHasFdeSetupHookOtherKernel(c *C) {
 	st.Lock()
 	defer st.Unlock()
 
-	s.makeModelAssertionInState(c, "canonical", "pc", map[string]interface{}{
+	s.makeModelAssertionInState(c, "canonical", "pc", map[string]any{
 		"architecture": "amd64",
 		"kernel":       "pc-kernel",
 		"gadget":       "pc",
@@ -1797,13 +1857,11 @@ func (s *deviceMgrSuite) TestHasFdeSetupHookOtherKernel(c *C) {
 	_, otherInfo := snaptest.MakeTestSnapInfoWithFiles(c, kernelYamlWithFdeSetup, nil, otherSI)
 	makeInstalledMockKernelSnap(c, st, kernelYamlNoFdeSetup)
 
-	hasHook, err := devicestate.DeviceManagerHasFDESetupHook(s.mgr, nil)
-	c.Assert(err, IsNil)
-	c.Check(hasHook, Equals, false)
+	_, err := devicestate.DeviceManagerHookKeyProtectorFactory(s.mgr, nil)
+	c.Assert(err, testutil.ErrorIs, secboot.ErrNoKeyProtector)
 
-	hasHook, err = devicestate.DeviceManagerHasFDESetupHook(s.mgr, otherInfo)
+	_, err = devicestate.DeviceManagerHookKeyProtectorFactory(s.mgr, otherInfo)
 	c.Assert(err, IsNil)
-	c.Check(hasHook, Equals, true)
 }
 
 func (s *deviceMgrSuite) TestRunFDESetupHookHappy(c *C) {
@@ -1811,7 +1869,7 @@ func (s *deviceMgrSuite) TestRunFDESetupHookHappy(c *C) {
 
 	st.Lock()
 	makeInstalledMockKernelSnap(c, st, kernelYamlWithFdeSetup)
-	s.makeModelAssertionInState(c, "canonical", "pc", map[string]interface{}{
+	s.makeModelAssertionInState(c, "canonical", "pc", map[string]any{
 		"architecture": "amd64",
 		"kernel":       "pc-kernel",
 		"gadget":       "pc",
@@ -1867,7 +1925,7 @@ func (s *deviceMgrSuite) TestRunFDESetupHookErrors(c *C) {
 
 	st.Lock()
 	makeInstalledMockKernelSnap(c, st, kernelYamlWithFdeSetup)
-	s.makeModelAssertionInState(c, "canonical", "pc", map[string]interface{}{
+	s.makeModelAssertionInState(c, "canonical", "pc", map[string]any{
 		"architecture": "amd64",
 		"kernel":       "pc-kernel",
 		"gadget":       "pc",
@@ -1902,7 +1960,7 @@ func (s *deviceMgrSuite) TestRunFDESetupHookErrorResult(c *C) {
 
 	st.Lock()
 	makeInstalledMockKernelSnap(c, st, kernelYamlWithFdeSetup)
-	s.makeModelAssertionInState(c, "canonical", "pc", map[string]interface{}{
+	s.makeModelAssertionInState(c, "canonical", "pc", map[string]any{
 		"architecture": "amd64",
 		"kernel":       "pc-kernel",
 		"gadget":       "pc",
@@ -2048,7 +2106,7 @@ func (s *deviceMgrSuite) TestCanAutoRefreshNTP(c *C) {
 
 	// CanAutoRefresh is ready
 	s.state.Set("seeded", true)
-	s.makeModelAssertionInState(c, "canonical", "pc", map[string]interface{}{
+	s.makeModelAssertionInState(c, "canonical", "pc", map[string]any{
 		"architecture": "amd64",
 		"kernel":       "pc-kernel",
 		"gadget":       "pc",
@@ -2175,53 +2233,96 @@ func (s *deviceMgrSuite) TestDeviceManagerEnsurePostFactoryResetEncrypted(c *C)
 `)
 	c.Assert(os.WriteFile(filepath.Join(dirs.SnapDeviceDir, "factory-reset"), factoryResetMarkercontent, 0644), IsNil)
 
-	completeCalls := 0
-	restore := devicestate.MockMarkFactoryResetComplete(func(encrypted bool) error {
-		completeCalls++
-		c.Check(encrypted, Equals, true)
+	transitionCalls := 0
+	restore := devicestate.MockSecbootTransitionEncryptionKeyChange(func(mountpoint string, key keys.EncryptionKey) error {
+		transitionCalls++
 		return nil
 	})
 	defer restore()
-	transitionCalls := 0
-	restore = devicestate.MockSecbootTransitionEncryptionKeyChange(func(mountpoint string, key keys.EncryptionKey) error {
-		transitionCalls++
+
+	restore = devicestate.MockDisksDMCryptUUIDFromMountPoint(func(mountpoint string) (string, error) {
 		c.Check(mountpoint, Equals, boot.InitramfsUbuntuSaveDir)
-		c.Check(key, DeepEquals, keys.EncryptionKey([]byte("save-key")))
+		return "FOOUUID", nil
+	})
+	defer restore()
+
+	removedOldHandles := 0
+	restore = devicestate.MockSecbootRemoveOldCounterHandles(func(node string, possibleOldKeys map[string]bool, possibleKeyFiles []string, hintExpectFDEHook bool) error {
+		removedOldHandles++
+		c.Check(hintExpectFDEHook, Equals, false)
+		c.Check(node, Equals, "/dev/disk/by-uuid/FOOUUID")
+		c.Check(possibleOldKeys, DeepEquals, map[string]bool{
+			"reprovision-default-fallback": true,
+		})
+		switch removedOldHandles {
+		case 1:
+			c.Check(possibleKeyFiles, DeepEquals, []string{
+				filepath.Join(boot.InitramfsSeedEncryptionKeyDir, "ubuntu-save.recovery.sealed-key"),
+			})
+		case 2:
+			c.Check(possibleKeyFiles, IsNil)
+		default:
+			c.Errorf("unexpected call")
+			return fmt.Errorf("unexpected call")
+		}
+		return nil
+	})
+	defer restore()
+
+	deleteOldSaveKey := 0
+	restore = devicestate.MockSecbootDeleteKeys(func(node string, matches map[string]bool) error {
+		c.Check(node, Equals, "/dev/disk/by-uuid/FOOUUID")
+		c.Check(matches, DeepEquals, map[string]bool{
+			"reprovision-default":          true,
+			"reprovision-default-fallback": true,
+		})
+		deleteOldSaveKey++
+		return nil
+	})
+	defer restore()
+
+	removeOldDiskKeys := 0
+	restore = devicestate.MockSecbootDeleteOldKeys(func(devicePath string) error {
+		c.Check(devicePath, Equals, "/dev/disk/by-uuid/FOOUUID")
+		removeOldDiskKeys++
 		return nil
 	})
 	defer restore()
 
 	err = s.mgr.Ensure()
 	c.Assert(err, IsNil)
-
-	c.Check(completeCalls, Equals, 1)
-	c.Check(transitionCalls, Equals, 1)
+	c.Check(transitionCalls, Equals, 0)
+	c.Check(deleteOldSaveKey, Equals, 1)
+	c.Check(removeOldDiskKeys, Equals, 1)
 	// factory reset marker is gone, the key was verified successfully
 	c.Check(filepath.Join(dirs.SnapDeviceDir, "factory-reset"), testutil.FileAbsent)
 	c.Check(filepath.Join(dirs.SnapFDEDir, "marker"), testutil.FilePresent)
+	c.Check(filepath.Join(boot.InitramfsSeedEncryptionKeyDir, "ubuntu-save.recovery.sealed-key.factory-reset"), testutil.FileAbsent)
 
-	completeCalls = 0
 	transitionCalls = 0
+	deleteOldSaveKey = 0
+	removeOldDiskKeys = 0
 	// try again, no marker, nothing should happen
 	devicestate.SetPostFactoryResetRan(s.mgr, false)
 	err = s.mgr.Ensure()
 	c.Assert(err, IsNil)
 	// nothing was called
-	c.Check(completeCalls, Equals, 0)
 	c.Check(transitionCalls, Equals, 0)
+	c.Check(deleteOldSaveKey, Equals, 0)
+	c.Check(removedOldHandles, Equals, 1)
+	c.Check(removeOldDiskKeys, Equals, 0)
 
-	// have the marker, but migrate the key as if boot code would do it and
+	// have the marker, but keep the keys rotated as if boot code would do it and
 	// try again, in this setup the marker hash matches the migrated key
-	c.Check(os.Rename(filepath.Join(boot.InitramfsSeedEncryptionKeyDir, "ubuntu-save.recovery.sealed-key.factory-reset"),
-		filepath.Join(boot.InitramfsSeedEncryptionKeyDir, "ubuntu-save.recovery.sealed-key")),
-		IsNil)
 	c.Assert(os.WriteFile(filepath.Join(dirs.SnapDeviceDir, "factory-reset"), factoryResetMarkercontent, 0644), IsNil)
 
 	devicestate.SetPostFactoryResetRan(s.mgr, false)
 	err = s.mgr.Ensure()
 	c.Assert(err, IsNil)
-	c.Check(completeCalls, Equals, 1)
-	c.Check(transitionCalls, Equals, 1)
+	c.Check(transitionCalls, Equals, 0)
+	c.Check(deleteOldSaveKey, Equals, 1)
+	c.Check(removedOldHandles, Equals, 2)
+	c.Check(removeOldDiskKeys, Equals, 1)
 	// the marker was again removed
 	c.Check(filepath.Join(dirs.SnapDeviceDir, "factory-reset"), testutil.FileAbsent)
 }
@@ -2249,18 +2350,9 @@ func (s *deviceMgrSuite) TestDeviceManagerEnsurePostFactoryResetEncryptedError(c
 `)
 	c.Assert(os.WriteFile(filepath.Join(dirs.SnapDeviceDir, "factory-reset"), factoryResetMarkercontent, 0644), IsNil)
 
-	completeCalls := 0
-	restore := devicestate.MockMarkFactoryResetComplete(func(encrypted bool) error {
-		completeCalls++
-		c.Check(encrypted, Equals, true)
-		return nil
-	})
-	defer restore()
-
 	err = s.mgr.Ensure()
 	c.Assert(err, ErrorMatches, "devicemgr: cannot verify factory reset marker: fallback sealed key digest mismatch, got d192153f0a50e826c6eb400c8711750ed0466571df1d151aaecc8c73095da7ec104318e7bf74d5e5ae2940827bf8402b expected uh-oh")
 
-	c.Check(completeCalls, Equals, 0)
 	// factory reset marker is gone, the key was verified successfully
 	c.Check(filepath.Join(dirs.SnapDeviceDir, "factory-reset"), testutil.FilePresent)
 	c.Check(filepath.Join(dirs.SnapFDEDir, "marker"), testutil.FilePresent)
@@ -2269,12 +2361,10 @@ func (s *deviceMgrSuite) TestDeviceManagerEnsurePostFactoryResetEncryptedError(c
 	devicestate.SetPostFactoryResetRan(s.mgr, false)
 	err = s.mgr.Ensure()
 	c.Assert(err, ErrorMatches, "devicemgr: cannot verify factory reset marker: fallback sealed key digest mismatch, got d192153f0a50e826c6eb400c8711750ed0466571df1d151aaecc8c73095da7ec104318e7bf74d5e5ae2940827bf8402b expected uh-oh")
-	c.Check(completeCalls, Equals, 0)
 
 	// and again, but not resetting the 'ran' check, so nothing is checked or called
 	err = s.mgr.Ensure()
 	c.Assert(err, IsNil)
-	c.Check(completeCalls, Equals, 0)
 }
 
 func (s *deviceMgrSuite) TestDeviceManagerEnsurePostFactoryResetUnencrypted(c *C) {
@@ -2290,18 +2380,9 @@ func (s *deviceMgrSuite) TestDeviceManagerEnsurePostFactoryResetUnencrypted(c *C
 	c.Assert(os.MkdirAll(dirs.SnapDeviceDir, 0755), IsNil)
 	c.Assert(os.WriteFile(filepath.Join(dirs.SnapDeviceDir, "factory-reset"), []byte("{}"), 0644), IsNil)
 
-	completeCalls := 0
-	restore := devicestate.MockMarkFactoryResetComplete(func(encrypted bool) error {
-		completeCalls++
-		c.Check(encrypted, Equals, false)
-		return nil
-	})
-	defer restore()
-
 	err := s.mgr.Ensure()
 	c.Assert(err, IsNil)
 
-	c.Check(completeCalls, Equals, 1)
 	// factory reset marker is gone
 	c.Check(filepath.Join(dirs.SnapDeviceDir, "factory-reset"), testutil.FileAbsent)
 
@@ -2309,8 +2390,6 @@ func (s *deviceMgrSuite) TestDeviceManagerEnsurePostFactoryResetUnencrypted(c *C
 	devicestate.SetPostFactoryResetRan(s.mgr, false)
 	err = s.mgr.Ensure()
 	c.Assert(err, IsNil)
-	// nothing was called
-	c.Check(completeCalls, Equals, 1)
 }
 
 func (s *deviceMgrSuite) mockSystemUser(c *C, username string, expiration time.Time) {
@@ -2494,30 +2573,30 @@ volumes:
 	makeSnap("pc=20")
 	optSnapPath := snaptest.MakeTestSnapWithFiles(c, seedtest.SampleSnapYaml["optional20-a"], nil)
 
-	model := map[string]interface{}{
+	model := map[string]any{
 		"display-name": "my model",
 		"architecture": "amd64",
 		"base":         "core20",
 		"grade":        "dangerous",
-		"snaps": []interface{}{
-			map[string]interface{}{
+		"snaps": []any{
+			map[string]any{
 				"name":            "pc-kernel",
 				"id":              seed20.AssertedSnapID("pc-kernel"),
 				"type":            "kernel",
 				"default-channel": "20",
 			},
-			map[string]interface{}{
+			map[string]any{
 				"name":            "pc",
 				"id":              seed20.AssertedSnapID("pc"),
 				"type":            "gadget",
 				"default-channel": "20",
 			},
-			map[string]interface{}{
+			map[string]any{
 				"name": "snapd",
 				"id":   seed20.AssertedSnapID("snapd"),
 				"type": "snapd",
 			},
-			map[string]interface{}{
+			map[string]any{
 				"name": "core20",
 				"id":   seed20.AssertedSnapID("core20"),
 				"type": "base",
@@ -2745,3 +2824,144 @@ func (s *deviceMgrSuite) TestDefaultRecoverySystem(c *C) {
 	c.Assert(err, IsNil)
 	c.Check(*system, Equals, expectedSystem)
 }
+
+func (s *deviceMgrSuite) TestSignConfdbControlNoSerial(c *C) {
+	s.state.Lock()
+	defer s.state.Unlock()
+
+	_, err := s.mgr.SignConfdbControl([]any{}, 2)
+	c.Assert(err, ErrorMatches, "cannot sign confdb-control without a serial")
+}
+
+func (s *deviceMgrSuite) TestSignConfdbControlNoKey(c *C) {
+	s.setPCModelInState(c)
+	s.state.Lock()
+	defer s.state.Unlock()
+
+	s.makeSerialAssertionInState(c, "canonical", "pc", "serialserialserial")
+
+	_, err := s.mgr.SignConfdbControl([]any{}, 3)
+	c.Assert(err, ErrorMatches, "cannot sign confdb-control without device key")
+}
+
+func (s *deviceMgrSuite) TestSignConfdbControlInvalid(c *C) {
+	s.setPCModelInState(c)
+	s.state.Lock()
+	defer s.state.Unlock()
+
+	s.makeSerialAssertionInState(c, "canonical", "pc", "serialserialserial")
+	s.addKeyToManagerInState(c)
+
+	groups := []any{map[string]any{"operators": []any{"jane"}}}
+	_, err := s.mgr.SignConfdbControl(groups, 4)
+	c.Assert(
+		err,
+		ErrorMatches,
+		"cannot assemble assertion confdb-control: cannot parse group at position 1: \"authentications\" must be provided",
+	)
+}
+
+func (s *deviceMgrSuite) TestSignConfdbControlOK(c *C) {
+	s.setPCModelInState(c)
+	s.state.Lock()
+	defer s.state.Unlock()
+
+	s.makeSerialAssertionInState(c, "canonical", "pc", "serialserialserial")
+	s.addKeyToManagerInState(c)
+
+	jane := map[string]any{
+		"operators":       []any{"jane"},
+		"authentications": []any{"operator-key"},
+		"views": []any{
+			"canonical/network/observe-interfaces",
+			"canonical/network/control-interfaces",
+		},
+	}
+	groups := []any{jane}
+
+	cc, err := s.mgr.SignConfdbControl(groups, 5)
+	c.Assert(err, IsNil)
+	c.Assert(cc.Revision(), Equals, 5)
+
+	// Confirm we can ack it
+	assertstatetest.AddMany(s.state, cc)
+}
+
+func (s *deviceMgrSuite) TestConfdbControlNoSerial(c *C) {
+	s.state.Lock()
+	defer s.state.Unlock()
+
+	_, err := s.mgr.ConfdbControl()
+	c.Assert(err, ErrorMatches, "device has no identity yet")
+}
+
+func (s *deviceMgrSuite) TestConfdbControlNotFound(c *C) {
+	s.setPCModelInState(c)
+	s.state.Lock()
+	defer s.state.Unlock()
+
+	s.makeSerialAssertionInState(c, "canonical", "pc", "serialserialserial")
+	s.addKeyToManagerInState(c)
+
+	cc, err := s.mgr.ConfdbControl()
+	c.Assert(cc, IsNil)
+	c.Assert(err, ErrorMatches, "no state entry for key")
+}
+
+func (s *deviceMgrSuite) TestConfbControlUnknownSigningKey(c *C) {
+	s.setPCModelInState(c)
+	s.state.Lock()
+	defer s.state.Unlock()
+
+	s.makeSerialAssertionInState(c, "canonical", "pc", "serialserialserial")
+	s.addKeyToManagerInState(c)
+
+	cc, err := s.mgr.SignConfdbControl([]any{}, 10)
+	c.Assert(err, IsNil)
+	assertstatetest.AddMany(s.state, cc)
+
+	// change the device key
+	anotherKey, _ := assertstest.GenerateKey(testKeyLength)
+	encDevKey, err := asserts.EncodePublicKey(anotherKey.PublicKey())
+	c.Assert(err, IsNil)
+
+	serial, err := s.brands.Signing("canonical").Sign(asserts.SerialType, map[string]any{
+		"brand-id":            "canonical",
+		"model":               "pc",
+		"serial":              "serialserialserial",
+		"device-key":          string(encDevKey),
+		"device-key-sha3-384": anotherKey.PublicKey().ID(),
+		"timestamp":           time.Now().Format(time.RFC3339),
+		"revision":            "1",
+	}, nil, "")
+	c.Assert(err, IsNil)
+
+	err = assertstate.Add(s.state, serial)
+	c.Assert(err, IsNil)
+
+	cc, err = s.mgr.ConfdbControl() // attempt to retrieve it
+	c.Assert(cc, IsNil)
+	c.Assert(err, ErrorMatches, "confdb-control's signing key doesn't match the device key")
+}
+
+func (s *deviceMgrSuite) TestConfdbControlFindExisting(c *C) {
+	s.setPCModelInState(c)
+	s.state.Lock()
+	defer s.state.Unlock()
+
+	s.makeSerialAssertionInState(c, "canonical", "pc", "serialserialserial")
+	s.addKeyToManagerInState(c)
+
+	// add assertion
+	cc, err := s.mgr.SignConfdbControl([]any{}, 10)
+	c.Assert(err, IsNil)
+	assertstatetest.AddMany(s.state, cc)
+
+	found, err := s.mgr.ConfdbControl()
+	c.Assert(err, IsNil)
+	c.Assert(found, DeepEquals, cc)
+}
+
+func (s *deviceMgrSuite) TestEnsureLoopLogging(c *C) {
+	testutil.CheckEnsureLoopLogging("devicemgr.go", c, true)
+}
