@@ -1,7 +1,7 @@
 // -*- Mode: Go; indent-tabs-mode: t -*-
 
 /*
- * Copyright (C) 2015-2019 Canonical Ltd
+ * Copyright (C) 2015-2024 Canonical Ltd
  *
  * This program is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License version 3 as
@@ -238,8 +238,10 @@ func (s *firstbootPreseedingClassic16Suite) SetUpTest(c *C) {
 	err := os.MkdirAll(filepath.Join(dirs.SnapSeedDir, "assertions"), 0755)
 	c.Assert(err, IsNil)
 
+	extraData := interfaces.SystemKeyExtraData{}
+
 	s.AddCleanup(interfaces.MockSystemKey(`{"core": "123"}`))
-	c.Assert(interfaces.WriteSystemKey(), IsNil)
+	c.Assert(interfaces.WriteSystemKey(extraData), IsNil)
 
 	restoreRelease := release.MockOnClassic(true)
 	s.AddCleanup(restoreRelease)
@@ -622,19 +624,19 @@ snaps:
 	c.Check(seedTime.IsZero(), Equals, false)
 
 	// verify that connections was made
-	var conns map[string]interface{}
+	var conns map[string]any
 	c.Assert(st.Get("conns", &conns), IsNil)
-	c.Assert(conns, DeepEquals, map[string]interface{}{
-		"foo:network core:network": map[string]interface{}{
+	c.Assert(conns, DeepEquals, map[string]any{
+		"foo:network core:network": map[string]any{
 			"auto": true, "interface": "network"},
-		"foo:shared-data-plug bar:shared-data-slot": map[string]interface{}{
+		"foo:shared-data-plug bar:shared-data-slot": map[string]any{
 			"auto": true, "interface": "content",
-			"plug-static": map[string]interface{}{
+			"plug-static": map[string]any{
 				"content": "mylib", "target": "import",
 			},
-			"slot-static": map[string]interface{}{
+			"slot-static": map[string]any{
 				"content": "mylib",
-				"read": []interface{}{
+				"read": []any{
 					"/",
 				},
 			},
