@@ -21,13 +21,12 @@ package devicestate
 import (
 	"errors"
 	"fmt"
-	"os/exec"
 	"time"
 
 	"gopkg.in/tomb.v2"
 
+	"github.com/snapcore/snapd/boot"
 	"github.com/snapcore/snapd/interfaces"
-	"github.com/snapcore/snapd/logger"
 	"github.com/snapcore/snapd/overlord/restart"
 	"github.com/snapcore/snapd/overlord/snapstate"
 	"github.com/snapcore/snapd/overlord/state"
@@ -38,11 +37,6 @@ func (m *DeviceManager) doMarkPreseeded(t *state.Task, _ *tomb.Tomb) error {
 	st.Lock()
 	defer st.Unlock()
 
-	snaps, err := snapstate.All(st)
-	if err != nil {
-		return err
-	}
-
 	systemKey, err := interfaces.RecordedSystemKey()
 	if err != nil {
 		return fmt.Errorf("cannot get recorded system key: %v", err)
@@ -61,17 +55,9 @@ func (m *DeviceManager) doMarkPreseeded(t *state.Task, _ *tomb.Tomb) error {
 		if !preseeded {
 			preseeded = true
 			t.Set("preseeded", preseeded)
-			// unmount all snaps
-			// TODO: move to snapstate.UnmountAllSnaps.
-			for _, snapSt := range snaps {
-				info, err := snapSt.CurrentInfo()
-				if err != nil {
-					return err
-				}
-				logger.Debugf("unmounting snap %s at %s", info.InstanceName(), info.MountDir())
-				if _, err := exec.Command("umount", "-d", "-l", info.MountDir()).CombinedOutput(); err != nil {
-					return err
-				}
+
+			if err := snapstate.UnmountAllSnaps(st); err != nil {
+				return err
 			}
 
 			st.Set("preseeded", preseeded)
@@ -154,8 +140,7 @@ func (m *DeviceManager) doMarkSeeded(t *state.Task, _ *tomb.Tomb) error {
 	}
 
 	if deviceCtx.HasModeenv() && deviceCtx.RunMode() {
-		// XXX make this a boot method
-		modeEnv, err := maybeReadModeenv()
+		modeEnv, err := boot.MaybeReadModeenv()
 		if err != nil {
 			return err
 		}
