@@ -34,6 +34,7 @@ import (
 	"github.com/snapcore/snapd/bootloader/bootloadertest"
 	"github.com/snapcore/snapd/dirs"
 	"github.com/snapcore/snapd/gadget/device"
+	"github.com/snapcore/snapd/logger"
 	"github.com/snapcore/snapd/overlord/fdestate/backend"
 	"github.com/snapcore/snapd/secboot"
 	"github.com/snapcore/snapd/testutil"
@@ -136,6 +137,8 @@ type fakeState struct {
 	isUnlocked          bool
 	hasUnlocked         int
 	EncryptedContainers []backend.EncryptedContainer
+	brokenPrimaryKey    bool
+	policyCounterHandle uint32
 }
 
 func (fs *fakeState) Update(role string, containerRole string, params *backend.SealingParameters) error {
@@ -160,6 +163,18 @@ func (fs *fakeState) Get(role string, containerRole string) (params *backend.Sea
 	return p, nil
 }
 
+func (fs *fakeState) RoleInfo(role string) (info *backend.RoleInfo, err error) {
+	info = &backend.RoleInfo{
+		PrimaryKeyID:                   0,
+		TPM2PCRPolicyRevocationCounter: fs.policyCounterHandle,
+	}
+	return info, nil
+}
+
+func (fs *fakeState) VerifyPrimaryKeyAgainstState(primaryKeyID int, primaryKey []byte) bool {
+	return !fs.brokenPrimaryKey
+}
+
 func (fs *fakeState) Unlock() (relock func()) {
 	fs.isUnlocked = true
 	fs.hasUnlocked++
@@ -179,7 +194,18 @@ type fakeSealedKey struct {
 	num int
 }
 
-func (s *resealTestSuite) testTPMResealHappy(c *C, revokeOldKeys bool, missingRunParams bool, missingRecoverParams bool, onClassic bool) {
+type tpmResealHappyCase struct {
+	revokeOldKeys             bool
+	missingRunParams          bool
+	missingRecoverParams      bool
+	onClassic                 bool
+	brokenPrimaryKey          bool
+	brokenRevocationCounter   bool
+	multipleRevocationCounter bool
+	noPrimaryKey              bool
+}
+
+func (s *resealTestSuite) testTPMResealHappy(c *C, tc tpmResealHappyCase) {
 	bl := bootloadertest.Mock("trusted", "").WithTrustedAssets()
 	bootloader.Force(bl)
 	defer bootloader.Force(nil)
@@ -221,11 +247,11 @@ func (s *resealTestSuite) testTPMResealHappy(c *C, revokeOldKeys bool, missingRu
 		bootIsResealNeededCalls++
 		switch bootIsResealNeededCalls {
 		case 1:
-			if missingRunParams {
+			if tc.missingRunParams {
 				return false, 0, nil
 			}
 		case 2:
-			if missingRecoverParams {
+			if tc.missingRecoverParams {
 				return false, 0, nil
 			}
 		}
@@ -234,7 +260,7 @@ func (s *resealTestSuite) testTPMResealHappy(c *C, revokeOldKeys bool, missingRu
 	})()
 
 	var model *asserts.Model
-	if onClassic {
+	if tc.onClassic {
 		model = boottest.MakeMockClassicWithModesModel()
 	} else {
 		model = boottest.MakeMockUC20Model()
@@ -360,14 +386,14 @@ func (s *resealTestSuite) testTPMResealHappy(c *C, revokeOldKeys bool, missingRu
 	restore := backend.MockSecbootBuildPCRProtectionProfile(func(modelParams []*secboot.SealKeyModelParams, allowInsufficientDmaProtection bool) (secboot.SerializedPCRProfile, error) {
 		buildProfileCalls++
 
-		c.Check(allowInsufficientDmaProtection, Equals, !onClassic)
+		c.Check(allowInsufficientDmaProtection, Equals, !tc.onClassic)
 
 		c.Assert(modelParams, HasLen, 1)
 		mp := modelParams[0]
 		c.Check(mp.Model.Model(), Equals, model.Model())
 		switch buildProfileCalls {
 		case 1:
-			if !missingRunParams {
+			if !tc.missingRunParams {
 				c.Check(mp.EFILoadChains, DeepEquals, []*secboot.LoadChain{
 					secboot.NewLoadChain(shimBf,
 						secboot.NewLoadChain(assetBf,
@@ -402,11 +428,6 @@ func (s *resealTestSuite) testTPMResealHappy(c *C, revokeOldKeys bool, missingRu
 	restore = backend.MockSecbootResealKey(func(key secboot.KeyDataLocation, params *secboot.ResealKeyParams) (secboot.UpdatedKeys, error) {
 		resealCalls++
 
-		skippedCalls := 0
-		if missingRunParams {
-			skippedCalls += 1
-		}
-
 		c.Check(params.PrimaryKeyDevices, DeepEquals, []string{"/dev/disk/by-uuid/123", "/dev/disk/by-uuid/456"})
 		c.Check(params.FallbackPrimaryKeyFiles, DeepEquals, []string{
 			filepath.Join(dirs.SnapSaveDir, "device/fde", "aux-key"),
@@ -415,8 +436,12 @@ func (s *resealTestSuite) testTPMResealHappy(c *C, revokeOldKeys bool, missingRu
 
 		c.Logf("key: %v", key)
 
-		c.Check(params.TpmPCRProfile, DeepEquals, []byte(`"serialized-pcr-profile"`))
-		switch resealCalls + skippedCalls {
+		tpmPCRProfile, profileErr := params.GetTpmPCRProfile()
+		if !tc.noPrimaryKey {
+			params.VerifyPrimaryKey([]byte{1, 2, 3, 4})
+		}
+
+		switch resealCalls {
 		case 1:
 			// Resealing the run+recover key for data partition
 			c.Check(key, DeepEquals, secboot.KeyDataLocation{
@@ -424,6 +449,12 @@ func (s *resealTestSuite) testTPMResealHappy(c *C, revokeOldKeys bool, missingRu
 				SlotName:   "default",
 				KeyFile:    filepath.Join(s.rootdir, "run/mnt/ubuntu-boot/device/fde/ubuntu-data.sealed-key"),
 			})
+			if tc.missingRunParams {
+				c.Assert(profileErr, NotNil)
+				return nil, profileErr
+			}
+			c.Assert(profileErr, IsNil)
+			c.Check(tpmPCRProfile, DeepEquals, []byte(`"serialized-pcr-profile"`))
 		case 2:
 			// Resealing the recovery key for both data partition
 			c.Check(key, DeepEquals, secboot.KeyDataLocation{
@@ -431,6 +462,12 @@ func (s *resealTestSuite) testTPMResealHappy(c *C, revokeOldKeys bool, missingRu
 				SlotName:   "default-fallback",
 				KeyFile:    filepath.Join(s.rootdir, "run/mnt/ubuntu-seed/device/fde/ubuntu-data.recovery.sealed-key"),
 			})
+			if tc.missingRecoverParams {
+				c.Assert(profileErr, NotNil)
+				return nil, profileErr
+			}
+			c.Assert(profileErr, IsNil)
+			c.Check(tpmPCRProfile, DeepEquals, []byte(`"serialized-pcr-profile"`))
 		case 3:
 			// Resealing the recovery key for both save partition
 			c.Check(key, DeepEquals, secboot.KeyDataLocation{
@@ -438,6 +475,12 @@ func (s *resealTestSuite) testTPMResealHappy(c *C, revokeOldKeys bool, missingRu
 				SlotName:   "default-fallback",
 				KeyFile:    filepath.Join(s.rootdir, "run/mnt/ubuntu-seed/device/fde/ubuntu-save.recovery.sealed-key"),
 			})
+			if tc.missingRecoverParams {
+				c.Assert(profileErr, NotNil)
+				return nil, profileErr
+			}
+			c.Assert(profileErr, IsNil)
+			c.Check(tpmPCRProfile, DeepEquals, []byte(`"serialized-pcr-profile"`))
 		default:
 			c.Errorf("unexpected additional call to secboot.ResealKey (call # %d)", resealCalls)
 		}
@@ -445,7 +488,14 @@ func (s *resealTestSuite) testTPMResealHappy(c *C, revokeOldKeys bool, missingRu
 	})
 	defer restore()
 
-	myState := &fakeState{}
+	myState := &fakeState{
+		brokenPrimaryKey:    tc.brokenPrimaryKey,
+		policyCounterHandle: 42,
+	}
+	if tc.brokenRevocationCounter {
+		myState.policyCounterHandle = 43
+	}
+
 	myState.EncryptedContainers = []backend.EncryptedContainer{
 		&encryptedContainer{
 			uuid:          "123",
@@ -465,10 +515,11 @@ func (s *resealTestSuite) testTPMResealHappy(c *C, revokeOldKeys bool, missingRu
 	}
 
 	defer backend.MockSecbootRevokeOldKeys(func(uk *secboot.UpdatedKeys, primaryKey []byte) error {
-		if !revokeOldKeys {
+		if !tc.revokeOldKeys {
 			c.Errorf("unexpected call")
 			return fmt.Errorf("unexpected call")
 		}
+
 		c.Assert(uk, NotNil)
 		c.Assert(*uk, HasLen, 3)
 		c.Check((*uk)[0].(*fakeSealedKey).num, Equals, 1)
@@ -488,88 +539,152 @@ func (s *resealTestSuite) testTPMResealHappy(c *C, revokeOldKeys bool, missingRu
 		return []byte{1, 2, 3, 4}, nil
 	})()
 
-	opts := boot.ResealKeyToModeenvOptions{ExpectReseal: true, RevokeOldKeys: revokeOldKeys}
+	backend.MockSecbootPCRPolicyCounterHandles(func(uk secboot.UpdatedKeys) []uint32 {
+		if tc.multipleRevocationCounter {
+			return []uint32{41, 42}
+		} else {
+			return []uint32{42}
+		}
+	})
+
+	loggerBuf, restore := logger.MockLogger()
+	defer restore()
+
+	opts := boot.ResealKeyToModeenvOptions{ExpectReseal: true, RevokeOldKeys: tc.revokeOldKeys}
 	err := backend.ResealKeyForBootChains(myState, device.SealingMethodTPM, s.rootdir, &boot.ResealKeyForBootChainsParams{BootChains: bootChains, Options: opts})
+	if tc.noPrimaryKey && tc.revokeOldKeys {
+		c.Assert(err, ErrorMatches, `Missing primary key`)
+		return
+	}
 	c.Assert(err, IsNil)
 
+	if tc.brokenPrimaryKey {
+		c.Check(loggerBuf.String(), testutil.Contains, "WARNING: primary key is not matching the FDE state")
+	} else {
+		c.Check(loggerBuf.String(), Not(testutil.Contains), "WARNING: primary key is not matching the FDE state")
+	}
+	if tc.brokenRevocationCounter {
+		c.Check(loggerBuf.String(), testutil.Contains, "WARNING: policy counter handle 42 is not matching the FDE state")
+	} else {
+		c.Check(loggerBuf.String(), Not(testutil.Contains), "WARNING: policy counter handle .* is not matching the FDE state")
+	}
+	if tc.multipleRevocationCounter {
+		c.Check(loggerBuf.String(), testutil.Contains, "WARNING: policy counter handle 41 is not matching the FDE state")
+	} else {
+		c.Check(loggerBuf.String(), Not(testutil.Contains), "WARNING: policy counter handle .* is not matching the FDE state")
+	}
+
 	c.Assert(bootIsResealNeededCalls, Equals, 2)
 
-	expectedResealCalls := 3
-	if missingRunParams {
-		expectedResealCalls -= 1
-	}
-	if missingRecoverParams {
-		expectedResealCalls -= 2
-	}
-	c.Check(resealCalls, Equals, expectedResealCalls)
+	c.Check(resealCalls, Equals, 3)
 
 	pbc, cnt, err := boot.ReadBootChains(filepath.Join(dirs.SnapFDEDir, "boot-chains"))
 	c.Assert(err, IsNil)
-	if !missingRunParams {
+	if !tc.missingRunParams {
 		c.Assert(cnt, Equals, 1)
 		c.Check(pbc, DeepEquals, boot.ToPredictableBootChains(removeKernelBootFiles(append(bootChains.RunModeBootChains, bootChains.RecoveryBootChainsForRunKey...))))
 	}
 
 	recoveryPbc, cnt, err := boot.ReadBootChains(filepath.Join(dirs.SnapFDEDir, "recovery-boot-chains"))
 	c.Assert(err, IsNil)
-	if !missingRecoverParams {
+	if !tc.missingRecoverParams {
 		c.Check(recoveryPbc, DeepEquals, boot.ToPredictableBootChains(removeKernelBootFiles(bootChains.RecoveryBootChains)))
 		c.Assert(cnt, Equals, 1)
 	}
 }
 
 func (s *resealTestSuite) TestTPMResealHappy(c *C) {
-	const revokeOldKeys = false
-	const missingRunParams = false
-	const missingRecoverParams = false
-	const onClassic = true
-	s.testTPMResealHappy(c, revokeOldKeys, missingRunParams, missingRecoverParams, onClassic)
+	tc := tpmResealHappyCase{
+		onClassic: true,
+	}
+	s.testTPMResealHappy(c, tc)
 }
 
 func (s *resealTestSuite) TestTPMResealHappyCore(c *C) {
-	const revokeOldKeys = false
-	const missingRunParams = false
-	const missingRecoverParams = false
-	const onClassic = false
-	s.testTPMResealHappy(c, revokeOldKeys, missingRunParams, missingRecoverParams, onClassic)
+	tc := tpmResealHappyCase{}
+	s.testTPMResealHappy(c, tc)
 }
 
 func (s *resealTestSuite) TestTPMResealHappyRevoke(c *C) {
-	const revokeOldKeys = true
-	const missingRunParams = false
-	const missingRecoverParams = false
-	const onClassic = true
-	s.testTPMResealHappy(c, revokeOldKeys, missingRunParams, missingRecoverParams, onClassic)
+	tc := tpmResealHappyCase{
+		onClassic:     true,
+		revokeOldKeys: true,
+	}
+	s.testTPMResealHappy(c, tc)
 }
 
 func (s *resealTestSuite) TestTPMResealHappyRevokeMissingRunParams(c *C) {
-	const revokeOldKeys = true
-	const missingRunParams = true
-	const missingRecoverParams = false
-	const onClassic = true
-	s.testTPMResealHappy(c, revokeOldKeys, missingRunParams, missingRecoverParams, onClassic)
+	tc := tpmResealHappyCase{
+		onClassic:        true,
+		revokeOldKeys:    true,
+		missingRunParams: true,
+	}
+	s.testTPMResealHappy(c, tc)
 }
 
 func (s *resealTestSuite) TestTPMResealHappyRevokeMissingRecoverParams(c *C) {
-	const revokeOldKeys = true
-	const missingRunParams = false
-	const missingRecoverParams = true
-	const onClassic = true
-	s.testTPMResealHappy(c, revokeOldKeys, missingRunParams, missingRecoverParams, onClassic)
+	tc := tpmResealHappyCase{
+		onClassic:            true,
+		revokeOldKeys:        true,
+		missingRecoverParams: true,
+	}
+	s.testTPMResealHappy(c, tc)
 }
 
 func (s *resealTestSuite) TestTPMResealHappyRevokeMissingParams(c *C) {
-	const revokeOldKeys = true
-	const missingRunParams = true
-	const missingRecoverParams = true
-	const onClassic = true
-	s.testTPMResealHappy(c, revokeOldKeys, missingRunParams, missingRecoverParams, onClassic)
+	tc := tpmResealHappyCase{
+		onClassic:            true,
+		revokeOldKeys:        true,
+		missingRunParams:     true,
+		missingRecoverParams: true,
+	}
+	s.testTPMResealHappy(c, tc)
+}
+
+func (s *resealTestSuite) TestTPMResealHappyBrokenPrimaryKey(c *C) {
+	tc := tpmResealHappyCase{
+		onClassic:        true,
+		brokenPrimaryKey: true,
+	}
+	s.testTPMResealHappy(c, tc)
+}
+
+func (s *resealTestSuite) TestTPMResealHappyNoPrimaryKey(c *C) {
+	tc := tpmResealHappyCase{
+		onClassic:    true,
+		noPrimaryKey: true,
+	}
+	s.testTPMResealHappy(c, tc)
+}
+
+func (s *resealTestSuite) TestTPMResealHappyNoPrimaryKeyRevoke(c *C) {
+	tc := tpmResealHappyCase{
+		onClassic:     true,
+		revokeOldKeys: true,
+		noPrimaryKey:  true,
+	}
+	s.testTPMResealHappy(c, tc)
+}
+
+func (s *resealTestSuite) TestTPMResealHappyBrokenRevocationCounter(c *C) {
+	tc := tpmResealHappyCase{
+		onClassic:               true,
+		brokenRevocationCounter: true,
+	}
+	s.testTPMResealHappy(c, tc)
+}
+
+func (s *resealTestSuite) TestTPMResealHappyMultiplePolicyCounters(c *C) {
+	tc := tpmResealHappyCase{
+		onClassic:                 true,
+		multipleRevocationCounter: true,
+	}
+	s.testTPMResealHappy(c, tc)
 }
 
 func (s *resealTestSuite) TestResealKeyForBootchainsWithSystemFallback(c *C) {
 	var prevPbc boot.PredictableBootChains
 	var prevRecoveryPbc boot.PredictableBootChains
-	myState := &fakeState{}
 
 	for idx, tc := range []struct {
 		reuseRunPbc      bool
@@ -604,6 +719,7 @@ func (s *resealTestSuite) TestResealKeyForBootchainsWithSystemFallback(c *C) {
 		dirs.SetRootDir(rootdir)
 		defer dirs.SetRootDir("/")
 
+		myState := &fakeState{}
 		myState.EncryptedContainers = []backend.EncryptedContainer{
 			&encryptedContainer{
 				uuid:          "123",
@@ -846,7 +962,8 @@ func (s *resealTestSuite) TestResealKeyForBootchainsWithSystemFallback(c *C) {
 			c.Check(params.HintExpectFDEHook, Equals, false)
 
 			resealKeysCalls++
-			c.Check(params.TpmPCRProfile, DeepEquals, []byte(`"serialized-pcr-profile"`))
+
+			tpmPCRProfile, profileErr := params.GetTpmPCRProfile()
 
 			checkRunParams := func() {
 				c.Check(key, DeepEquals, secboot.KeyDataLocation{
@@ -854,6 +971,13 @@ func (s *resealTestSuite) TestResealKeyForBootchainsWithSystemFallback(c *C) {
 					SlotName:   "default",
 					KeyFile:    filepath.Join(boot.InitramfsBootEncryptionKeyDir, "ubuntu-data.sealed-key"),
 				})
+
+				if !tc.reuseRunPbc {
+					c.Assert(profileErr, IsNil)
+					c.Check(tpmPCRProfile, DeepEquals, []byte(`"serialized-pcr-profile"`))
+				} else {
+					c.Assert(profileErr, NotNil)
+				}
 			}
 
 			checkRecoveryParamsData := func() {
@@ -862,6 +986,13 @@ func (s *resealTestSuite) TestResealKeyForBootchainsWithSystemFallback(c *C) {
 					SlotName:   "default-fallback",
 					KeyFile:    filepath.Join(boot.InitramfsSeedEncryptionKeyDir, "ubuntu-data.recovery.sealed-key"),
 				})
+
+				if !tc.reuseRecoveryPbc {
+					c.Assert(profileErr, IsNil)
+					c.Check(tpmPCRProfile, DeepEquals, []byte(`"serialized-pcr-profile"`))
+				} else {
+					c.Assert(profileErr, NotNil)
+				}
 			}
 
 			checkRecoveryParamsSave := func() {
@@ -870,6 +1001,13 @@ func (s *resealTestSuite) TestResealKeyForBootchainsWithSystemFallback(c *C) {
 					SlotName:   "default-fallback",
 					KeyFile:    filepath.Join(boot.InitramfsSeedEncryptionKeyDir, "ubuntu-save.recovery.sealed-key"),
 				})
+
+				if !tc.reuseRecoveryPbc {
+					c.Assert(profileErr, IsNil)
+					c.Check(tpmPCRProfile, DeepEquals, []byte(`"serialized-pcr-profile"`))
+				} else {
+					c.Assert(profileErr, NotNil)
+				}
 			}
 
 			switch resealKeysCalls {
@@ -883,6 +1021,10 @@ func (s *resealTestSuite) TestResealKeyForBootchainsWithSystemFallback(c *C) {
 				c.Errorf("unexpected additional call to secboot.ResealKeys (call # %d)", resealKeysCalls)
 			}
 
+			if profileErr != nil {
+				return nil, profileErr
+			}
+
 			return nil, tc.resealErr
 		})
 		defer restore()
@@ -1188,6 +1330,23 @@ func (s *resealTestSuite) TestResealKeyForBootchainsWithSystemFallback(c *C) {
 		}
 		prevRecoveryPbc = recoveryPbc
 		c.Check(recoveryPbc, DeepEquals, boot.PredictableBootChains(removeKernelBootFiles(recoveryBootChains)))
+
+		if tc.reuseRunPbc {
+			sealingParams, err := myState.Get("run", "all")
+			c.Assert(err, IsNil)
+			c.Assert(sealingParams, IsNil)
+			sealingParams, err = myState.Get("run+recover", "all")
+			c.Assert(err, IsNil)
+			c.Assert(sealingParams, IsNil)
+		}
+		if tc.reuseRecoveryPbc {
+			sealingParams, err := myState.Get("recover", "system-data")
+			c.Assert(err, IsNil)
+			c.Assert(sealingParams, IsNil)
+			sealingParams, err = myState.Get("recover", "system-save")
+			c.Assert(err, IsNil)
+			c.Assert(sealingParams, IsNil)
+		}
 	}
 }
 
@@ -1295,7 +1454,9 @@ func (s *resealTestSuite) TestResealKeyForBootchainsRecoveryKeysForGoodSystemsOn
 		c.Check(params.HintExpectFDEHook, Equals, false)
 
 		resealKeysCalls++
-		c.Check(params.TpmPCRProfile, DeepEquals, []byte(`"serialized-pcr-profile"`))
+		tpmPCRProfile, err := params.GetTpmPCRProfile()
+		c.Assert(err, IsNil)
+		c.Check(tpmPCRProfile, DeepEquals, []byte(`"serialized-pcr-profile"`))
 
 		switch resealKeysCalls {
 		case 1: // run key
@@ -1637,7 +1798,9 @@ func (s *resealTestSuite) testResealKeyForBootchainsWithTryModel(c *C, shimId, g
 		c.Check(params.HintExpectFDEHook, Equals, false)
 
 		resealKeysCalls++
-		c.Check(params.TpmPCRProfile, DeepEquals, []byte(`"serialized-pcr-profile"`))
+		tpmPCRProfile, err := params.GetTpmPCRProfile()
+		c.Assert(err, IsNil)
+		c.Check(tpmPCRProfile, DeepEquals, []byte(`"serialized-pcr-profile"`))
 
 		switch resealKeysCalls {
 		case 1: // run key
@@ -1891,7 +2054,9 @@ func (s *resealTestSuite) TestResealKeyForBootchainsFallbackCmdline(c *C) {
 
 		c.Check(params.NewPCRPolicyVersion, Equals, false)
 		c.Check(params.HintExpectFDEHook, Equals, false)
-		c.Check(params.TpmPCRProfile, DeepEquals, []byte(`"serialized-pcr-profile"`))
+		tpmPCRProfile, err := params.GetTpmPCRProfile()
+		c.Assert(err, IsNil)
+		c.Check(tpmPCRProfile, DeepEquals, []byte(`"serialized-pcr-profile"`))
 
 		c.Logf("reseal: %+v", params)
 		switch resealKeysCalls {
@@ -2337,7 +2502,9 @@ func (s *resealTestSuite) TestResealKeyForSignatureDBUpdate(c *C) {
 		c.Check(params.NewPCRPolicyVersion, Equals, false)
 		c.Check(params.HintExpectFDEHook, Equals, false)
 
-		c.Check(params.TpmPCRProfile, DeepEquals, []byte(`"serialized-pcr-profile-with-dbx"`))
+		tpmPCRProfile, err := params.GetTpmPCRProfile()
+		c.Assert(err, IsNil)
+		c.Check(tpmPCRProfile, DeepEquals, []byte(`"serialized-pcr-profile-with-dbx"`))
 		c.Logf("reseal: %+v", params)
 
 		return nil, nil
