@@ -26,7 +26,6 @@ import (
 	"fmt"
 	"io"
 	"os"
-	"os/user"
 	"path/filepath"
 	"strings"
 
@@ -35,6 +34,7 @@ import (
 	"github.com/snapcore/snapd/client"
 	"github.com/snapcore/snapd/dirs"
 	"github.com/snapcore/snapd/gadget/quantity"
+	"github.com/snapcore/snapd/osutil/user"
 	"github.com/snapcore/snapd/overlord/configstate/config"
 	"github.com/snapcore/snapd/overlord/servicestate"
 	"github.com/snapcore/snapd/overlord/servicestate/servicestatetest"
@@ -239,7 +239,7 @@ NeedDaemonReload=no
 				Snap:      snp,
 				Name:      "dbus-slot",
 				Interface: "dbus",
-				Attrs: map[string]interface{}{
+				Attrs: map[string]any{
 					"bus":  "system",
 					"name": "org.example.Svc",
 				},
@@ -283,7 +283,7 @@ NeedDaemonReload=no
 				Snap:      snp,
 				Name:      "dbus-slot",
 				Interface: "dbus",
-				Attrs: map[string]interface{}{
+				Attrs: map[string]any{
 					"bus":  "session",
 					"name": "org.example.Svc",
 				},
@@ -391,7 +391,7 @@ NeedDaemonReload=no
 				Snap:      snp,
 				Name:      "dbus-slot",
 				Interface: "dbus",
-				Attrs: map[string]interface{}{
+				Attrs: map[string]any{
 					"bus":  "session",
 					"name": "org.example.Svc",
 				},
@@ -1040,3 +1040,7 @@ func (s *snapServiceOptionsSuite) TestLogReaderNamespaces(c *C) {
 	c.Assert(err, IsNil)
 	c.Check(jctlCalls, Equals, 1)
 }
+
+func (s *snapServiceOptionsSuite) TestEnsureLoopLogging(c *C) {
+	testutil.CheckEnsureLoopLogging("servicemgr.go", c, true)
+}
