@@ -37,6 +37,7 @@ import (
 	"github.com/snapcore/snapd/logger"
 	"github.com/snapcore/snapd/osutil"
 	"github.com/snapcore/snapd/overlord/auth"
+	"github.com/snapcore/snapd/overlord/confdbstate"
 	"github.com/snapcore/snapd/overlord/configstate/config"
 	"github.com/snapcore/snapd/overlord/hookstate"
 	"github.com/snapcore/snapd/overlord/ifacestate/ifacerepo"
@@ -86,9 +87,11 @@ func (s *autorefreshGatingSuite) SetUpTest(c *C) {
 	s.state.Set("refresh-privacy-key", "privacy-key")
 
 	restore := snapstate.MockEnforcedValidationSets(func(st *state.State, extraVss ...*asserts.ValidationSet) (*snapasserts.ValidationSets, error) {
-		return nil, nil
+		return snapasserts.NewValidationSets(), nil
 	})
 	s.AddCleanup(restore)
+
+	snapstate.IsConfdbHookname = confdbstate.IsConfdbHookname
 }
 
 func (r *autoRefreshGatingStore) SnapAction(ctx context.Context, currentSnaps []*store.CurrentSnap, actions []*store.SnapAction, assertQuery store.AssertionQuery, user *auth.UserState, opts *store.RefreshOptions) ([]store.SnapActionResult, []store.AssertionResult, error) {
@@ -1011,7 +1014,10 @@ func (s *autorefreshGatingSuite) TestAffectedByBase(c *C) {
 	snapB := mockInstalledSnap(c, s.state, snapByaml, useHook)
 	mockInstalledSnap(c, s.state, baseSnapByaml, noHook)
 
-	c.Assert(s.repo.AddSnap(snapB), IsNil)
+	snapBAppSet, err := interfaces.NewSnapAppSet(snapB, nil)
+	c.Assert(err, IsNil)
+
+	c.Assert(s.repo.AddAppSet(snapBAppSet), IsNil)
 
 	updates := []string{baseSnapA.InstanceName()}
 	affected, err := snapstate.AffectedByRefresh(st, updates)
@@ -1036,9 +1042,16 @@ func (s *autorefreshGatingSuite) TestAffectedByCore(c *C) {
 	core := mockInstalledSnap(c, s.state, coreYaml, noHook)
 	snapB := mockInstalledSnap(c, s.state, snapByaml, useHook)
 
-	c.Assert(s.repo.AddSnap(core), IsNil)
-	c.Assert(s.repo.AddSnap(snapB), IsNil)
-	c.Assert(s.repo.AddSnap(snapC), IsNil)
+	coreAppSet, err := interfaces.NewSnapAppSet(core, nil)
+	c.Assert(err, IsNil)
+	snapBAppSet, err := interfaces.NewSnapAppSet(snapB, nil)
+	c.Assert(err, IsNil)
+	snapCAppSet, err := interfaces.NewSnapAppSet(snapC, nil)
+	c.Assert(err, IsNil)
+
+	c.Assert(s.repo.AddAppSet(coreAppSet), IsNil)
+	c.Assert(s.repo.AddAppSet(snapBAppSet), IsNil)
+	c.Assert(s.repo.AddAppSet(snapCAppSet), IsNil)
 
 	updates := []string{core.InstanceName()}
 	affected, err := snapstate.AffectedByRefresh(st, updates)
@@ -1131,11 +1144,18 @@ func (s *autorefreshGatingSuite) TestAffectedBySlot(c *C) {
 	// unrelated snap
 	snapF := mockInstalledSnap(c, s.state, snapFyaml, useHook)
 
-	c.Assert(s.repo.AddSnap(snapF), IsNil)
-	c.Assert(s.repo.AddSnap(snapD), IsNil)
-	c.Assert(s.repo.AddSnap(snapE), IsNil)
+	snapFAppSet, err := interfaces.NewSnapAppSet(snapF, nil)
+	c.Assert(err, IsNil)
+	snapDAppSet, err := interfaces.NewSnapAppSet(snapD, nil)
+	c.Assert(err, IsNil)
+	snapEAppSet, err := interfaces.NewSnapAppSet(snapE, nil)
+	c.Assert(err, IsNil)
+
+	c.Assert(s.repo.AddAppSet(snapFAppSet), IsNil)
+	c.Assert(s.repo.AddAppSet(snapDAppSet), IsNil)
+	c.Assert(s.repo.AddAppSet(snapEAppSet), IsNil)
 	cref := &interfaces.ConnRef{PlugRef: interfaces.PlugRef{Snap: "snap-e", Name: "plug"}, SlotRef: interfaces.SlotRef{Snap: "snap-d", Name: "slot"}}
-	_, err := s.repo.Connect(cref, nil, nil, nil, nil, nil)
+	_, err = s.repo.Connect(cref, nil, nil, nil, nil, nil)
 	c.Assert(err, IsNil)
 
 	updates := []string{snapD.InstanceName()}
@@ -1162,12 +1182,19 @@ func (s *autorefreshGatingSuite) TestNotAffectedByCoreOrSnapdSlot(c *C) {
 	core := mockInstalledSnap(c, s.state, coreYaml, noHook)
 	snapB := mockInstalledSnap(c, s.state, snapByaml, useHook)
 
-	c.Assert(s.repo.AddSnap(snapG), IsNil)
-	c.Assert(s.repo.AddSnap(core), IsNil)
-	c.Assert(s.repo.AddSnap(snapB), IsNil)
+	snapGAppSet, err := interfaces.NewSnapAppSet(snapG, nil)
+	c.Assert(err, IsNil)
+	coreAppSet, err := interfaces.NewSnapAppSet(core, nil)
+	c.Assert(err, IsNil)
+	snapBAppSet, err := interfaces.NewSnapAppSet(snapB, nil)
+	c.Assert(err, IsNil)
+
+	c.Assert(s.repo.AddAppSet(snapGAppSet), IsNil)
+	c.Assert(s.repo.AddAppSet(coreAppSet), IsNil)
+	c.Assert(s.repo.AddAppSet(snapBAppSet), IsNil)
 
 	cref := &interfaces.ConnRef{PlugRef: interfaces.PlugRef{Snap: "snap-g", Name: "mir"}, SlotRef: interfaces.SlotRef{Snap: "core", Name: "mir"}}
-	_, err := s.repo.Connect(cref, nil, nil, nil, nil, nil)
+	_, err = s.repo.Connect(cref, nil, nil, nil, nil, nil)
 	c.Assert(err, IsNil)
 
 	updates := []string{core.InstanceName()}
@@ -1190,11 +1217,18 @@ func (s *autorefreshGatingSuite) TestNotAffectedByPlugWithMountBackend(c *C) {
 	// unrelated snap
 	snapF := mockInstalledSnap(c, s.state, snapFyaml, useHook)
 
-	c.Assert(s.repo.AddSnap(snapF), IsNil)
-	c.Assert(s.repo.AddSnap(snapD), IsNil)
-	c.Assert(s.repo.AddSnap(snapE), IsNil)
+	snapFAppSet, err := interfaces.NewSnapAppSet(snapF, nil)
+	c.Assert(err, IsNil)
+	snapDAppSet, err := interfaces.NewSnapAppSet(snapD, nil)
+	c.Assert(err, IsNil)
+	snapEAppSet, err := interfaces.NewSnapAppSet(snapE, nil)
+	c.Assert(err, IsNil)
+
+	c.Assert(s.repo.AddAppSet(snapFAppSet), IsNil)
+	c.Assert(s.repo.AddAppSet(snapDAppSet), IsNil)
+	c.Assert(s.repo.AddAppSet(snapEAppSet), IsNil)
 	cref := &interfaces.ConnRef{PlugRef: interfaces.PlugRef{Snap: "snap-e", Name: "plug"}, SlotRef: interfaces.SlotRef{Snap: "snap-d", Name: "slot"}}
-	_, err := s.repo.Connect(cref, nil, nil, nil, nil, nil)
+	_, err = s.repo.Connect(cref, nil, nil, nil, nil, nil)
 	c.Assert(err, IsNil)
 
 	// snapE has a plug using mount backend and is refreshed, this doesn't affect slot of snap-d.
@@ -1218,11 +1252,18 @@ func (s *autorefreshGatingSuite) TestAffectedByPlugWithMountBackendSnapdSlot(c *
 	// unrelated snap
 	snapF := mockInstalledSnap(c, s.state, snapFyaml, useHook)
 
-	c.Assert(s.repo.AddSnap(snapF), IsNil)
-	c.Assert(s.repo.AddSnap(snapdSnap), IsNil)
-	c.Assert(s.repo.AddSnap(snapG), IsNil)
+	snapFAppSet, err := interfaces.NewSnapAppSet(snapF, nil)
+	c.Assert(err, IsNil)
+	snapdSnapAppSet, err := interfaces.NewSnapAppSet(snapdSnap, nil)
+	c.Assert(err, IsNil)
+	snapGAppSet, err := interfaces.NewSnapAppSet(snapG, nil)
+	c.Assert(err, IsNil)
+
+	c.Assert(s.repo.AddAppSet(snapFAppSet), IsNil)
+	c.Assert(s.repo.AddAppSet(snapdSnapAppSet), IsNil)
+	c.Assert(s.repo.AddAppSet(snapGAppSet), IsNil)
 	cref := &interfaces.ConnRef{PlugRef: interfaces.PlugRef{Snap: "snap-g", Name: "desktop"}, SlotRef: interfaces.SlotRef{Snap: "snapd", Name: "desktop"}}
-	_, err := s.repo.Connect(cref, nil, nil, nil, nil, nil)
+	_, err = s.repo.Connect(cref, nil, nil, nil, nil, nil)
 	c.Assert(err, IsNil)
 
 	// snapE has a plug using mount backend, refreshing snapd affects snapE.
@@ -1249,10 +1290,16 @@ func (s *autorefreshGatingSuite) TestAffectedByPlugWithMountBackendCoreSlot(c *C
 	coreSnap := mockInstalledSnap(c, s.state, coreYaml, noHook)
 	snapG := mockInstalledSnap(c, s.state, snapGyaml, useHook)
 
-	c.Assert(s.repo.AddSnap(coreSnap), IsNil)
-	c.Assert(s.repo.AddSnap(snapG), IsNil)
+	coreAppSet, err := interfaces.NewSnapAppSet(coreSnap, nil)
+	c.Assert(err, IsNil)
+
+	snapGAppSet, err := interfaces.NewSnapAppSet(snapG, nil)
+	c.Assert(err, IsNil)
+
+	c.Assert(s.repo.AddAppSet(coreAppSet), IsNil)
+	c.Assert(s.repo.AddAppSet(snapGAppSet), IsNil)
 	cref := &interfaces.ConnRef{PlugRef: interfaces.PlugRef{Snap: "snap-g", Name: "desktop"}, SlotRef: interfaces.SlotRef{Snap: "core", Name: "desktop"}}
-	_, err := s.repo.Connect(cref, nil, nil, nil, nil, nil)
+	_, err = s.repo.Connect(cref, nil, nil, nil, nil, nil)
 	c.Assert(err, IsNil)
 
 	// snapG has a plug using mount backend, refreshing core affects snapE.
@@ -1973,7 +2020,7 @@ func (s *snapmgrTestSuite) testAutoRefreshPhase2(c *C, beforePhase1 func(), gate
 	c.Assert(err, IsNil)
 	c.Check(names, DeepEquals, []string{"base-snap-b", "snap-a"})
 
-	var snaps map[string]interface{}
+	var snaps map[string]any
 	c.Assert(tss[0].Tasks()[0].Kind(), Equals, "conditional-auto-refresh")
 	c.Assert(tss[0].Tasks()[0].Get("snaps", &snaps), IsNil)
 	c.Assert(snaps, HasLen, 2)
@@ -2058,7 +2105,7 @@ func (s *snapmgrTestSuite) TestAutoRefreshPhase2(c *C) {
 	tasks := chg.Tasks()
 	c.Check(tasks[len(tasks)-1].Summary(), Equals, `Monitoring snaps "base-snap-b", "snap-a" to determine whether extra refresh steps are required`)
 
-	var snaps map[string]interface{}
+	var snaps map[string]any
 	c.Assert(chg.Tasks()[0].Kind(), Equals, "conditional-auto-refresh")
 	chg.Tasks()[0].Get("snaps", &snaps)
 	c.Assert(snaps, HasLen, 2)
@@ -2115,7 +2162,7 @@ func (s *snapmgrTestSuite) TestAutoRefreshPhase2Held(c *C) {
 	c.Assert(logbuf.String(), testutil.Contains, `skipping refresh of held snaps: base-snap-b`)
 	tasks := chg.Tasks()
 
-	var snaps map[string]interface{}
+	var snaps map[string]any
 	c.Assert(chg.Tasks()[0].Kind(), Equals, "conditional-auto-refresh")
 	chg.Tasks()[0].Get("snaps", &snaps)
 	c.Assert(snaps, HasLen, 1)
@@ -2633,12 +2680,12 @@ func (s *autorefreshGatingSuite) TestAutoRefreshForGatingSnap(c *C) {
 	c.Assert(chg.Kind(), Equals, "auto-refresh")
 	c.Check(chg.Summary(), Equals, `Auto-refresh snaps "base-snap-b", "snap-b"`)
 	var snapNames []string
-	var apiData map[string]interface{}
+	var apiData map[string]any
 	c.Assert(chg.Get("snap-names", &snapNames), IsNil)
 	c.Check(snapNames, DeepEquals, []string{"base-snap-b", "snap-b"})
 	c.Assert(chg.Get("api-data", &apiData), IsNil)
-	c.Check(apiData, DeepEquals, map[string]interface{}{
-		"snap-names": []interface{}{"base-snap-b", "snap-b"},
+	c.Check(apiData, DeepEquals, map[string]any{
+		"snap-names": []any{"base-snap-b", "snap-b"},
 	})
 
 	tasks := chg.Tasks()
@@ -2740,12 +2787,12 @@ func (s *autorefreshGatingSuite) TestAutoRefreshForGatingSnapMoreAffectedSnaps(c
 	c.Assert(chg.Kind(), Equals, "auto-refresh")
 	c.Check(chg.Summary(), Equals, `Auto-refresh snaps "base-snap-b", "snap-b"`)
 	var snapNames []string
-	var apiData map[string]interface{}
+	var apiData map[string]any
 	c.Assert(chg.Get("snap-names", &snapNames), IsNil)
 	c.Check(snapNames, DeepEquals, []string{"base-snap-b", "snap-b"})
 	c.Assert(chg.Get("api-data", &apiData), IsNil)
-	c.Check(apiData, DeepEquals, map[string]interface{}{
-		"snap-names": []interface{}{"base-snap-b", "snap-b"},
+	c.Check(apiData, DeepEquals, map[string]any{
+		"snap-names": []any{"base-snap-b", "snap-b"},
 	})
 
 	tasks := chg.Tasks()
@@ -3083,13 +3130,13 @@ func (s *validationSetsSuite) TestAutoRefreshPhase1WithValidationSets(c *C) {
 	var requiredRevision string
 	restoreEnforcedValidationSets := snapstate.MockEnforcedValidationSets(func(st *state.State, extraVss ...*asserts.ValidationSet) (*snapasserts.ValidationSets, error) {
 		vs := snapasserts.NewValidationSets()
-		someSnap := map[string]interface{}{
+		someSnap := map[string]any{
 			"id":       "yOqKhntON3vR7kwEbVPsILm7bUViPDzx",
 			"name":     "some-snap",
 			"presence": "required",
 			"revision": requiredRevision,
 		}
-		snapC := map[string]interface{}{
+		snapC := map[string]any{
 			"id":       "aOqKhntON3vR7kwEbVPsILm7bUViPDzz",
 			"name":     "snap-c",
 			"presence": "required",
@@ -3137,18 +3184,21 @@ func (s *validationSetsSuite) TestAutoRefreshPhase1WithValidationSets(c *C) {
 			Revision:      snap.R(1),
 			Epoch:         snap.E("1*"),
 			RefreshedDate: refreshedDate,
+			Resources:     make(map[string]snap.Revision),
 		}, {
 			InstanceName:  "some-other-snap",
 			SnapID:        "some-other-snap-id",
 			Revision:      snap.R(1),
 			Epoch:         snap.E("1*"),
 			RefreshedDate: refreshedDate,
+			Resources:     make(map[string]snap.Revision),
 		}, {
 			InstanceName:  "some-snap",
 			SnapID:        "some-snap-id",
 			Revision:      snap.R(1),
 			Epoch:         snap.E("1*"),
 			RefreshedDate: refreshedDate,
+			Resources:     make(map[string]snap.Revision),
 		}},
 	}, {
 		op: "storesvc-snap-action:action",
