@@ -152,7 +152,7 @@ func (s *autoRefreshTestSuite) SetUpTest(c *C) {
 	s.AddCleanup(snapstatetest.MockDeviceModel(DefaultModel()))
 
 	restore := snapstate.MockEnforcedValidationSets(func(st *state.State, extraVss ...*asserts.ValidationSet) (*snapasserts.ValidationSets, error) {
-		return nil, nil
+		return snapasserts.NewValidationSets(), nil
 	})
 	s.AddCleanup(restore)
 }
@@ -173,11 +173,6 @@ func (s *autoRefreshTestSuite) TestLastRefresh(c *C) {
 }
 
 func (s *autoRefreshTestSuite) TestLastRefreshRefreshManaged(c *C) {
-	snapstate.CanManageRefreshes = func(st *state.State) bool {
-		return true
-	}
-	defer func() { snapstate.CanManageRefreshes = nil }()
-
 	logbuf, restore := logger.MockLogger()
 	defer restore()
 
@@ -220,11 +215,6 @@ func (s *autoRefreshTestSuite) TestLastRefreshRefreshManaged(c *C) {
 }
 
 func (s *autoRefreshTestSuite) TestRefreshManagedTimerWins(c *C) {
-	snapstate.CanManageRefreshes = func(st *state.State) bool {
-		return true
-	}
-	defer func() { snapstate.CanManageRefreshes = nil }()
-
 	s.state.Lock()
 	defer s.state.Unlock()
 
@@ -248,18 +238,7 @@ func (s *autoRefreshTestSuite) TestRefreshManagedTimerWins(c *C) {
 	c.Check(err, IsNil)
 }
 
-func (s *autoRefreshTestSuite) TestRefreshManagedDenied(c *C) {
-	canManageCalled := false
-	snapstate.CanManageRefreshes = func(st *state.State) bool {
-		canManageCalled = true
-		// always deny
-		return false
-	}
-	defer func() { snapstate.CanManageRefreshes = nil }()
-
-	logbuf, restore := logger.MockLogger()
-	defer restore()
-
+func (s *autoRefreshTestSuite) TestRefreshManagedIsRespected(c *C) {
 	s.state.Lock()
 	defer s.state.Unlock()
 
@@ -275,23 +254,11 @@ func (s *autoRefreshTestSuite) TestRefreshManagedDenied(c *C) {
 			err := af.Ensure()
 			s.state.Lock()
 			c.Check(err, IsNil)
-			c.Check(s.store.ops, DeepEquals, []string{"list-refresh"})
-
-			refreshScheduleStr, _, err := af.RefreshSchedule()
-			c.Check(refreshScheduleStr, Equals, snapstate.DefaultRefreshSchedule)
-			c.Check(err, IsNil)
-			c.Check(canManageCalled, Equals, true)
-			count := strings.Count(logbuf.String(),
-				": managed refresh schedule denied, no properly configured snapd-control\n")
-			c.Check(count, Equals, 1, Commentf("too many occurrences:\n%s", logbuf.String()))
-
-			canManageCalled = false
+			c.Check(s.store.ops, HasLen, 0)
 		}
 
 		// ensure clean config for the next run
 		s.state.Set("config", nil)
-		logbuf.Reset()
-		canManageCalled = false
 	}
 }
 
@@ -1031,7 +998,7 @@ func (s *autoRefreshTestSuite) TestSubsequentInhibitRefreshWithinInhibitWindow(c
 	defer restore()
 
 	instant := time.Now()
-	pastInstant := instant.Add(-snapstate.MaxInhibition / 2) // In the middle of the allowed window
+	pastInstant := instant.Add(-snapstate.MaxInhibitionDuration(s.state) / 2) // In the middle of the allowed window
 
 	si := &snap.SideInfo{RealName: "pkg", Revision: snap.R(1)}
 	info := &snap.Info{SideInfo: *si}
@@ -1075,7 +1042,7 @@ func (s *autoRefreshTestSuite) TestInhibitRefreshRefreshesWhenOverdue(c *C) {
 	defer restore()
 
 	instant := time.Now()
-	pastInstant := instant.Add(-snapstate.MaxInhibition * 2)
+	pastInstant := instant.Add(-snapstate.MaxInhibitionDuration(s.state) * 2)
 
 	si := &snap.SideInfo{RealName: "pkg", Revision: snap.R(1)}
 	info := &snap.Info{SideInfo: *si}
@@ -1106,7 +1073,7 @@ func (s *autoRefreshTestSuite) TestInhibitNoNotificationOnManualRefresh(c *C) {
 	})
 	defer restore()
 
-	pastInstant := time.Now().Add(-snapstate.MaxInhibition)
+	pastInstant := time.Now().Add(-snapstate.MaxInhibitionDuration(s.state))
 
 	si := &snap.SideInfo{RealName: "pkg", Revision: snap.R(1)}
 	info := &snap.Info{SideInfo: *si}
@@ -1230,7 +1197,7 @@ func (s *autoRefreshTestSuite) TestTooSoonError(c *C) {
 	c.Check(snapstate.TooSoonError{}.Error(), Equals, "cannot auto-refresh so soon")
 }
 
-func setStoreAccess(s *state.State, access interface{}) {
+func setStoreAccess(s *state.State, access any) {
 	s.Lock()
 	defer s.Unlock()
 
@@ -1626,3 +1593,55 @@ func (s *autoRefreshTestSuite) TestMaybeAsyncPendingRefreshNotificationSkips(c *
 
 	snapstate.MaybeAsyncPendingRefreshNotification(context.TODO(), s.state, &userclient.PendingSnapRefreshInfo{})
 }
+
+func (s *autoRefreshTestSuite) TestAutoRefreshWithConfdbs(c *C) {
+	si := &snap.SideInfo{
+		RealName: "foo",
+		SnapID:   "foo-id",
+		Revision: snap.R(1),
+	}
+	s.state.Lock()
+	snapstate.Set(s.state, "foo", &snapstate.SnapState{
+		Active:   true,
+		Sequence: snapstatetest.NewSequenceFromSnapSideInfos([]*snap.SideInfo{si}),
+		Current:  si.Revision,
+		SnapType: string(snap.TypeApp),
+	})
+	s.state.Unlock()
+
+	info := &snap.Info{
+		SnapType:      snap.TypeApp,
+		Architectures: []string{"all"},
+		SideInfo: snap.SideInfo{
+			RealName: "foo",
+			Revision: snap.R(8),
+		},
+	}
+	info.Plugs = map[string]*snap.PlugInfo{
+		"my-plug": {
+			Snap:      info,
+			Name:      "my-plug",
+			Interface: "confdb",
+			Attrs: map[string]any{
+				"account": "my-publisher",
+				"view":    "my-reg/my-view",
+			},
+		},
+	}
+	s.store.refreshable = append(s.store.refreshable, info)
+
+	af := snapstate.NewAutoRefresh(s.state)
+	err := af.Ensure()
+	c.Check(err, IsNil)
+
+	s.state.Lock()
+	defer s.state.Unlock()
+
+	chgs := s.state.Changes()
+	c.Assert(chgs, HasLen, 1)
+	task := findLastTask(chgs[0], "validate-snap")
+
+	snapsup, err := snapstate.TaskSnapSetup(task)
+	c.Assert(err, IsNil)
+	c.Assert(snapsup.PluggedConfdbIDs, DeepEquals, []snapstate.ConfdbSchemaID{{Account: "my-publisher", Name: "my-reg"}})
+}
