@@ -37,6 +37,7 @@ import (
 	"github.com/snapcore/snapd/overlord/snapstate/backend"
 	"github.com/snapcore/snapd/overlord/snapstate/sequence"
 	"github.com/snapcore/snapd/overlord/state"
+	"github.com/snapcore/snapd/overlord/swfeats"
 	"github.com/snapcore/snapd/randutil"
 	"github.com/snapcore/snapd/release"
 	"github.com/snapcore/snapd/sandbox"
@@ -52,6 +53,21 @@ import (
 	"github.com/snapcore/snapd/wrappers"
 )
 
+var (
+	removeSnapChangeKind           = swfeats.RegisterChangeKind("remove-snap")
+	transitionUbuntuCoreChangeKind = swfeats.RegisterChangeKind("transition-ubuntu-core")
+)
+
+func init() {
+	swfeats.RegisterEnsure("SnapManager", "ensureVulnerableSnapConfineVersionsRemovedOnClassic")
+	swfeats.RegisterEnsure("SnapManager", "ensureForceDevmodeDropsDevmodeFromState")
+	swfeats.RegisterEnsure("SnapManager", "ensureUbuntuCoreTransition")
+	swfeats.RegisterEnsure("SnapManager", "atSeed")
+	swfeats.RegisterEnsure("SnapManager", "ensureMountsUpdated")
+	swfeats.RegisterEnsure("SnapManager", "ensureDesktopFilesUpdated")
+	swfeats.RegisterEnsure("SnapManager", "ensureDownloadsCleaned")
+}
+
 // SnapManager is responsible for the installation and removal of snaps.
 type SnapManager struct {
 	state   *state.State
@@ -107,7 +123,7 @@ type SnapSetup struct {
 
 	DownloadInfo *snap.DownloadInfo `json:"download-info,omitempty"`
 	SideInfo     *snap.SideInfo     `json:"side-info,omitempty"`
-	auxStoreInfo
+	backend.AuxStoreInfo
 
 	// InstanceKey is set by the user during installation and differs for
 	// each instance of given snap
@@ -150,9 +166,9 @@ type SnapSetup struct {
 	// effect on which tasks get created to update the snap.
 	AlwaysUpdate bool `json:"-"`
 
-	// Confdbs is the set of confdbs that the snap plugs, identified by
-	// account and confdb name pairs.
-	Confdbs []ConfdbID `json:"confdbs,omitempty"`
+	// PluggedConfdbIDs is the set of confdb schema IDs that the snap plugs,
+	// identified by account and confdb schema name pairs.
+	PluggedConfdbIDs []ConfdbSchemaID `json:"plugged-confdb-ids,omitempty"`
 
 	// PreUpdateKernelModuleComponents is set if the kernel-modules component
 	// that are set up, prior to any changes to the state. This is used in the
@@ -165,12 +181,12 @@ type SnapSetup struct {
 	ComponentExclusiveOperation bool `json:"component-exclusive-operation,omitempty"`
 }
 
-// ConfdbID identifies a confdb.
-type ConfdbID struct {
-	// Account is the name of the account that publishes the confdb.
+// ConfdbSchemaID identifies a confdb schema.
+type ConfdbSchemaID struct {
+	// Account is the name of the account that publishes the confdb schema.
 	Account string
-	// Confdb is the name of the confdb within the account namespace.
-	Confdb string
+	// Name is the name of the confdb schema within the account namespace.
+	Name string
 }
 
 func (snapsup *SnapSetup) InstanceName() string {
@@ -409,6 +425,9 @@ type SnapState struct {
 
 	// RefreshFailures tracks information about snap failed refreshes.
 	RefreshFailures *snap.RefreshFailuresInfo `json:"refresh-failures,omitempty"`
+
+	// Base indicates the snap's base snap.
+	Base string `json:"base,omitempty"`
 }
 
 // PendingSecurityState holds information about snaps that have
@@ -624,7 +643,7 @@ func readInfo(name string, si *snap.SideInfo, flags int) (*snap.Info, error) {
 		err = nil
 	}
 	if err == nil && flags&withAuxStoreInfo != 0 {
-		if err := retrieveAuxStoreInfo(info); err != nil {
+		if err := backend.RetrieveAuxStoreInfo(info); err != nil {
 			logger.Debugf("cannot read auxiliary store info for snap %q: %v", name, err)
 		}
 	}
@@ -886,7 +905,8 @@ func Manager(st *state.State, runner *state.TaskRunner) (*SnapManager, error) {
 	runner.AddHandler("prepare-kernel-modules-components", m.doPrepareKernelModulesComponents, m.undoPrepareKernelModulesComponents)
 
 	// control serialisation
-	runner.AddBlocked(m.blockedTask)
+	runner.AddBlocked(m.otherPrereqRunning)
+	runner.AddBlocked(affectsRunningHooks)
 
 	RegisterAffectedSnapsByKind("conditional-auto-refresh", conditionalAutoRefreshAffectedSnaps)
 
@@ -955,7 +975,7 @@ func genRefreshRequestSalt(st *state.State) error {
 	return nil
 }
 
-func (m *SnapManager) blockedTask(cand *state.Task, running []*state.Task) bool {
+func (m *SnapManager) otherPrereqRunning(cand *state.Task, running []*state.Task) bool {
 	// Serialize "prerequisites", the state lock is not enough as
 	// Install() inside doPrerequisites() will unlock to talk to
 	// the store.
@@ -970,6 +990,77 @@ func (m *SnapManager) blockedTask(cand *state.Task, running []*state.Task) bool
 	return false
 }
 
+func affectsRunningHooks(cand *state.Task, running []*state.Task) (block bool) {
+	st := cand.State()
+
+	affectingTasks := map[string][]state.Status{
+		"unlink-current-snap": {state.DoStatus, state.DoingStatus},
+		// this will block any undo link-snap, although only undos of first installs/snap
+		// unlink. It's unlikely that a hook runs between the link-snap and the end
+		// of the change and, in the worst case, we delay the undoing
+		"link-snap":   {state.UndoStatus, state.UndoingStatus},
+		"unlink-snap": {state.DoStatus, state.DoingStatus},
+	}
+
+	statuses, ok := affectingTasks[cand.Kind()]
+	if !ok {
+		return false
+	}
+
+	var unlinkingTask bool
+	for _, status := range statuses {
+		if cand.Status() == status {
+			unlinkingTask = true
+			break
+		}
+	}
+
+	if !unlinkingTask {
+		return false
+	}
+
+	snapsup, err := TaskSnapSetup(cand)
+	if err != nil {
+		logger.Noticef("internal error: cannot obtain snap-setup from task %q: %v", cand.ID(), err)
+		return false
+	}
+	candSnap := snapsup.InstanceName()
+
+	for _, t := range running {
+		if t.Kind() != "run-hook" {
+			continue
+		}
+
+		type hookSetup struct {
+			Snap string `json:"snap"`
+		}
+
+		var hooksup hookSetup
+		if err := t.Get("hook-setup", &hooksup); err != nil {
+			logger.Noticef("internal error: cannot obtain hook-setup from task %q: %v", t.ID(), err)
+			return false
+		}
+
+		// this snap has a hook running, retry later
+		if candSnap == hooksup.Snap {
+			return true
+		}
+
+		var hookSnapst SnapState
+		if err := Get(st, hooksup.Snap, &hookSnapst); err != nil {
+			logger.Noticef("internal error: cannot get snapstate for %q: %v", hooksup.Snap, err)
+			return false
+		}
+
+		// this is a base for a snap with a hook running, retry later
+		if candSnap == hookSnapst.Base {
+			return true
+		}
+	}
+
+	return false
+}
+
 // NextRefresh returns the time the next update of the system's snaps
 // will be attempted.
 // The caller should be holding the state lock.
@@ -1106,7 +1197,7 @@ func (m *SnapManager) ensureVulnerableSnapRemoved(name string) error {
 
 		msg := fmt.Sprintf(i18n.G("Remove inactive vulnerable %q snap (%v)"), name, rev)
 
-		chg := m.state.NewChange("remove-snap", msg)
+		chg := m.state.NewChange(removeSnapChangeKind, msg)
 		chg.AddAll(tss)
 		chg.Set("snap-names", []string{name})
 	}
@@ -1129,6 +1220,8 @@ func (m *SnapManager) ensureVulnerableSnapConfineVersionsRemovedOnClassic() erro
 		return nil
 	}
 
+	logger.Trace("ensure", "manager", "SnapManager", "func", "ensureVulnerableSnapConfineVersionsRemovedOnClassic")
+
 	m.state.Lock()
 	defer m.state.Unlock()
 
@@ -1168,6 +1261,8 @@ func (m *SnapManager) ensureForceDevmodeDropsDevmodeFromState() error {
 		return nil
 	}
 
+	logger.Trace("ensure", "manager", "SnapManager", "func", "ensureForceDevmodeDropsDevmodeFromState")
+
 	for _, name := range []string{"core", "ubuntu-core"} {
 		var snapst SnapState
 		if err := Get(m.state, name, &snapst); errors.Is(err, state.ErrNoState) {
@@ -1253,6 +1348,8 @@ func (m *SnapManager) ensureUbuntuCoreTransition() error {
 		return nil
 	}
 
+	logger.Trace("ensure", "manager", "SnapManager", "func", "ensureUbuntuCoreTransition")
+
 	m.state.Set("ubuntu-core-transition-last-retry-time", now)
 
 	var retryCount int
@@ -1267,7 +1364,7 @@ func (m *SnapManager) ensureUbuntuCoreTransition() error {
 	}
 
 	msg := i18n.G("Transition ubuntu-core to core")
-	chg := m.state.NewChange("transition-ubuntu-core", msg)
+	chg := m.state.NewChange(transitionUbuntuCoreChangeKind, msg)
 	for _, ts := range tss {
 		chg.AddAll(ts)
 	}
@@ -1285,6 +1382,7 @@ func (m *SnapManager) atSeed() error {
 		// already seeded or other error
 		return err
 	}
+	logger.Trace("ensure", "manager", "SnapManager", "func", "atSeed")
 	if err := m.autoRefresh.AtSeed(); err != nil {
 		return err
 	}
@@ -1390,6 +1488,8 @@ func (m *SnapManager) ensureMountsUpdated() error {
 		return err
 	}
 
+	logger.Trace("ensure", "manager", "SnapManager", "func", "ensureMountsUpdated")
+
 	if len(allStates) != 0 {
 		sysd := getSystemD()
 
@@ -1470,6 +1570,7 @@ func (m *SnapManager) ensureDesktopFilesUpdated() error {
 		}
 		snaps = append(snaps, info)
 	}
+	logger.Trace("ensure", "manager", "SnapManager", "func", "ensureDesktopFilesUpdated")
 	if err := wrappers.EnsureSnapDesktopFiles(snaps); err != nil {
 		return err
 	}
@@ -1497,6 +1598,8 @@ func (m *SnapManager) ensureDownloadsCleaned() error {
 		return nil
 	}
 
+	logger.Trace("ensure", "manager", "SnapManager", "func", "ensureDownloadsCleaned")
+
 	if err := cleanDownloads(m.state); err != nil {
 		return err
 	}
