@@ -20,9 +20,10 @@ package cgroup
 
 import (
 	"context"
+	"syscall"
 	"time"
 
-	"github.com/godbus/dbus"
+	"github.com/godbus/dbus/v5"
 
 	"github.com/snapcore/snapd/testutil"
 )
@@ -41,6 +42,8 @@ var (
 	SecurityTagFromCgroupPath = securityTagFromCgroupPath
 
 	ApplyToSnap = applyToSnap
+
+	KillProcessesInCgroup = killProcessesInCgroup
 )
 
 func MockFsTypeForPath(mock func(string) (int64, error)) (restore func()) {
@@ -105,12 +108,6 @@ func MockCreateScopeJobTimeout(d time.Duration) (restore func()) {
 	}
 }
 
-func MockCgroupsFilePath(path string) (restore func()) {
-	r := testutil.Backup(&cgroupsFilePath)
-	cgroupsFilePath = path
-	return r
-}
-
 func MonitorDelete(folders []string, name string, channel chan string) error {
 	return currentWatcher.monitorDelete(folders, name, channel)
 }
@@ -135,3 +132,31 @@ func (iw *inotifyWatcher) MonitorDelete(folders []string, name string, channel c
 }
 
 var NewInotifyWatcher = newInotifyWatcher
+
+func MockFreezeSnapProcessesImplV1(fn func(ctx context.Context, snapName string) error) (restore func()) {
+	return testutil.Mock(&freezeSnapProcessesImplV1, fn)
+}
+
+func MockThawSnapProcessesImplV1(fn func(snapName string) error) (restore func()) {
+	return testutil.Mock(&thawSnapProcessesImplV1, fn)
+}
+
+func MockKillProcessesInCgroup(fn func(ctx context.Context, dir string, freeze func(ctx context.Context), thaw func()) error) (restore func()) {
+	return testutil.Mock(&killProcessesInCgroup, fn)
+}
+
+func MockSyscallKill(fn func(pid int, sig syscall.Signal) error) (restore func()) {
+	return testutil.Mock(&syscallKill, fn)
+}
+
+func MockOsReadFile(fn func(name string) ([]byte, error)) (restore func()) {
+	return testutil.Mock(&osReadFile, fn)
+}
+
+func MockMaxKillTimeout(t time.Duration) (restore func()) {
+	return testutil.Mock(&maxKillTimeout, t)
+}
+
+func MockKillThawCooldown(t time.Duration) (restore func()) {
+	return testutil.Mock(&killThawCooldown, t)
+}
