@@ -9,11 +9,12 @@ import (
 	"sync"
 	"time"
 
-	"github.com/godbus/dbus"
+	"github.com/godbus/dbus/v5"
 
 	"github.com/snapcore/snapd/dbusutil"
 	"github.com/snapcore/snapd/logger"
 	"github.com/snapcore/snapd/randutil"
+	"github.com/snapcore/snapd/systemd"
 )
 
 var osGetuid = os.Getuid
@@ -81,13 +82,18 @@ func CreateTransientScopeForTracking(securityTag string, opts *TrackingOptions)
 		return err
 	}
 
+	securityTagUnitName, err := systemd.SecurityTagToUnitName(securityTag)
+	if err != nil {
+		return err
+	}
+
 	// Enforcing uniqueness is preferred to reusing an existing scope for
 	// simplicity since doing otherwise by joining an existing scope has
 	// limitations:
 	// - the originally started scope must be marked as a delegate, with all
 	//   consequences.
 	// - the method AttachProcessesToUnit is unavailable on Ubuntu 16.04
-	unitName := fmt.Sprintf("%s-%s.scope", securityTag, uuid)
+	unitName := fmt.Sprintf("%s-%s.scope", securityTagUnitName, uuid)
 
 	pid := osGetpid()
 	start := time.Now()
@@ -156,6 +162,11 @@ tryAgain:
 //
 // If the application process is not tracked then ErrCannotTrackProcess is returned.
 func ConfirmSystemdAppTracking(securityTag string) error {
+	unitName, err := systemd.SecurityTagToUnitName(securityTag)
+	if err != nil {
+		return err
+	}
+
 	pid := osGetpid()
 	path, err := cgroupProcessPathInTrackingCgroup(pid)
 	if err != nil {
@@ -164,7 +175,7 @@ func ConfirmSystemdAppTracking(securityTag string) error {
 
 	// the transient scope of the application carries the security tag, eg:
 	// snap.hello-world.sh-4706fe54-7802-4808-aa7e-ae8b567239e0.scope
-	if strings.HasPrefix(filepath.Base(path), securityTag+"-") && strings.HasSuffix(path, ".scope") {
+	if strings.HasPrefix(filepath.Base(path), unitName+"-") && strings.HasSuffix(path, ".scope") {
 		return nil
 	}
 
@@ -260,18 +271,18 @@ func startTransientScope(conn *dbus.Conn, unitName string, pid int) (job dbus.Ob
 	// jobMode string  // corresponds to --job-mode= (see systemctl(1) manual page)
 	// properties []struct{
 	//   Name string
-	//   Value interface{}
+	//   Value any
 	// } // properties describe properties of the started unit
 	// auxUnits []struct {
 	//   Name string
 	//   Properties []struct{
 	//   	Name string
-	//   	Value interface{}
+	//   	Value any
 	//	 }
 	// } // auxUnits describe any additional units to define.
 	type property struct {
 		Name  string
-		Value interface{}
+		Value any
 	}
 	type auxUnit struct {
 		Name  string
