@@ -22,6 +22,7 @@ package secboot
 
 import (
 	"context"
+	"encoding/json"
 	"io"
 	"os"
 	"os/exec"
@@ -71,7 +72,7 @@ func MockSbPreinstallNewRunChecksContext(f func(initialFlags sb_preinstall.Check
 	}
 }
 
-func MockSbPreinstallRun(f func(checkCtx *sb_preinstall.RunChecksContext, ctx context.Context, action sb_preinstall.Action, args ...any) (*sb_preinstall.CheckResult, error)) (restore func()) {
+func MockSbPreinstallRun(f func(checkCtx *sb_preinstall.RunChecksContext, ctx context.Context, action sb_preinstall.Action, args map[string]json.RawMessage) (*sb_preinstall.CheckResult, error)) (restore func()) {
 	old := sbPreinstallRunChecks
 	sbPreinstallRunChecks = f
 	return func() {
@@ -557,7 +558,7 @@ func NewKeyData(kd *sb.KeyData) KeyData {
 	return &keyData{kd: kd}
 }
 
-func MockResealKeysWithFDESetupHook(f func(keys []KeyDataLocation, primaryKeyDevices []string, fallbackPrimaryKeyFiles []string, models []ModelForSealing, bootModes []string) error) (restore func()) {
+func MockResealKeysWithFDESetupHook(f func(keys []KeyDataLocation, primaryKeyDevices []string, fallbackPrimaryKeyFiles []string, verifyPrimaryKey func([]byte), models []ModelForSealing, bootModes []string) error) (restore func()) {
 	return testutil.Mock(&resealKeysWithFDESetupHook, f)
 }
 
@@ -568,3 +569,7 @@ func MockResealKeysWithTPM(f func(params *resealKeysWithTPMParams, newPCRPolicyV
 func MockSbKeyDataPlatformName(f func(d *sb.KeyData) string) (restore func()) {
 	return testutil.Mock(&sbKeyDataPlatformName, f)
 }
+
+func MockSbPCRPolicyCounterHandle(f func(skd *sb_tpm2.SealedKeyData) tpm2.Handle) (restore func()) {
+	return testutil.Mock(&sbPCRPolicyCounterHandle, f)
+}
