@@ -308,12 +308,14 @@ type ResealKeyParams struct {
 	// FallbackPrimaryKeyFiles is the list of files that might contain
 	// the primary key.
 	FallbackPrimaryKeyFiles []string
+	// VerifyPrimaryKey is called if the primary key was read
+	VerifyPrimaryKey func([]byte)
 	// The allowed boot modes (run, recover, factory-reset)
 	BootModes []string
 	// The allowed models
 	Models []ModelForSealing
-	// The TPM policy profile. May be be nil when using hooks.
-	TpmPCRProfile []byte
+	// Function that returns TPM policy profile. It is only called when resealing TPM keys.
+	GetTpmPCRProfile func() ([]byte, error)
 	// Whether a incremented value of the counter is allowed
 	// (before a revocation)
 	NewPCRPolicyVersion bool
