@@ -224,7 +224,7 @@ var setAuthorizedBootModesOnHooksKeydata = setAuthorizedBootModesOnHooksKeydataI
 
 // resealKeysWithFDESetupHookImpl updates hook based keydatas for given
 // files with a specific list of models
-func resealKeysWithFDESetupHookImpl(keys []KeyDataLocation, primaryKeyDevices []string, fallbackPrimaryKeyFiles []string, models []ModelForSealing, bootModes []string) error {
+func resealKeysWithFDESetupHookImpl(keys []KeyDataLocation, primaryKeyDevices []string, fallbackPrimaryKeyFiles []string, verifyPrimaryKey func([]byte), models []ModelForSealing, bootModes []string) error {
 	var sbModels []sb.SnapModel
 	for _, model := range models {
 		sbModels = append(sbModels, model)
@@ -264,6 +264,7 @@ func resealKeysWithFDESetupHookImpl(keys []KeyDataLocation, primaryKeyDevices []
 			if err != nil {
 				return err
 			}
+			verifyPrimaryKey(pk)
 			primaryKey = pk
 		}
 		if keyData.Generation() == 1 {
