@@ -42,8 +42,24 @@ func (s *errorsSuite) TestNotSnapErrorWithDetails(c *C) {
 	c.Check(err, ErrorMatches, `cannot process snap or snapdir: cannot open "some path"`)
 }
 
+func (s *errorsSuite) TestComponentNotInstalledErrorDetails(c *C) {
+	err := snap.ComponentNotInstalledError{
+		NotInstalledError: snap.NotInstalledError{Snap: "mysnap", Rev: snap.R(1)},
+		Component:         "mycomp",
+		CompRev:           snap.R(7),
+	}
+	c.Check(err, ErrorMatches,
+		`revision 7 of component "mycomp" is not installed for revision 1 of snap "mysnap"`)
+
+	err = snap.ComponentNotInstalledError{
+		NotInstalledError: snap.NotInstalledError{Snap: "mysnap", Rev: snap.R(1)},
+		Component:         "mycomp",
+	}
+	c.Check(err, ErrorMatches, `component "mycomp" is not installed for revision 1 of snap "mysnap"`)
+}
+
 func (s *errorsSuite) TestNotInstalledErrorIs(c *C) {
-	err := snap.NotInstalledError{}
-	c.Check(err.Is(&snap.NotInstalledError{}), Equals, true)
-	c.Check(err.Is(errors.New("some error")), Equals, false)
+	err := &snap.NotInstalledError{Snap: "foo", Rev: snap.R(33)}
+	c.Check(errors.Is(err, &snap.NotInstalledError{}), Equals, true)
+	c.Check(errors.Is(errors.New("some error"), &snap.NotInstalledError{}), Equals, false)
 }
