@@ -198,11 +198,14 @@ func (u *unsquashfsStderrWriter) Err() error {
 	}
 }
 
+// Unpack unpacks the snap to the given directory.
+//
+// Extended attributes are not preserved. This affects capabilities granted to specific executables.
 func (s *Snap) Unpack(src, dstDir string) error {
 	usw := newUnsquashfsStderrWriter()
 
 	var output bytes.Buffer
-	cmd := exec.Command("unsquashfs", "-n", "-f", "-d", dstDir, s.path, src)
+	cmd := exec.Command("unsquashfs", "-no-xattrs", "-n", "-f", "-d", dstDir, s.path, src)
 	cmd.Stderr = io.MultiWriter(&output, usw)
 	if err := cmd.Run(); err != nil {
 		return fmt.Errorf("cannot extract %q to %q: %v", src, dstDir, osutil.OutputErr(output.Bytes(), err))
@@ -234,7 +237,7 @@ func (s *Snap) withUnpackedFile(filePath string, f func(p string) error) error {
 	defer os.RemoveAll(tmpdir)
 
 	unpackDir := filepath.Join(tmpdir, "unpack")
-	if output, err := exec.Command("unsquashfs", "-n", "-i", "-d", unpackDir, s.path, filePath).CombinedOutput(); err != nil {
+	if output, err := exec.Command("unsquashfs", "-no-xattrs", "-n", "-i", "-d", unpackDir, s.path, filePath).CombinedOutput(); err != nil {
 		return fmt.Errorf("cannot run unsquashfs: %v", osutil.OutputErr(output, err))
 	}
 
@@ -470,9 +473,7 @@ func (e *errPathsNotReadable) Error() string {
 
 	b.WriteString("cannot access the following locations in the snap source directory:\n")
 	for _, p := range e.paths {
-		fmt.Fprintf(&b, "- ")
-		fmt.Fprintf(&b, p)
-		fmt.Fprintf(&b, "\n")
+		fmt.Fprintf(&b, "- %s\n", p)
 	}
 	if len(e.paths) == maxErrPaths {
 		fmt.Fprintf(&b, "- too many errors, listing first %v entries\n", maxErrPaths)
@@ -591,7 +592,11 @@ func (s *Snap) Build(sourceDir string, opts *BuildOpts) error {
 		}
 	}
 	snapType := opts.SnapType
-	if snapType != "os" && snapType != "core" && snapType != "base" {
+	switch snapType {
+	case "os", "core", "base", "snapd":
+		// -xattrs is default, but let's be explicit about it
+		cmd.Args = append(cmd.Args, "-xattrs")
+	default:
 		cmd.Args = append(cmd.Args, "-all-root", "-no-xattrs")
 	}
 
