@@ -108,11 +108,15 @@ type SquashfsTestSuite struct {
 	testutil.BaseTest
 
 	oldStdout, oldStderr, outf *os.File
+
+	realMksquashfs string
 }
 
 var _ = Suite(&SquashfsTestSuite{})
 
 func (s *SquashfsTestSuite) SetUpTest(c *C) {
+	s.BaseTest.SetUpTest(c)
+
 	d := c.MkDir()
 	dirs.SetRootDir(d)
 	s.AddCleanup(func() { dirs.SetRootDir("") })
@@ -126,6 +130,10 @@ func (s *SquashfsTestSuite) SetUpTest(c *C) {
 	c.Assert(err, IsNil)
 	s.oldStdout, s.oldStderr = os.Stdout, os.Stderr
 	os.Stdout, os.Stderr = s.outf, s.outf
+
+	p, err := exec.LookPath("mksquashfs")
+	c.Assert(err, IsNil, Commentf("cannot find real mksquashfs in $PATH"))
+	s.realMksquashfs = p
 }
 
 func (s *SquashfsTestSuite) TearDownTest(c *C) {
@@ -137,6 +145,8 @@ func (s *SquashfsTestSuite) TearDownTest(c *C) {
 	outbuf, err := io.ReadAll(s.outf)
 	c.Assert(err, IsNil)
 	c.Check(string(outbuf), Equals, "")
+
+	s.BaseTest.TearDownTest(c)
 }
 
 func (s *SquashfsTestSuite) TestFileHasSquashfsHeader(c *C) {
@@ -760,7 +770,7 @@ func (s *SquashfsTestSuite) TestBuildSupportsMultipleExcludesWithOnlyOneWildcard
 		c.Check(cmd, Equals, "/usr/bin/mksquashfs")
 		return nil, errors.New("bzzt")
 	})()
-	mksq := testutil.MockCommand(c, "mksquashfs", `/usr/bin/mksquashfs "$@"`)
+	mksq := testutil.MockCommand(c, "mksquashfs", s.realMksquashfs+` "$@"`)
 	defer mksq.Restore()
 
 	fakeSourcedir := c.MkDir()
@@ -781,7 +791,7 @@ func (s *SquashfsTestSuite) TestBuildSupportsMultipleExcludesWithOnlyOneWildcard
 		// the usual:
 		"mksquashfs", ".", snapPath, "-noappend", "-comp", "xz", "-no-fragments", "-no-progress",
 		// the interesting bits:
-		"-wildcards", "-ef", "exclude1", "-ef", "exclude2", "-ef", "exclude3",
+		"-wildcards", "-ef", "exclude1", "-ef", "exclude2", "-ef", "exclude3", "-xattrs",
 	})
 }
 
@@ -790,7 +800,7 @@ func (s *SquashfsTestSuite) TestBuildUsesMksquashfsFromCoreIfAvailable(c *C) {
 	defer squashfs.MockCommandFromSystemSnap(func(cmd string, args ...string) (*exec.Cmd, error) {
 		usedFromCore = true
 		c.Check(cmd, Equals, "/usr/bin/mksquashfs")
-		fakeCmd := exec.Cmd{Path: "/usr/bin/mksquashfs", Args: []string{"/usr/bin/mksquashfs"}}
+		fakeCmd := exec.Cmd{Path: s.realMksquashfs, Args: []string{s.realMksquashfs}}
 		return &fakeCmd, nil
 	})()
 	mksq := testutil.MockCommand(c, "mksquashfs", "exit 1")
@@ -812,7 +822,7 @@ func (s *SquashfsTestSuite) TestBuildUsesMksquashfsFromClassicIfCoreUnavailable(
 		c.Check(cmd, Equals, "/usr/bin/mksquashfs")
 		return nil, errors.New("bzzt")
 	})()
-	mksq := testutil.MockCommand(c, "mksquashfs", `/usr/bin/mksquashfs "$@"`)
+	mksq := testutil.MockCommand(c, "mksquashfs", s.realMksquashfs+` "$@"`)
 	defer mksq.Restore()
 
 	buildDir := c.MkDir()
@@ -847,15 +857,16 @@ func (s *SquashfsTestSuite) TestBuildVariesArgsByType(c *C) {
 	defer squashfs.MockCommandFromSystemSnap(func(cmd string, args ...string) (*exec.Cmd, error) {
 		return nil, errors.New("bzzt")
 	})()
-	mksq := testutil.MockCommand(c, "mksquashfs", `/usr/bin/mksquashfs "$@"`)
+	mksq := testutil.MockCommand(c, "mksquashfs", s.realMksquashfs+` "$@"`)
 	defer mksq.Restore()
 
 	buildDir := c.MkDir()
 	filename := filepath.Join(c.MkDir(), "foo.snap")
 	snap := squashfs.New(filename)
 
-	permissiveTypeArgs := []string{".", filename, "-noappend", "-comp", "xz", "-no-fragments", "-no-progress"}
-	restrictedTypeArgs := append(permissiveTypeArgs, "-all-root", "-no-xattrs")
+	commonTypeArgs := []string{".", filename, "-noappend", "-comp", "xz", "-no-fragments", "-no-progress"}
+	permissiveTypeArgs := append(commonTypeArgs, "-xattrs")
+	restrictedTypeArgs := append(commonTypeArgs, "-all-root", "-no-xattrs")
 	tests := []struct {
 		snapType string
 		args     []string
@@ -864,7 +875,7 @@ func (s *SquashfsTestSuite) TestBuildVariesArgsByType(c *C) {
 		{"app", restrictedTypeArgs},
 		{"gadget", restrictedTypeArgs},
 		{"kernel", restrictedTypeArgs},
-		{"snapd", restrictedTypeArgs},
+		{"snapd", permissiveTypeArgs},
 		{"base", permissiveTypeArgs},
 		{"os", permissiveTypeArgs},
 		{"core", permissiveTypeArgs},
