@@ -34,11 +34,11 @@ import (
 	"strings"
 	"sync"
 	"sync/atomic"
-	"text/template"
 	"time"
 
 	"github.com/snapcore/snapd/dirs"
 	"github.com/snapcore/snapd/gadget/quantity"
+	"github.com/snapcore/snapd/logger"
 	"github.com/snapcore/snapd/osutil"
 	"github.com/snapcore/snapd/osutil/squashfs"
 	"github.com/snapcore/snapd/sandbox/selinux"
@@ -229,6 +229,12 @@ func getVersion() (int, error) {
 		}
 	}
 
+	// ignore the pre-release suffixes, since we otherwise we can't convert to an
+	// int and we only compare versions coarsely to check systemd is not too old
+	if i := strings.IndexRune(verstr, '~'); i != -1 {
+		verstr = verstr[:i]
+	}
+
 	ver, err := strconv.Atoi(verstr)
 	if err != nil {
 		return 0, fmt.Errorf("cannot convert systemd version to number: %s", verstr)
@@ -323,9 +329,14 @@ type MountUnitOptions struct {
 	Fstype      string
 	Options     []string
 	Origin      string
+	// RootDir is the root of the filesystem where the unit will be created
+	RootDir string
 	// PreventRestartIfModified is set if we do not want to restart the
 	// mount unit if modified
 	PreventRestartIfModified bool
+	// EnsureStartIfUnchanged is set if we want to make sure that the unit
+	// is started even if it was not modified.
+	EnsureStartIfUnchanged bool
 }
 
 // Backend identifies the implementation backend in use by a Systemd instance.
@@ -340,12 +351,12 @@ const (
 	EmulationModeBackend
 )
 
-type mountUpdateStatus int
+type MountUpdateStatus int
 
 const (
-	mountUnchanged mountUpdateStatus = iota
-	mountUpdated
-	mountCreated
+	MountUnchanged MountUpdateStatus = iota
+	MountUpdated
+	MountCreated
 )
 
 // EnsureMountUnitFlags contains flags that modify behavior of EnsureMountUnitFile
@@ -440,6 +451,8 @@ type Systemd interface {
 	CurrentTasksCount(unit string) (uint64, error)
 	// Run a command
 	Run(command []string, opts *RunOptions) ([]byte, error)
+	// Set log level for the system
+	SetLogLevel(logLevel string) error
 }
 
 // KeyringMode describes how the kernel keyring is setup, see systemd.exec(5)
@@ -1053,9 +1066,9 @@ func (s *systemd) Stop(serviceNames []string) error {
 	// systemctl mocking in some modules are implemented very simplistically, and
 	// assumes that the 'stop' argument comes before the 'show'.
 	errorRet := make(chan error)
-	quit := make(chan interface{})
+	quit := make(chan any)
 
-	go func() {
+	go func(serviceNames []string) {
 		// The polling routine is the 'errorRet' channel sender, so we make
 		// sure we exit closing the channel explicitly, even though we always
 		// return the error result first. Closing the channel does not free the
@@ -1126,7 +1139,7 @@ func (s *systemd) Stop(serviceNames []string) error {
 				notifyShowFirst = false
 			}
 		}
-	}()
+	}(serviceNames)
 
 	// This command blocks until the 'systemctl stop' completes
 	_, errStop := s.systemctl(append([]string{"stop"}, serviceNames...)...)
@@ -1366,16 +1379,20 @@ func MountUnitPath(baseDir string) string {
 	return filepath.Join(dirs.SnapServicesDir, escapedPath+".mount")
 }
 
-// MountUnitPathWithLifetime returns the path of a {,auto}mount unit
-// created in the systemd directory suitable for the given unit lifetime
-func MountUnitPathWithLifetime(lifetime UnitLifetime, mountPointDir string) string {
+// mountUnitPathWithLifetime returns the path of a {,auto}mount unit created in
+// the systemd directory suitable for the given unit lifetime. rootDir is the
+// directory for the root filesystem.
+func mountUnitPathWithLifetime(lifetime UnitLifetime, mountPointDir, rootDir string) string {
+	if rootDir == "" {
+		rootDir = dirs.GlobalRootDir
+	}
 	escapedPath := EscapeUnitNamePath(mountPointDir)
 	var servicesPath string
 	switch lifetime {
 	case Persistent:
-		servicesPath = dirs.SnapServicesDir
+		servicesPath = dirs.SnapServicesDirUnder(rootDir)
 	case Transient:
-		servicesPath = dirs.SnapRuntimeServicesDir
+		servicesPath = dirs.SnapRuntimeServicesDirUnder(rootDir)
 	default:
 		panic(fmt.Sprintf("unknown systemd unit lifetime %q", lifetime))
 	}
@@ -1386,7 +1403,7 @@ func MountUnitPathWithLifetime(lifetime UnitLifetime, mountPointDir string) stri
 func ExistingMountUnitPath(mountPointDir string) string {
 	lifetimes := []UnitLifetime{Persistent, Transient}
 	for _, lifetime := range lifetimes {
-		unit := MountUnitPathWithLifetime(lifetime, mountPointDir)
+		unit := mountUnitPathWithLifetime(lifetime, mountPointDir, "")
 		if osutil.FileExists(unit) {
 			return unit
 		}
@@ -1396,74 +1413,70 @@ func ExistingMountUnitPath(mountPointDir string) string {
 
 var squashfsFsType = squashfs.FsType
 
-// Note that WantedBy=multi-user.target and Before=local-fs.target are
-// only used to allow downgrading to an older version of snapd.
-//
-// We want (see isBeforeDrivers) some snaps and components to be mounted before
-// modules are loaded (that is before systemd-{udevd,modules-load}).
-const snapMountUnitTmpl = `[Unit]
-Description={{.Description}}
+const (
+	snappyOriginModule = "X-SnapdOrigin"
+)
+
+func assembleMountUnitContent(u *MountUnitOptions) string {
+	before := "Before=snapd.mounts.target"
+	if u.MountUnitType == BeforeDriversLoadMountUnit {
+		// We want some snaps and components to be mounted before
+		// modules are loaded (that is before systemd-{udevd,modules-load}).
+		before += "\nBefore=systemd-udevd.service systemd-modules-load.service\nBefore=usr-lib-modules.mount usr-lib-firmware.mount"
+	}
+	origin := ""
+	if u.Origin != "" {
+		origin = fmt.Sprintf("%s=%s\n", snappyOriginModule, u.Origin)
+	}
+	// Note that WantedBy=multi-user.target is only used to allow
+	// downgrading to an older version of snapd.
+	unitContent := fmt.Sprintf(`[Unit]
+Description=%[1]s
 After=snapd.mounts-pre.target
-Before=snapd.mounts.target{{if isBeforeDrivers .MountUnitType}}
-Before=systemd-udevd.service systemd-modules-load.service{{end}}
+%[2]s
 
 [Mount]
-What={{.What}}
-Where={{.Where}}
-Type={{.Fstype}}
-Options={{join .Options ","}}
+What=%[3]s
+Where=%[4]s
+Type=%[5]s
+Options=%[6]s
 LazyUnmount=yes
 
 [Install]
 WantedBy=snapd.mounts.target
 WantedBy=multi-user.target
-{{- with .Origin}}
-X-SnapdOrigin={{.}}
-{{- end}}
-`
-
-func isBeforeDriversLoadMountUnit(mType MountUnitType) bool {
-	return mType == BeforeDriversLoadMountUnit
+%[7]s`, u.Description, before, u.What, u.Where, u.Fstype, strings.Join(u.Options, ","), origin)
+	return unitContent
 }
 
-var templateFuncs = template.FuncMap{"join": strings.Join,
-	"isBeforeDrivers": isBeforeDriversLoadMountUnit}
-var parsedMountUnitTmpl = template.Must(template.New("unit").Funcs(templateFuncs).Parse(snapMountUnitTmpl))
-
-const (
-	snappyOriginModule = "X-SnapdOrigin"
-)
-
-func ensureMountUnitFile(u *MountUnitOptions) (mountUnitName string, modified mountUpdateStatus, err error) {
+// EnsureMountUnitFileContent creates a mount unit file.
+func EnsureMountUnitFileContent(u *MountUnitOptions) (mountUnitName string, modified MountUpdateStatus, err error) {
 	if u == nil {
-		return "", mountUnchanged, errors.New("ensureMountUnitFile() expects valid mount options")
+		return "", MountUnchanged, errors.New("ensureMountUnitFile() expects valid mount options")
 	}
 
-	mu := MountUnitPathWithLifetime(u.Lifetime, u.Where)
-	var unitContent bytes.Buffer
-	if err := parsedMountUnitTmpl.Execute(&unitContent, &u); err != nil {
-		return "", mountUnchanged, fmt.Errorf("cannot generate mount unit: %v", err)
-	}
+	mu := mountUnitPathWithLifetime(u.Lifetime, u.Where, u.RootDir)
 
 	if osutil.FileExists(mu) {
-		modified = mountUpdated
+		modified = MountUpdated
 	} else {
-		modified = mountCreated
+		modified = MountCreated
 	}
 
 	if err := os.MkdirAll(filepath.Dir(mu), 0755); err != nil {
-		return "", mountUnchanged, fmt.Errorf("cannot create directory %s: %v", filepath.Dir(mu), err)
+		return "", MountUnchanged, fmt.Errorf("cannot create directory %s: %v", filepath.Dir(mu), err)
 	}
 
+	unitContent := assembleMountUnitContent(u)
 	stateErr := osutil.EnsureFileState(mu, &osutil.MemoryFileState{
-		Content: unitContent.Bytes(),
+		Content: []byte(unitContent),
 		Mode:    0644,
 	})
 
 	if stateErr == osutil.ErrSameState {
-		modified = mountUnchanged
+		modified = MountUnchanged
 	} else if stateErr != nil {
-		return "", mountUnchanged, stateErr
+		return "", MountUnchanged, stateErr
 	}
 
 	return filepath.Base(mu), modified, nil
@@ -1481,10 +1494,10 @@ func fsMountOptions(fstype string) []string {
 	return options
 }
 
-// hostFsTypeAndMountOptions returns filesystem type and options to actually
+// HostFsTypeAndMountOptions returns filesystem type and options to actually
 // mount the given fstype at runtime, i.e. it determines if fuse should be used
 // for squashfs.
-func hostFsTypeAndMountOptions(fstype string) (hostFsType string, options []string) {
+func HostFsTypeAndMountOptions(fstype string) (hostFsType string, options []string) {
 	options = fsMountOptions(fstype)
 	hostFsType = fstype
 	if fstype == "squashfs" {
@@ -1496,7 +1509,7 @@ func hostFsTypeAndMountOptions(fstype string) (hostFsType string, options []stri
 }
 
 func (s *systemd) EnsureMountUnitFile(description, what, where, fstype string, flags EnsureMountUnitFlags) (string, error) {
-	hostFsType, options := hostFsTypeAndMountOptions(fstype)
+	hostFsType, options := HostFsTypeAndMountOptions(fstype)
 	if osutil.IsDirectory(what) {
 		options = append(options, "bind")
 		hostFsType = "none"
@@ -1520,29 +1533,38 @@ func (s *systemd) EnsureMountUnitFileWithOptions(unitOptions *MountUnitOptions)
 	daemonReloadLock.Lock()
 	defer daemonReloadLock.Unlock()
 
-	mountUnitName, modified, err := ensureMountUnitFile(unitOptions)
+	mountUnitName, modified, err := EnsureMountUnitFileContent(unitOptions)
 	if err != nil {
 		return "", err
 	}
-	if modified != mountUnchanged {
+	units := []string{mountUnitName}
+	if modified != MountUnchanged {
 		// we need to do a daemon-reload here to ensure that systemd really
 		// knows about this new mount unit file
 		if err := s.daemonReloadNoLock(); err != nil {
 			return "", err
 		}
 
-		units := []string{mountUnitName}
 		if err := s.EnableNoReload(units); err != nil {
 			return "", err
 		}
 
 		// If just modified, some times it is not convenient to restart
-		if modified != mountUpdated || !unitOptions.PreventRestartIfModified {
+		if modified != MountUpdated || !unitOptions.PreventRestartIfModified {
 			// Start/restart the created or modified unit now
 			if err := s.RestartNoWaitForStop(units); err != nil {
 				return "", err
 			}
 		}
+	} else if unitOptions.EnsureStartIfUnchanged {
+		// Make sure the unit is actually started. This is useful for
+		// removable units, in case the device was unplugged and then
+		// replugged and "snapctl mount" is called again. Note that we
+		// avoid calling restart as we do not want to stop the unit in
+		// case it was already active.
+		if err := s.StartNoBlock(units); err != nil {
+			return "", err
+		}
 	}
 
 	return mountUnitName, nil
@@ -1745,3 +1767,20 @@ func (s *systemd) Run(command []string, opts *RunOptions) ([]byte, error) {
 	}
 	return stdout, nil
 }
+
+func (s *systemd) SetLogLevel(logLevel string) error {
+	_, err := s.systemctl("log-level", logLevel)
+
+	// Older systemd versions used systemd-analyze instead, try that if error
+	if err != nil {
+		if stdout, stderr, err2 := osutil.RunSplitOutput(
+			"systemd-analyze", "set-log-level", logLevel); err2 == nil {
+			return nil
+		} else {
+			logger.Noticef("while running systemd-analyze: %v",
+				osutil.OutputErrCombine(stdout, stderr, err2))
+		}
+	}
+
+	return err
+}
