@@ -75,7 +75,7 @@ distro_name_package() {
             arch_name_package "$1"
             ;;
         *)
-            echo "ERROR: Unsupported distribution $SPREAD_SYSTEM"
+            echo "ERROR: Unsupported distribution $SPREAD_SYSTEM" >&2
             exit 1
             ;;
     esac
@@ -121,7 +121,7 @@ distro_install_local_package() {
             pacman -U --noconfirm "$@"
             ;;
         *)
-            echo "ERROR: Unsupported distribution $SPREAD_SYSTEM"
+            echo "ERROR: Unsupported distribution $SPREAD_SYSTEM" >&2
             exit 1
             ;;
     esac
@@ -223,7 +223,7 @@ distro_install_package() {
             retval=$?
             ;;
         *)
-            echo "ERROR: Unsupported distribution $SPREAD_SYSTEM"
+            echo "ERROR: Unsupported distribution $SPREAD_SYSTEM" >&2
             exit 1
             ;;
     esac
@@ -268,7 +268,7 @@ distro_purge_package() {
             pacman -Rnsc --noconfirm "$@"
             ;;
         *)
-            echo "ERROR: Unsupported distribution $SPREAD_SYSTEM"
+            echo "ERROR: Unsupported distribution $SPREAD_SYSTEM" >&2
             exit 1
             ;;
     esac
@@ -294,7 +294,7 @@ distro_update_package_db() {
             pacman -Syq
             ;;
         *)
-            echo "ERROR: Unsupported distribution $SPREAD_SYSTEM"
+            echo "ERROR: Unsupported distribution $SPREAD_SYSTEM" >&2
             exit 1
             ;;
     esac
@@ -318,7 +318,7 @@ distro_clean_package_cache() {
             pacman -Sccq --noconfirm
             ;;
         *)
-            echo "ERROR: Unsupported distribution $SPREAD_SYSTEM"
+            echo "ERROR: Unsupported distribution $SPREAD_SYSTEM" >&2
             exit 1
             ;;
     esac
@@ -340,7 +340,7 @@ distro_auto_remove_packages() {
         arch-*)
             ;;
         *)
-            echo "ERROR: Unsupported distribution '$SPREAD_SYSTEM'"
+            echo "ERROR: Unsupported distribution $SPREAD_SYSTEM" >&2
             exit 1
             ;;
     esac
@@ -364,7 +364,7 @@ distro_query_package_info() {
             pacman -Si "$1"
             ;;
         *)
-            echo "ERROR: Unsupported distribution '$SPREAD_SYSTEM'"
+            echo "ERROR: Unsupported distribution $SPREAD_SYSTEM" >&2
             exit 1
             ;;
     esac
@@ -489,8 +489,10 @@ distro_install_build_snapd(){
         if ! systemctl is-enabled snapd.socket ; then
             # Can't use --now here as not all distributions we run on support it
             systemctl enable snapd.socket
-            systemctl restart snapd.socket
         fi
+        # # If this is a re-installation (like in snap-mgmt), snapd.socket and snapd.service might be stopped
+        systemctl restart snapd.socket snapd.service
+        snap wait system seed.loaded
     fi
 }
 
@@ -693,7 +695,7 @@ pkg_linux_image_extra (){
         if apt-cache show "linux-modules-extra-$(uname -r)" > /dev/null 2>&1; then
             echo "linux-modules-extra-$(uname -r)";
         else
-            echo "cannot find a matching kernel modules package";
+            echo "cannot find a matching kernel modules package" >&2;
             exit 1;
         fi;
     fi
@@ -966,7 +968,7 @@ distro_upgrade() {
             fi
             ;;
         *)
-            echo "WARNING: distro upgrade not supported on $SPREAD_SYSTEM"
+            echo "WARNING: distro upgrade not supported on $SPREAD_SYSTEM" >&2
             ;;
     esac
 }
