@@ -70,6 +70,28 @@ create_test_user(){
 }
 
 build_deb(){
+    # debian-sid packaging is special
+    if os.query is-debian sid; then
+        if [ ! -d packaging/debian-sid ]; then
+            echo "no packaging/debian-sid/ directory "
+            echo "broken test setup"
+            exit 1
+        fi
+
+        # remove etckeeper
+        apt purge -y etckeeper
+
+        # debian has its own packaging
+        rm -f debian
+        # the debian dir must be a real dir, a symlink will make
+        # dpkg-buildpackage choke later.
+        mv packaging/debian-sid debian
+
+        apt build-dep -y ./
+
+        # ensure we really build without vendored packages
+        mv ./vendor /tmp
+    fi
     newver="$(dpkg-parsechangelog --show-field Version)"
 
     case "$SPREAD_SYSTEM" in
@@ -81,11 +103,6 @@ build_deb(){
     # Use fake version to ensure we are always bigger than anything else
     dch --newversion "1337.$newver" "testing build"
 
-    if os.query is-debian sid; then
-        # ensure we really build without vendored packages
-        mv ./vendor /tmp
-    fi
-
     unshare -n -- \
             su -l -c "cd $PWD && DEB_BUILD_OPTIONS='nocheck testkeys ${FIPS_BUILD_OPTION}' dpkg-buildpackage -tc -b -Zgzip -uc -us" test
     # put our debs to a safe place
@@ -162,37 +179,17 @@ build_arch_pkg() {
     base_version="$(head -1 debian/changelog | awk -F '[()]' '{print $2}')"
     version="1337.$base_version"
     packaging_path=packaging/arch
-    archive_name=snapd-$version.tar
 
     rm -rf /tmp/pkg
-    mkdir -p /tmp/pkg/sources/snapd
-    cp -ra -- * /tmp/pkg/sources/snapd/
+    mkdir -p /tmp/pkg/
+    cp -av "$packaging_path"/* /tmp/pkg
 
     # shellcheck disable=SC2086
-    tar -C /tmp/pkg/sources -cf "/tmp/pkg/$archive_name" "snapd"
-    cp "$packaging_path"/* "/tmp/pkg"
+    ./packaging/pack-source -v "$version" -o "/tmp/pkg/" -s
 
-    # fixup PKGBUILD which builds a package named snapd-git with dynamic version
-    #  - update pkgname to use snapd
-    #  - kill dynamic version
-    #  - packaging functions are named package_<pkgname>(), update it to package_snapd()
-    #  - update source path to point to local archive instead of git
-    #  - fix package version to $version
     sed -i \
-        -e "s/^source=.*/source=(\"$archive_name\")/" \
-        -e "s/pkgname=snapd.*/pkgname=snapd/" \
         -e "s/pkgver=.*/pkgver=$version/" \
-        -e "s/package_snapd-git()/package_snapd()/" \
         /tmp/pkg/PKGBUILD
-    # comment out automatic package version update block `pkgver() { ... }` as
-    # it's only useful when building the package manually
-    awk '
-    /BEGIN/ { strip = 0; last = 0 }
-    /pkgver\(\)/ { strip = 1 }
-    /^}/ { if (strip) last = 1 }
-    // { if (strip) { print "#" $0; if (last) { last = 0; strip = 0}} else { print $0}}
-    ' < /tmp/pkg/PKGBUILD > /tmp/pkg/PKGBUILD.tmp
-    mv /tmp/pkg/PKGBUILD.tmp /tmp/pkg/PKGBUILD
 
     chown -R test:test /tmp/pkg
     unshare -n -- \
@@ -238,20 +235,29 @@ install_dependencies_gce_bucket(){
 ###
 
 prepare_project() {
-    # we install chrony on xenial (as its also used in later distros), so the
-    # ntp service was in conflict, degrading the systemd unit and sometimes breaking
-    # NTP syncs
-    if os.query is-xenial; then
-      systemctl stop ntp.service
-      systemctl disable ntp.service
-      apt-get remove --purge -y ntp
-      systemctl reset-failed
-    fi
-
     if os.query is-ubuntu && os.query is-classic; then
         apt-get remove --purge -y lxd lxcfs || true
         apt-get autoremove --purge -y
         "$TESTSTOOLS"/lxd-state undo-mount-changes
+
+        if [ -n "$UPDATE_UBUNTU_KERNEL_PATTERN" ] && [ "$SPREAD_REBOOT" = 0 ]; then
+            KERNEL_VER="$(apt-cache search "^linux-headers-${UPDATE_UBUNTU_KERNEL_PATTERN}$" | tail -n1 | awk '{ print $1 }' | sed 's/^linux-headers-//')"
+            if [ -z "$KERNEL_VER" ]; then
+                echo "Kernel version not found using pattern: $UPDATE_UBUNTU_KERNEL_PATTERN"
+                exit 1
+            fi
+
+            # Install the kernel version found
+            apt-get update
+            apt-get install -y linux-image-"$KERNEL_VER" linux-headers-"$KERNEL_VER"
+
+            # Update grub to set this kernel as default
+            echo "[*] Updating GRUB to set $KERNEL_VER as default..."
+            grub-set-default "Advanced options for Ubuntu>Ubuntu, with Linux $KERNEL_VER"
+            update-grub
+
+            REBOOT
+        fi
     fi
 
     # Check if running inside a container.
@@ -366,36 +372,6 @@ prepare_project() {
         fi
     fi
 
-    # debian-sid packaging is special
-    if os.query is-debian sid; then
-        if [ ! -d packaging/debian-sid ]; then
-            echo "no packaging/debian-sid/ directory "
-            echo "broken test setup"
-            exit 1
-        fi
-
-        # remove etckeeper
-        apt purge -y etckeeper
-
-        # debian has its own packaging
-        rm -f debian
-        # the debian dir must be a real dir, a symlink will make
-        # dpkg-buildpackage choke later.
-        mv packaging/debian-sid debian
-
-        # get the build-deps
-        apt build-dep -y ./
-
-        # and ensure we don't take any of the vendor deps
-        rm -rf vendor/*/
-
-        # and create a fake upstream tarball
-        tar -c -z -f ../snapd_"$(dpkg-parsechangelog --show-field Version|cut -d- -f1)".orig.tar.gz --exclude=./debian --exclude=./.git --exclude='*.pyc' .
-
-        # and build a source package - this will be used during the sbuild test
-        dpkg-buildpackage -S -uc -us
-    fi
-
     # so is ubuntu-14.04
     if os.query is-trusty; then
         if [ ! -d packaging/ubuntu-14.04 ]; then
@@ -558,10 +534,10 @@ prepare_project() {
                     # we are limited by the FIPS variants of go toolchain
                     # available from the PPA, and we need to match the Go
                     # version expected by during FIPS build of the deb, which
-                    # currently expects 1.21, see:
+                    # currently expects 1.23, see:
                     # https://launchpad.net/~ubuntu-toolchain-r/+archive/ubuntu/golang-fips
-                    best_golang=golang-1.21
-                    quiet apt install -y golang-1.21
+                    best_golang=golang-1.23
+                    quiet apt install -y golang-1.23
                     ;;
             esac
             # in 16.04: "apt build-dep -y ./" would also work but not on 14.04
@@ -577,21 +553,44 @@ prepare_project() {
     esac
 
     # Retry go mod vendor to minimize the number of connection errors during the sync
+    # It is required in any case because the testing tools like the fakestore are always compiled
     retry -n 10 go mod vendor
-    # Update C dependencies
-    ( cd c-vendor && retry -n 10 ./vendor.sh )
-
-    # go mod runs as root and will leave strange permissions
-    chown test:test -R "$SPREAD_PATH"
 
     # We are testing snapd snap on top of snapd from the archive
     # of the tested distribution. Download snapd and snap-confine
     # as they exist in the archive for further use.
     if tests.info is-snapd-from-archive; then
-        ( cd "${GOHOME}" && tests.pkgs download snapd snap-confine)
+        case "$SPREAD_SYSTEM" in
+            debian-*)
+                # In Debian 14+, the snap-confine transitional package was removed.
+                # In earlier versions it was just an empty package so it's not worth pulling.
+                ( cd "${GOHOME}" && tests.pkgs download snapd )
+                ;;
+            *)
+                ( cd "${GOHOME}" && tests.pkgs download snapd snap-confine)
+                ;;
+        esac
+    elif [ "$USE_PREBUILT_PACKAGES" = "true" ]; then
+        find "$PROJECT_PATH/built-pkgs/$SPREAD_SYSTEM" -type f -exec cp -v {} "${GOHOME}" \;
+        case "$SPREAD_SYSTEM" in
+            ubuntu-*)
+                # set the version to ensure core-initrd/build-source-pkgs.sh doesn't fail
+                newver="$(dpkg-parsechangelog --show-field Version)"
+                dch --newversion "1337.$newver" "testing build"
+                ;;
+        esac
     else
+        # Update C dependencies
+        ( cd c-vendor && retry -n 10 ./vendor.sh )
+
+        # go mod runs as root and will leave strange permissions
+        chown test:test -R "$SPREAD_PATH"
         case "$SPREAD_SYSTEM" in
             ubuntu-*|debian-*)
+                # in 16.04: "apt build-dep -y ./" would also work but not on 14.04
+                gdebi --quiet --apt-line ./debian/control >deps.txt
+                quiet xargs -r eatmydata apt-get install -y < deps.txt
+                
                 build_deb
                 ;;
             fedora-*|opensuse-*|amazon-*|centos-*)
@@ -619,6 +618,7 @@ prepare_project() {
     # Build additional utilities we need for testing
     go install ./tests/lib/fakedevicesvc
     go install ./tests/lib/systemd-escape
+    go install ./tests/lib/plz-run
 
     # Build the tool for signing model assertions
     go install ./tests/lib/gendeveloper1
@@ -646,6 +646,7 @@ prepare_project_each() {
 prepare_suite() {
     # shellcheck source=tests/lib/prepare.sh
     . "$TESTSLIB"/prepare.sh
+
     # os.query cannot be used because first time the suite is prepared, the current system
     # is classic ubuntu, so it is needed to check the system set in $SPREAD_SYSTEM
     if is_test_target_core; then
@@ -654,6 +655,10 @@ prepare_suite() {
         prepare_classic
     fi
 
+    if [ -n "$TAG_FEATURES" ]; then
+        snap set system journal.persistent=true
+    fi
+
     # Make sure the suite starts with a clean environment and with the snapd state restored
     # shellcheck source=tests/lib/reset.sh
     "$TESTSLIB"/reset.sh --reuse-core
@@ -676,16 +681,6 @@ prepare_suite_each() {
 
     # Save all the installed packages
     if os.query is-classic; then
-        # lxd-installer is in cloud images starting from 24.04. This package
-        # installs lxd when any lxc command is run. This caused problems
-        # because if we install snapcraft & lxd, in the restore step lxd is
-        # removed, and after that snapcraft is removed. However, snapcraft's
-        # remove hook calls lxd and triggers a new installation of lxd, and in
-        # turn when we try to remove core22, it fails as lxd has been
-        # re-installed and depends on that base. Therefore, we remove it to
-        # prevent these issues, and we do that before we get the list of
-        # installed packages to make sure we do not re-install it again.
-        apt remove -y --purge lxd-installer || true
         tests.pkgs list-installed > installed-initial.pkgs
     fi
 
@@ -696,11 +691,13 @@ prepare_suite_each() {
     echo -n "${SPREAD_JOB:-} " >> "$RUNTIME_STATE_PATH/runs"
 
     # Restart journal log and reset systemd journal cursor.
-    systemctl reset-failed systemd-journald.service
-    if ! systemctl restart systemd-journald.service; then
-        systemctl status systemd-journald.service || true
-        echo "Failed to restart systemd-journald.service, exiting..."
-        exit 1
+    if systemctl is-failed systemd-journald.service; then
+        systemctl reset-failed systemd-journald.service
+        if ! systemctl restart systemd-journald.service; then
+            systemctl status systemd-journald.service || true
+            echo "Failed to restart systemd-journald.service, exiting..."
+            exit 1
+        fi
     fi
     "$TESTSTOOLS"/journal-state start-new-log
 
@@ -739,6 +736,10 @@ prepare_suite_each() {
 }
 
 restore_suite_each() {
+    if not tests.nested is-nested; then
+        "$TESTSLIB"/collect-artifacts.sh features --after-non-nested-task
+        "$TESTSLIB"/collect-artifacts.sh locks
+    fi
     local variant="$1"
 
     rm -f "$RUNTIME_STATE_PATH/audit-stamp"
@@ -903,7 +904,7 @@ restore_project_each() {
 
     # TODO: move this to tests.invariant.
     case "$SPREAD_SYSTEM" in
-        fedora-*|centos-*)
+        fedora-*|centos-*|opensuse-*-selinux-*)
             # Make sure that we are not leaving behind incorrectly labeled snap
             # files on systems supporting SELinux
             (
@@ -914,6 +915,13 @@ restore_project_each() {
             find /var/snap -printf '%Z\t%H/%P\n' | grep -c -v snappy_var_t  | MATCH "0"
             ;;
     esac
+
+    case "$SPREAD_SYSTEM" in
+        ubuntu-core-*)
+            # TODO fishing for the test which broke the keys
+            stat --format %A /etc/ssh/ssh_host_rsa_key | MATCH -e '-rw-------'
+            ;;
+    esac
 }
 
 restore_project() {
