@@ -101,7 +101,7 @@ EOF
     # We change the service configuration so reload and restart
     # the units to get them applied
     systemctl daemon-reload
-    # restart the service (it pulls up the socket)
+    # restart the service (it pulls up the socket)    
     systemctl restart snapd.service
 }
 
@@ -478,7 +478,19 @@ prepare_classic() {
         rm -rf "$build_dir"
         mkdir -p "$build_dir"
         build_snapd_snap "$build_dir"
-        snap install --dangerous "$build_dir/"snapd_*.snap
+        case "$SPREAD_SYSTEM" in
+            ubuntu-fips-24.04-*)
+                # we're expecting snapd installation to fail due to SNAPDENG-35482
+                not snap install --dangerous "$build_dir/"snapd_*.snap
+                journalctl -u snapd | MATCH "opensslcrypto: can't enable FIPS mode for OpenSSL"
+                echo "this failure is expected"
+                exit 1
+                ;;
+            *)
+                # we're expecting snapd installation to fail due to SNAPDENG-
+                snap install --dangerous "$build_dir/"snapd_*.snap
+                ;;
+        esac
         snap wait system seed.loaded
     fi
     snap list snapd
@@ -641,11 +653,7 @@ build_snapd_snap() {
     for snap in "${snapd_snap_cache}"/snapd_*.snap; do
         if ! [ -f "${snap}" ]; then
             if [ "${USE_PREBUILT_SNAPD_SNAP}" = true ]; then
-                if [ -n "${USE_SNAPD_SNAP_URL}" ]; then
-                    wget -q "$USE_SNAPD_SNAP_URL" -O "${snapd_snap_cache}/snapd_from_ci.snap"
-                else
-                    cp "${PROJECT_PATH}/built-snap"/snapd_1337.*.snap.keep "${snapd_snap_cache}/snapd_from_ci.snap"
-                fi
+                cp "${PROJECT_PATH}/built-snap"/snapd_1337.*.snap.keep "${snapd_snap_cache}/snapd_from_ci.snap"
             else
                 # This is not reliable across classic releases so only allow on
                 # ARM variants as a special case since we cannot cross build
@@ -678,11 +686,7 @@ _get_snapd() {
     mkdir -p "${TARGET}"
 
     if [ "${USE_PREBUILT_SNAPD_SNAP}" = true ]; then
-        if [ -n "${USE_SNAPD_SNAP_URL}" ]; then
-            wget -q "$USE_SNAPD_SNAP_URL" -O "${TARGET}"/snapd_from_snapcraft.snap
-        else
-            cp "${PROJECT_PATH}/built-snap"/snapd_1337.*.snap.keep "${TARGET}/snapd_from_snapcraft.snap"
-        fi
+        cp "${PROJECT_PATH}/built-snap"/snapd_1337.*.snap.keep "${TARGET}/snapd_from_snapcraft.snap"
     else
         touch "${PROJECT_PATH}"/test-build
         chmod -R go+r "${PROJECT_PATH}/tests"
@@ -1311,6 +1315,9 @@ EOF
     # the writeable-path sync-boot won't work
     mkdir -p /mnt/system-data/etc/systemd
 
+    # make sure we use the same timesync configuration than in the host machine
+    cp /etc/systemd/timesyncd.conf /mnt/system-data/etc/systemd/timesyncd.conf
+
     mkdir -p /mnt/system-data/var/lib/console-conf
 
     # NOTE: The here-doc below must use tabs for proper operation.
@@ -1467,17 +1474,29 @@ EOF
         IMAGE_CHANNEL="$KERNEL_CHANNEL"
     else
         IMAGE_CHANNEL="$GADGET_CHANNEL"
-        if is_test_target_core_le 18; then
-            if is_test_target_core 16; then
-                BRANCH=latest
-            elif is_test_target_core 18; then
-                BRANCH=18
-            fi
+    fi
+
+    if is_test_target_core_le 18; then
+        if is_test_target_core 16; then
+            BRANCH=latest
+        else
+            BRANCH=18
+        fi
+
+        if is_test_target_core 18 && [[ "$SPREAD_BACKEND" =~ openstack ]]; then
+            # When running in openstack backend and uc18 it is required to use an specific
+            # kernel snap which is using the 5.4.0 instead of the 4.15 because this
+            # version has a fix for https://bugs.launchpad.net/qemu/+bug/1844053
+            wget -q -O pc-kernel.snap https://storage.googleapis.com/snapd-spread-tests/snaps/pc-kernel_5.4.0-221.241.1_amd64.snap
+        elif [ "$KERNEL_CHANNEL" != "$GADGET_CHANNEL" ]; then
             # download pc-kernel snap for the specified channel and set
             # ubuntu-image channel to that of the gadget, so that we don't
             # need to download it. Do this only for UC16/18 as the UC20+
             # case is considered a few lines below.
-            snap download --basename=pc-kernel --channel="$BRANCH/$KERNEL_CHANNEL" pc-kernel
+            snap download --basename=pc-kernel --channel="${BRANCH}/${KERNEL_CHANNEL}" pc-kernel
+        fi
+
+        if [ -f pc-kernel.snap ]; then
             # Repack to prevent reboots as the image channel (which will become
             # the tracked channel) is different to the kernel channel.
             unsquashfs -d pc-kernel pc-kernel.snap
@@ -1486,6 +1505,7 @@ EOF
             rm -rf pc-kernel
             mv pc-kernel-repacked.snap pc-kernel.snap
             EXTRA_FUNDAMENTAL="--snap $PWD/pc-kernel.snap"
+            chmod 0600 pc-kernel.snap
         fi
     fi
 
@@ -1511,7 +1531,7 @@ EOF
 
         # also add debug command line parameters to the kernel command line via
         # the gadget in case things go side ways and we need to debug
-        snap download --basename=pc --channel="${BRANCH}/${KERNEL_CHANNEL}" pc
+        snap download --basename=pc --channel="${BRANCH}/${GADGET_CHANNEL}" pc
         test -e pc.snap
         unsquashfs -d pc-gadget pc.snap
         # TODO: it would be desirable when we need to do in-depth debugging of
@@ -1833,6 +1853,14 @@ prepare_ubuntu_core() {
         setup_reflash_magic
         REBOOT
     fi
+
+    # Wait for the snap command to become available and snapd is active.
+    if [ "$SPREAD_BACKEND" != "external" ] && [ "$SPREAD_BACKEND" != "testflinger" ]; then
+        # shellcheck disable=SC2016
+        retry -n 120 --wait 1 sh -c 'test "$(command -v snap)" = /usr/bin/snap && snap version | grep -E -q "snapd +1337.*"'
+    fi
+    retry -n 10 --wait 1 sh -c 'systemctl is-active snapd snapd.socket'
+
     setup_snapd_proxy
 
     disable_journald_rate_limiting
@@ -1847,12 +1875,6 @@ prepare_ubuntu_core() {
         fi
     fi
 
-    # Wait for the snap command to become available.
-    if [ "$SPREAD_BACKEND" != "external" ] && [ "$SPREAD_BACKEND" != "testflinger" ]; then
-        # shellcheck disable=SC2016
-        retry -n 120 --wait 1 sh -c 'test "$(command -v snap)" = /usr/bin/snap && snap version | grep -E -q "snapd +1337.*"'
-    fi
-
     # Wait for seeding to finish.
     snap wait system seed.loaded
 
